<?php declare(strict_types=1);

namespace Plugin\agi_shopgate_connect\Backend;

use JTL\Alert\Alert;
use JTL\DB\ReturnType;
use JTL\Helpers\Form;
use JTL\Helpers\Request;
use JTL\Shop;
use stdClass;

/**
 * Class AdminApiEinrichtung
 * @package Plugin\agi_shopgate_connect\Backend
 */
class AdminApiEinrichtung extends CustomLink
{
  /**
   * @return bool
   */
  protected function controller(): bool
  {
    
    $this->getPlugin()->getConfig()->getValue('sgRedirectActive') === 'Y';
    define('SHOPGATE_LIBRARY_FOLDER', $this->getPlugin()->getPaths()->getBasePath().'shopgate_sdk/');
    define('SHOPGATE_CONFIG_FOLDER', SHOPGATE_LIBRARY_FOLDER.'config/');
    $aLanguages = array();
    $nDefaultLanguage = 1;
    $query = "
      SELECT kSprache, cNameDeutsch, cShopStandard
      FROM tsprache
      ORDER BY cNameDeutsch
    ";
    $aRes = $this->db->queryPrepared(
      $query,
      [],
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $row) {
      $aLanguages[$row -> kSprache] = array('kSprache'=>$row->kSprache, 'cName'=>$row->cNameDeutsch);
      if ($row->cShopStandard === 'Y') {
        $nDefaultLanguage = $row->kSprache;
      }
    }
    
    $shopgateConfigDefault = array (
      'customer_number'     => '',
      'shop_number'         => '',
      'apikey'              => '',
      'jtlSprache'          => $nDefaultLanguage,
      'plugin'              => 'AGIShopgateConnect',
      'shop_is_active'      => 1
    );
    
    $textFileIsNotWritable = 'ist <strong>nicht beschreibbar</strong><br />&Auml;nderungen k&ouml;nnen nicht gespeichert werden. Setzen Sie die Dateirechte auf 777 um vorgenommene &Auml;nderungen zu speichern.';
    $textFileIsWritable = '<strong>ist beschreibbar</strong>';
    
    $config_files = array();
    $files = scandir(SHOPGATE_CONFIG_FOLDER);
    foreach ($files as $file) {
      if (!is_file(SHOPGATE_CONFIG_FOLDER.$file)) {
        continue;
      }
      if (preg_match('/^config_([0-9]{4,})/', $file, $matches)) {
        $confFileIsWritable = is_writable(SHOPGATE_CONFIG_FOLDER.$file);
        $sfile_number = $matches[1];
        require SHOPGATE_CONFIG_FOLDER.$file;
        $config_files[$sfile_number] = array(
          'file'        => SHOPGATE_CONFIG_FOLDER.$file,
          'shopnumber'  => $sfile_number,
          'data'        => $shopgate_config,
          'writable'    => $confFileIsWritable
        );
      }
    }

    
    if (Form::validateToken() && ($posted = Request::postVar('api_action')) !== null) {
      $value = Request::postVar('inputSgRedirectActive') !== null ? 'Y' : 'N';
      $values = $this->getPlugin()->getConfig()->getOptions();
      $this->updatePluginOption($values, 'sgRedirectActive', $value);
      
      $allowed_chars = '/^[A-Za-z0-9_äöüÄÖÜß &;().,`´@\'\"\-\/]*$/';
      $sgLanguagesToShops = array();
      if (isset($_POST['sg_shop_number_ref'])) {
        foreach ($_POST['sg_shop_number_ref'] as $skey) {
          if (preg_match($allowed_chars, $skey)) {
            if (Request::postVar('sg_delete_'.$skey) !== null) {
              if (array_key_exists($skey, $config_files)) {
                @unlink($config_files[$skey]['file']);
                unset($config_files[$skey]);
              }
            }
            else {
              $cfg_shopnumber = Request::postVar('sg_shop_number_'.$skey);
              if (!empty($cfg_shopnumber) && preg_match($allowed_chars, $cfg_shopnumber)) {
                $tmp = trim($cfg_shopnumber);
                if (!empty($tmp)) {
                  $use_key = $skey;
                  if ($tmp != $skey) {
                    @unlink($config_files[$skey]['file']);
                    unset($config_files[$skey]);
                    $config_files[$tmp] = array(
                      'file'        => SHOPGATE_CONFIG_FOLDER.'config_'.$tmp.'.php',
                      'shopnumber'  => $tmp,
                      'data'        => $shopgateConfigDefault,
                      'writable'    => false
                    );
                    $use_key = $tmp;
                  }
                  $config_files[$tmp]['data']['shop_number'] = $tmp;
                  
                  $cfg_customer = Request::postVar('sg_customer_number_'.$skey);
                  if (isset($cfg_customer) && preg_match($allowed_chars, $cfg_customer)) {
                    $tmp = trim($cfg_customer);
                    if (!empty($tmp)) {
                      $config_files[$use_key]['data']['customer_number'] = $tmp;
                    }
                  }
                  
                  $cfg_apikey = Request::postVar('sg_apikey_'.$skey);
                  if (isset($cfg_apikey) && preg_match($allowed_chars, $cfg_apikey)) {
                    $tmp = trim($cfg_apikey);
                    if (!empty($tmp)) {
                      $config_files[$use_key]['data']['apikey'] = $tmp;
                    }
                  }
                  
                  $cfg_lang = Request::postVar('sg_lang_'.$skey);
                  if (isset($cfg_lang) && array_key_exists($cfg_lang, $aLanguages)) {
                    $tmp = trim($cfg_lang);
                    $config_files[$use_key]['data']['jtlSprache'] = !empty($tmp) ? $tmp : $nDefaultLanguage;
                  }
                }
              }
            }
          }
        }
      }
      
      $new_shopnumber = Request::postVar('sg_shop_number_new');
      if ($new_shopnumber !== null) {
        $tmp = trim($new_shopnumber);
        if (!empty($tmp)) {
          $use_key = $tmp;
          $config_files[$tmp] = array(
            'file'        => SHOPGATE_CONFIG_FOLDER.'config_'.$tmp.'.php',
            'shopnumber'  => $tmp,
            'data'        => $shopgateConfigDefault,
            'writable'    => false
          );
          $config_files[$tmp]['data']['shop_number'] = $tmp;
          
          $new_customer = Request::postVar('sg_customer_number_new');
          if ($new_customer !== null && preg_match($allowed_chars, $new_customer)) {
            $tmp = trim($new_customer);
            if (!empty($tmp)) {
              $config_files[$use_key]['data']['customer_number'] = $tmp;
            }
          }
          
          $new_apikey = Request::postVar('sg_apikey_new');
          if ($new_apikey !== null && preg_match($allowed_chars, $new_apikey)) {
            $tmp = trim($new_apikey);
            if (!empty($tmp)) {
              $config_files[$use_key]['data']['apikey'] = $tmp;
            }
          }
          
          $new_lang = Request::postVar('sg_lang_new');
          if ($new_lang !== null && array_key_exists($new_lang, $aLanguages)) {
            $tmp = trim($new_lang);
            $config_files[$use_key]['data']['jtlSprache'] = !empty($tmp) ? $tmp : $nDefaultLanguage;
          }
        }
      }
      
      foreach ($config_files as $fkey => $fdata) {
        $fp = @fopen($fdata['file'], 'w+');
        if ($fp) {
          $content  = '<?php'.chr(10);
          $content .= '$shopgate_config = array ('.chr(10);
          foreach ($fdata['data'] as $pkey => $pValue) {
            $content .= chr(9)."'$pkey' => '$pValue',".chr(10);
          }
          $content .= ');'.chr(10);
          $content .= '?>';
          fwrite($fp, $content);
          fclose($fp);
          $sgLanguagesToShops[$fdata['data']['jtlSprache']] = $fkey;
        }
        $config_files[$fkey]['writable'] = !file_exists($fdata['file']) ? false : is_writable($fdata['file']);
      }
      $values = $this->getPlugin()->getConfig()->getOptions();
      $this->updatePluginOption($values, 'sgLanguagesToShops', empty($sgLanguagesToShops) ? '' : serialize($sgLanguagesToShops));
    }
    
    Shop::Smarty()->assign('shopConfigArray', $config_files);
    Shop::Smarty()->assign('cTextFileIsWritable', $textFileIsWritable);
    Shop::Smarty()->assign('cTextFileIsNotWritable', $textFileIsNotWritable);
    Shop::Smarty()->assign('bConfDirIsWritable',is_writable(SHOPGATE_CONFIG_FOLDER));
    Shop::Smarty()->assign('cShopgateConfigDir', SHOPGATE_CONFIG_FOLDER);
    Shop::Smarty()->assign('aShopSprachen', $aLanguages);
    Shop::Smarty()->assign('aShopSpracheDefault', $nDefaultLanguage);
    Shop::Smarty()->assign('shop_new', $shopgateConfigDefault);

    Shop::Smarty()->assign('bTempDirIsWritable',is_writable(SHOPGATE_LIBRARY_FOLDER.'temp/'));
    Shop::Smarty()->assign('cShopgateTempDir', SHOPGATE_LIBRARY_FOLDER.'temp/');

    Shop::Smarty()->assign('bCacheDirIsWritable',is_writable(SHOPGATE_LIBRARY_FOLDER.'temp/cache/'));
    Shop::Smarty()->assign('cShopgateCacheDir', SHOPGATE_LIBRARY_FOLDER.'temp/cache/');

    Shop::Smarty()->assign('bLogDirIsWritable',is_writable(SHOPGATE_LIBRARY_FOLDER.'temp/logs/'));
    Shop::Smarty()->assign('cShopgateLogDir', SHOPGATE_LIBRARY_FOLDER.'temp/logs/');

    Shop::Smarty()->assign('valueSgRedirectActive', $this->getPlugin()->getConfig()->getValue('sgRedirectActive'));

    Shop::Smarty()->assign('SHOPGATE_PLUGIN_URL', Shop::getURL().'/plugins/agi_shopgate_connect/shopgate_request.php');

    return true;
  }
  
  /**
   * @return string
   */
  protected function getTemplate(): string
  {
    return 'templates/tab_api.tpl';
  }
}
