<?php declare(strict_types=1);

namespace Plugin\agi_shopgate_connect\Backend;

use JTL\Alert\Alert;
use JTL\DB\ReturnType;
use JTL\Helpers\Form;
use JTL\Helpers\Request;
use JTL\Shop;
use stdClass;

/**
 * Class AdminApiEinrichtung
 * @package Plugin\agi_shopgate_connect\Backend
 */
class AdminZahlartenZuordnen extends CustomLink
{
  /**
   * @return bool
   */
  protected function controller(): bool
  {
    $aZahlarten = array();
    $aZahlarten[0] = array('kZahlungsart'=>0,'cName'=>'keine Zuordnung');
    
    // load own plugins-configuration
    $own_config = array();
    $aRes = $this->db->queryPrepared(
      "SELECT cName,cWert FROM xplugin_agi_shopgate_connect_config WHERE cName IN ('cZahlungsartenZuordnen','cShopgateZahlartenZuordnen')",
      [],
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $row) {
      $own_config[$row->cName] = $row->cWert;
    }
    $tmpShopgateZahlarten = empty($own_config['cZahlungsartenZuordnen']) ? array() : json_decode($own_config['cZahlungsartenZuordnen'], true);
    
    $query = "
      SELECT kZahlungsart, cName
      FROM tzahlungsart
      WHERE nActive = 1
      ORDER BY kZahlungsart
    ";
    $aRes = $this->db->queryPrepared(
      $query,
      [],
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $row) {
      $aZahlarten[$row->kZahlungsart] = array('kZahlungsart'=>$row->kZahlungsart, 'cName'=>$row->cName);
    }
    
    $query = "
      SELECT kPlugin, cPluginID
      FROM tplugin
      WHERE `cPluginID` IN('s360_amazon_lpa_shop4', 's360_amazon_lpa') AND `nStatus`=2
    ";
    $aRes = $this->db->queryPrepared(
      $query,
      [],
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $row) {
      $aZahlarten['5360'] = array('kZahlungsart'=>'5360','cName'=>'s360 Amazon Payments');
      break;
    }

    $aSGPaymentGroups = array(
      array('name'=>'Fallback - wenn die Shoppgate-Zahlungsart nicht zugeordnet ist','methods'=>array('OTHER')),
      array('name'=>'Amazon Payments','methods'=>array('MWS')),
      array('name'=>'Cash on Delivery / Nachnahme','methods'=>array('COD')),
      array('name'=>'Credit Card','methods'=>array('AUTHN_CC','BRAINTR_CC','CHASE_CC','CMPTOP_CC','CONCAR_CC','CYBRSRC_CC','DT_CC','DTCASH_CC','EWAY_CC','FRSTDAT_CC','HDLPAY_CC','MASTPAY_CC','OGONE_CC','PAY4ONE_CC','PAYJUNC_CC','PAYONE_CC','PP_WSPP_CC','REALEX_CC','SAGEPAY_CC','STRIPE_CC','TELECSH_CC','TRNSFST_CC','USAEPAY_CC','WCARDS_CC','WLDPDIR_CC')),
      array('name'=>'Credit card (redirect)','methods'=>array('HIPAY','SIX_CC','SKRILL_CC','WIRECRD_CC','WORLDLINE')),
      array('name'=>'Direct Debit / Lastschrift','methods'=>array('DEBIT','PAYMRW_DBT','PAYONE_DBT','SPONE_DBT','WCARDS_DBT')),
      array('name'=>'Direct Debit (external)','methods'=>array('CMPTOP_DBT','SKRILL_DBT','WCARD_DBT')),
      array('name'=>'Giropay','methods'=>array('PAYONE_GP')),
      array('name'=>'Installment / Raten','methods'=>array('ACCRD_INS','PAYOL_INS')),
      array('name'=>'Invoice / Rechnung','methods'=>array('INVOICE','ACCRD_INV','BILLSAFE','KLARNA_INV','MSTPAY_INV','PAYMRW_INV','PAYOL_INV','PAYONE_INV','PAYONE_KLV','WCARD_INV')),
      array('name'=>'MasterCard','methods'=>array('UPAID_MCM')),
      array('name'=>'Merchant payment / eigene Zahlungsart','methods'=>array('MERCH_PM','MERCH_PM_2','MERCH_PM_3')),
      array('name'=>'On-site pickup / Abholung','methods'=>array('COLL_STORE')),
      array('name'=>'Pay in store / barzahlen','methods'=>array('BARZAHLEN')),
      array('name'=>'PayPal','methods'=>array('PAYPAL','PPC_PAYPAL','PPC_CC','PPC_INV','PPC_SEPA','PPC_EPS','PPC_GP','PPC_IDEAL','PPC_SOFORT','PPAL_PLUS','CMPTOP_PP','MASTPAY_PP','PAYONE_PP','SAGEPAY_PP','SIX_PP','WCARD_PP','WCARDS_PP')),
      array('name'=>'PayU','methods'=>array('PAYU')),
      array('name'=>'Prepayment / Vorkasse','methods'=>array('PREPAY')),
      array('name'=>'SOFORT Überweisung','methods'=>array('SUE','HDLPAY_SUE','MASTPAY_SUE','PAYONE_SUE','SKRILL_SUE','WCARD_SUE','WCARDS_SUE')),
      //array('name'=>'Shopgate','methods'=>array('shopgate')),
      array('name'=>'iDEAL','methods'=>array('PAYONE_IDL','SIX_IDEAL','SKRILL_IDL','WCARD_IDL','WCARDS_IDL')),
      array('name'=>'Klarna','methods'=>array('KLARNA_PAY'))
    );

    $aSGPaymentMethods = array(
      // Fallback
      'OTHER' => 'Standardwert',
      
      // Amazon Payments
      'MWS' => 'Amazon Payments',
      
      // Cash on Delivery
      'COD' => 'Nachnahme',
      
      // Credit Card
      'AUTHN_CC'    => 'Authorize.net',
      'BRAINTR_CC'  => 'Braintree',
      'CHASE_CC'    => 'Chase Paymentech (Orbital)',
      'CMPTOP_CC'   => 'Computop',
      'CONCAR_CC'   => 'ConCardis (DirectLink)',
      'CYBRSRC_CC'  => 'CyberSource',
      'DT_CC'       => 'Datatrans',
      'DTCASH_CC'   => 'DataCash',
      'EWAY_CC'     => 'eWAY',
      'FRSTDAT_CC'  => 'FirstData US / Global Gateway e4',
      'HDLPAY_CC'   => 'HeidelPay',
      'MASTPAY_CC'  => 'Masterpayment',
      'OGONE_CC'    => 'Ingenico (Ogone)',
      'PAY4ONE_CC'  => 'Pay4one',
      'PAYJUNC_CC'  => 'PayJunction',
      'PAYONE_CC'   => 'PAYONE',
      'PP_WSPP_CC'  => 'PayPal Website Payments Pro',
      'REALEX_CC'   => 'Relax',
      'SAGEPAY_CC'  => 'SagePay',
      'STRIPE_CC'   => 'Stripe',
      'TELECSH_CC'  => 'Telecash',
      'TRNSFST_CC'  => 'TransFirst',
      'USAEPAY_CC'  => 'USA ePay',
      'WCARDS_CC'   => 'Wirecard (Checkout Seamless)',
      'WLDPDIR_CC'  => 'WorldPay(SecureNet)',
      
      // Credid Card (external)
      'HIPAY'       => 'Hipay (Offsite)',
      'SIX_CC'      => 'SIX Saferpay',
      'SKRILL_CC'   => 'Skrill',
      'WIRECRD_CC'  => 'Wirecard (Checkout Page)',
      'WORLDLINE'   => 'Worldline (Offsite)',
      
      // Direct Debit
      'DEBIT'       => 'Lastschrift',
      'PAYMRW_DBT'  => 'Direct debit (Paymorrow)',
      'PAYONE_DBT'  => 'Direct debit (PAYONE)',
      'SPONE_DBT'   => 'Direct debit (SEPAone)',
      'WCARDS_DBT'  => 'Direct debit (Wirecard (Checkout Seamless))',
      
      // Direct Debit (external)
      'CMPTOP_DBT'  => 'Direct debit (Computop)',
      'SKRILL_DBT'  => 'Direct debit (external) (Skrill)',
      'WCARD_DBT'   => 'Direct debit (Wirecard (Checkout Page))',
      
      // Giropay
      'PAYONE_GP'   => 'Giropay (PAYONE)',
      
      // Installment 
      'ACCRD_INS'   => 'Installment (Accarda)',
      'PAYOL_INS'   => 'Installment (Payolution)',
      
      // Invoice
      'INVOICE'     => 'Rechnung',
      'ACCRD_INV'   => 'Invoice (Accarda)',
      'BILLSAFE'    => 'Billsafe',
      'KLARNA_INV'  => 'Klarna (alt)',
      'MSTPAY_INV'  => 'Masterpayment',
      'PAYMRW_INV'  => 'Invoice (Paymorrow)',
      'PAYOL_INV'   => 'Invoice (Payolution)',
      'PAYONE_INV'  => 'Invoice (PAYONE)',
      'PAYONE_KLV'  => 'Invoice (externe) (PAYONE)',
      'WCARD_INV'   => 'Invoice (Wirecard (Checkout Page))',
      
      // MasterCard
      'UPAID_MCM' => 'MasterPass',
      
      // Merchant payment
      'MERCH_PM'    => 'Merchant payment (Merchant payment)',
      'MERCH_PM_2'  => 'Merchant payment (Merchant payment 2)',
      'MERCH_PM_3'  => 'Merchant payment (Merchant payment 3)',
      
      // On-site pickup
      'COLL_STORE'  => 'Collection at store (processed by you)',
      
      // Pay in store
      'BARZAHLEN'   => 'Prepayment (Barzahlen)',
      
      // PayPal
      'PAYPAL'      => 'PayPal',
      'PPC_PAYPAL'  => 'PayPal Checkout - PayPal',
      'PPC_CC'      => 'PayPal Checkout - Kreditkarte',
      'PPC_INV'     => 'PayPal Checkout - Rechnung',
      'PPC_SEPA'    => 'PayPal Checkout - Lastschrift',
      'PPC_EPS'     => 'PayPal Checkout - EPS',
      'PPC_GP'      => 'PayPal Checkout - GiroPay',
      'PPC_IDEAL'   => 'PayPal Checkout - IDEAL',
      'PPC_SOFORT'  => 'PayPal Checkout - Sofort.',
      'PPAL_PLUS'   => 'PayPal Plus',
      'CMPTOP_PP'   => 'PayPal (Computop)',
      'MASTPAY_PP'  => 'PayPal (Masterpayment)',
      'PAYONE_PP'   => 'PayPal (PAYONE)',
      'SAGEPAY_PP'  => 'PayPal (SagePay)',
      'SIX_PP'      => 'PayPal (SIX Saferpay)',
      'WCARD_PP'    => 'PayPal (Wirecard (Checkout Page))',
      'WCARDS_PP'   => 'PayPal (Wirecard (Checkout Seamless))',
      
      // PayU
      'PAYU'        => 'PayU (Your PayU account)',
      
      // Prepayment
      'PREPAY'      => 'Vorkasse',
      
      // SOFORT Überweisung
      'SUE'         => 'Sofortüberweisung',
      'HDLPAY_SUE'  => 'SOFORT Überweisung (HeidelPay)',
      'MASTPAY_SUE' => 'SOFORT Überweisung (Masterpayment)',
      'PAYONE_SUE'  => 'SOFORT banking (PAYONE)',
      'SKRILL_SUE'  => 'Sofortüberweisung (Skrill)',
      'WCARD_SUE'   => 'Sofortüberweisung (Wirecard (Checkout Page))',
      'WCARDS_SUE'  => 'Sofortüberweisung (Wirecard (Checkout Seamless))',
      
      // Shopgate'
      //'shopgate'    => '',
      
      // iDEAL
      'PAYONE_IDL'  => 'iDEAL (PAYONE)',
      'SIX_IDEAL'   => 'iDEAL (SIX Saferpay)',
      'SKRILL_IDL'  => 'iDEAL (Skrill)',
      'WCARD_IDL'   => 'iDEAL (Wirecard (Checkout Page))',
      'WCARDS_IDL'  => 'iDEAL (Wirecard (Checkout Seamless))',
      
      // KLARNA
      'KLARNA_PAY'  => 'Klarna Payments',
      
      'OTHER'       => 'Sonstiges'
    );
    
    $shopgateZahlarten = array();
    $aOverwrite = array();
    foreach ($aSGPaymentMethods as $cKey => $name) {
      if (array_key_exists($cKey, $tmpShopgateZahlarten)) {
        $shopgateZahlarten[$cKey] = array(
          'id'      => $tmpShopgateZahlarten[$cKey]['id'], 
          'name'    => $name,
          'useName' => array_key_exists('useName',$tmpShopgateZahlarten[$cKey]) ? $tmpShopgateZahlarten[$cKey]['useName'] : '',
          'isPayed' => array_key_exists('isPayed',$tmpShopgateZahlarten[$cKey]) ? $tmpShopgateZahlarten[$cKey]['isPayed'] : 0
        );
      }
      else {
        $shopgateZahlarten[$cKey] = array('id' => 0, 'name' => $name, 'useName'=>'','isPayed'=>0);
      }
      $aOverwrite[$cKey] = $name;
    }
    $aSGPaymentMethods = $aOverwrite;
    /* nicht abändern */
    
    if (Form::validateToken() && ($posted = Request::postVar('api_action')) !== null) {
      $values = $this->getPlugin()->getConfig()->getOptions();
      foreach ($aSGPaymentMethods as $key => $name) {
        $iz = Request::postVar('inputZahlart_'.$key);
        if ($iz !== null && array_key_exists($iz, $aZahlarten)) {
          $izName = Request::postVar('inputZahlartName_'.$key);
          $shopgateZahlarten[$key] = array(
            'id'      => $iz,
            'name'    => $name,
            'useName' => !empty($izName) ? $izName : '',
            'isPayed' => Request::postVar('inputZahlartPayed_'.$key) !== null ? 1 : 0
          );
        }
        else {
          $shopgateZahlarten[$key] = array('id' => 0, 'name' => $name, 'useName' => '', 'isPayed' => 0);
        }
      }
      $this->db->queryPrepared(
        "UPDATE xplugin_agi_shopgate_connect_config SET cWert= :cWert WHERE cName='cZahlungsartenZuordnen'",
        ['cWert' => json_encode($shopgateZahlarten)],
        ReturnType::QUERYSINGLE
      );
      
      $sgRelations = Request::postVar('inputPaymentRelationSG');
      if ($sgRelations !== null) {
        $tmp = trim($sgRelations);
        $this->db->queryPrepared(
          "UPDATE xplugin_agi_shopgate_connect_config SET cWert= :cWert WHERE cName='cShopgateZahlartenZuordnen'",
          ['cWert' => $tmp],
          ReturnType::QUERYSINGLE
        );
        $own_config['cShopgateZahlartenZuordnen'] = $tmp;
      }
    }
    
    
    Shop::Smarty()->assign('aZahlarten', $aZahlarten);
    Shop::Smarty()->assign('aShopgateZahlarten', $shopgateZahlarten);
    Shop::Smarty()->assign('aShopgateZahlartenGruppen', $aSGPaymentGroups);
    Shop::Smarty()->assign('valuePaymentRelationSG', $own_config['cShopgateZahlartenZuordnen']);
    
    return true;
  }
  
  /**
   * @return string
   */
  protected function getTemplate(): string
  {
    return 'templates/tab_zahlarten.tpl';
  }
}
