<?php declare(strict_types=1);

namespace Plugin\agi_shopgate_connect\Backend;

use Illuminate\Support\Collection;
use JTL\DB\DbInterface;
use JTL\Helpers\Text;
use JTL\Plugin\PluginInterface;
use JTL\Smarty\JTLSmarty;
use SmartyException;

/**
 * Class CustomLink
 * @package Plugin\agi_shopgate_connect\Backend
 */
abstract class CustomLink
{
  /**
   * @var PluginInterface
   */
  protected $plugin;

  /**
   * @var DbInterface
   */
  protected $db;

  /**
   * CustomLink constructor.
   * @param PluginInterface $plugin
   * @param DbInterface     $db
   * @param array           $requestData
   */
  private function __construct(PluginInterface $plugin, DbInterface $db)
  {
    $this->plugin      = $plugin;
    $this->db          = $db;
  }

  /**
   * @param PluginInterface $plugin
   * @param DbInterface     $db
   * @param array           $requestData
   * @return static
   */
  public static function handleRequest(PluginInterface $plugin, DbInterface $db): self
  {
    $instance = new static($plugin, $db);
    $instance->controller();

    return $instance;
  }

  /**
   * @return bool
   */
  abstract protected function controller(): bool;

  /**
   * @return string
   */
  abstract protected function getTemplate(): string;

  /**
   * @return PluginInterface
   */
  public function getPlugin(): PluginInterface
  {
    return $this->plugin;
  }

  /**
   * @param JTLSmarty $smarty
   * @return string
   */
  public function display(JTLSmarty $smarty): string
  {
    try {
      return $smarty->assign('kPlugin', $this->getPlugin()->getID())
        ->fetch($this->getPlugin()->getPaths()->getAdminPath() . $this->getTemplate());
    } catch (SmartyException $e) {
      return $e->getMessage();
    }
  }
    /**
   * @param Collection $options
   * @param string     $name
   * @param string     $value
   */
  protected function updatePluginOption(Collection $options, string $name, string $value): void
  {
    $hit = $options->first(static function ($e) use ($name) {
      return $e->valueID === $name;
    });
    if ($hit !== null) {
      $hit->value = $value;
      $this->db->update(
        'tplugineinstellungen',
        ['kPlugin', 'cName'],
        [$this->getPlugin()->getID(), $name],
        (object)['cWert' => $value]
      );
    }
    $this->getPlugin()->getConfig()->setOptions($options);
  }
}

