<?php

namespace Plugin\agi_shopgate_connect;

use JTL\DB\ReturnType;
use JTL\Events\Dispatcher;
use JTL\Plugin\Bootstrapper;
use JTL\Plugin\Data\Config;
use JTL\Plugin\PluginInterface;
use JTL\Shop;
use JTL\Smarty\JTLSmarty;
use Plugin\agi_shopgate_connect\Backend\AdminApiEinrichtung;
use Plugin\agi_shopgate_connect\Backend\AdminZahlartenZuordnen;

/**
 * Class Bootstrap
 * @package Plugin\agi_shopgate_connect
 */
class Bootstrap extends Bootstrapper
{
  private $config;
  
  /**
   * @inheritDoc
   */
  public function boot(Dispatcher $dispatcher): void
  {
    parent::boot($dispatcher);

    $this->config   = clone $this->getPlugin()->getConfig();
    if (Shop::isFrontend()) {
      $plugin       = $this->getPlugin();
      if ($this->config->getValue('sgRedirectActive') === 'Y') {
        $dispatcher->listen(
          'shop.hook.' . \HOOK_SMARTY_OUTPUTFILTER ,
          [$this, 'agi_shopgate_frontend_redirect'],
          10
        );
      }
    } else {
      //$dispatcher->listen('shop.hook.' . \HOOK_PLUGIN_SAVE_OPTIONS, [$this, 'saveOptions']);
    }
  }
  
  /**
   * @inheritDoc
   */
  public function updated($oldVersion, $newVersion)
  {
    parent::updated($oldVersion, $newVersion);

    if ((int)$oldVersion > 100 && \strlen($oldVersion) === 3) {
      //DBMigrationHelper::migrateToInnoDButf8('old_database');
    }
  }
  
  /**
   * @inheritdoc
   */
  public function renderAdminMenuTab(string $tabName, int $menuID, JTLSmarty $smarty): string
  {
    $smarty->assign('menuID', $menuID);
    if ($tabName === 'Zahlarten zuordnen') {
      return AdminZahlartenZuordnen::handleRequest($this->getPlugin(), $this->getDB(), $menuID)->display($smarty);
    }
    if ($tabName === 'API-Einrichtung') {
      return AdminApiEinrichtung::handleRequest($this->getPlugin(), $this->getDB(), $menuID)->display($smarty);
    }
    return parent::renderAdminMenuTab($tabName, $menuID, $smarty);
  }

  public function agi_shopgate_frontend_redirect(array &$args)
  {
    // Hier Frontend-Umleitung
    $agi_frontent_conf = array(
      'cISO'          => Shop::getLanguage(true),
      'AktuelleSeite' => Shop::$AktuelleSeite,
      'kKategorie'    => Shop::$kKategorie,
      'kArtikel'      => Shop::$kArtikel
    );
    $jtl_to_page = 'no';
    if ($agi_frontent_conf['kKategorie'] > 0 || $agi_frontent_conf['kArtikel'] > 0) {
      if ($agi_frontent_conf['kArtikel'] > 0) {
        $jtl_to_page = 'article';
      }
      elseif ($agi_frontent_conf['kKategorie'] > 0) {
        $jtl_to_page = 'category';
      }
      elseif ($agi_frontent_conf['AktuelleSeite']=="STARTSEITE") {
        $jtl_to_page = 'start'; // in 405 nicht mehr unterstützt
      }
      else {
        $jtl_to_page = 'default';
      }
    }
    else {
      $aStartseite = array(PAGE_STARTSEITE,PAGE_SITEMAP,PAGE_404);
      $aNoRedirect = array(PAGE_UNBEKANNT,PAGE_WRB,PAGE_PLUGIN,PAGE_BESTELLABSCHLUSS,PAGE_MEDIA,PAGE_GRATISGESCHENK);
      if (in_array(Shop::getPageType(),$aStartseite)) {
        $jtl_to_page = 'start';
      }
      elseif (!in_array(Shop::getPageType(),$aNoRedirect)) {
        $jtl_to_page = 'default';
      }
    }
    //var_dump(SHOP::getPageType(), SHOP::$AktuelleSeite);
    $tmp = $this->getDB()->queryPrepared(
      "SELECT kSprache FROM tsprache WHERE cISO= :cISO",
      ["cISO" => $agi_frontent_conf['cISO']],
      ReturnType::SINGLE_OBJECT
    );

    if ($tmp) {
      if (!empty($this->config->getValue('sgLanguagesToShops'))) {
        $data = unserialize($this->config->getValue('sgLanguagesToShops'));
        if (array_key_exists($tmp->kSprache, $data)) {
          $sgShopnumber = $data[$tmp->kSprache];
          $sgConfigurationFound = true;
        }
      }

      if ($sgConfigurationFound) {
        require_once PFAD_ROOT . 'plugins/agi_shopgate_connect/shopgate_sdk/shopgate.php';
        $sg_config = new \ShopgateConfig(); 
        $sg_config->loadByShopNumber((int) $sgShopnumber);
        $shopgateBuilder = new \ShopgateBuilder($sg_config);
        $shopgateRedirector = $shopgateBuilder->buildRedirect();
        
        $string = '';
        if ($jtl_to_page!='no') {
          if ($jtl_to_page == 'article') {
            $string = $shopgateRedirector->buildScriptItem($agi_frontent_conf['kArtikel']);
          }
          elseif ($jtl_to_page == 'category') {
            $string = $shopgateRedirector->buildScriptCategory($agi_frontent_conf['kKategorie']);
          }
          elseif ($jtl_to_page == 'start') {
            $string = $shopgateRedirector->buildScriptShop();
          }
          else {
            $string = $shopgateRedirector->buildScriptDefault();
          }
        }
        pq('head')->append($string);
      }
    }
  }
}
