<?php

namespace Plugin\agi_shopgate_connect;

ini_set('date.timezone', 'Europe/Berlin');
//error_reporting(E_ALL ^E_DEPRECATED);
//ini_set('display_errors','on');

/* BOF includes from includes/globalinclude.php */
use JTL\Helpers\PHPSettings;
use JTL\Language\LanguageHelper;
use JTL\Plugin\Helper;
use JTL\Profiler;
use JTL\Router\Router;
use JTL\Router\State;
use JTL\Session\Frontend;
use JTL\Shop;
use JTLShop\SemVer\Version;
use JTL\DB\ReturnType;

$nStartzeit = microtime(true);

if (file_exists(__DIR__ . '/../../includes/config.JTL-Shop.ini.php')) {
    require_once __DIR__ . '/../../includes/config.JTL-Shop.ini.php';
}

/**
 * @param string $message
 */
function handleFatal(string $message): void
{
    header('Cache-Control: no-store, no-cache, must-revalidate, max-age=0');
    header('Cache-Control: post-check=0, pre-check=0', false);
    header('Pragma: no-cache', true, 500);
    die($message);
}

if (defined('PFAD_ROOT')) {
    require_once PFAD_ROOT . 'includes/defines.php';
} else {
    handleFatal('Could not get configuration from config file. ' .
        'For shop installation <a href="install/">click here</a>.');
}

require_once PFAD_ROOT . PFAD_INCLUDES . 'autoload.php';

defined('DB_HOST') || handleFatal('Kein MySql-Datenbank Host angegeben. Bitte config.JTL-Shop.ini.php bearbeiten!');
defined('DB_NAME') || handleFatal('Kein MySql Datenbanknamen angegeben. Bitte config.JTL-Shop.ini.php bearbeiten!');
defined('DB_USER') || handleFatal('Kein MySql-Datenbank Benutzer angegeben. Bitte config.JTL-Shop.ini.php bearbeiten!');
defined('DB_PASS') || handleFatal('Kein MySql-Datenbank Passwort angegeben. Bitte config.JTL-Shop.ini.php bearbeiten!');

$db     = null;
$config = null;
$shop   = Shop::getInstance();

if (!function_exists('Shop')) {
    /**
     * @return Shop
     */
    function Shop(): Shop
    {
        return Shop::getInstance();
    }
}
// PHP memory_limit work around
if (!PHPSettings::getInstance()->hasMinLimit(64 * 1024 * 1024)) {
    ini_set('memory_limit', '64M');
}

require_once PFAD_ROOT . PFAD_INCLUDES . 'tools.Global.php';

try {
    $db = Shop::Container()->getDB();
} catch (Exception $exc) {
    handleFatal($exc->getMessage());
}
require_once PFAD_ROOT . PFAD_INCLUDES . 'plugin_inc.php';
if (!defined('CLI_BATCHRUN')) {
    $cache = Shop::Container()->getCache();
    $cache->setJtlCacheConfig($db->selectAll('teinstellungen', 'kEinstellungenSektion', CONF_CACHING));
    $config = Shop::getSettings([CONF_GLOBAL]);
    Shop::setRouter(new Router($db, $cache, new State(), Shop::Container()->getAlertService(), $config));
    $lang   = LanguageHelper::getInstance($db, $cache);
}
// if (!JTL_INCLUDE_ONLY_DB && !defined('CLI_BATCHRUN')) {
    require_once PFAD_ROOT . PFAD_INCLUDES . 'artikel_inc.php';
    require_once PFAD_ROOT . PFAD_INCLUDES . 'sprachfunktionen.php';
    require_once PFAD_ROOT . PFAD_INCLUDES . 'artikelsuchspecial_inc.php';
    $pluginHooks    = Helper::getHookList();
    executeHook(HOOK_GLOBALINCLUDE_INC);
// }
/* EOF includes from includes/globalinclude.php */

$db = Shop::Container()->getDB();
$query = "
  SELECT *
  FROM tplugin
  WHERE cPluginID = 'agi_shopgate_connect'
";
$oPlugin = $db->queryPrepared(
  $query,
  [],
  ReturnType::SINGLE_OBJECT
);
if ($oPlugin) {
  if ($oPlugin->nStatus > 1) {
    if ($oPlugin->nStatus >= 5) {
      echo '{"error_text":"JTL-Plugin License not active","error":999,"trace_id":"'.((isset($_REQUEST['trace_id']) && !empty($_REQUEST['trace_id'])) ? $_REQUEST['trace_id'] : '').'"}';
    } else {
      require_once __DIR__.'/shopgate_sdk/shopgate.php';
      require_once __DIR__.'/src/agi_shopgate_connect.php';
      define('AGI_SHOPGATE_CONNECT_KPLUGIN', $oPlugin->kPlugin);
      $agiPlugin = new \agi_shopgate_connect();
      $agiPlugin->handleRequest($_POST);
    }
  }
  else {
    echo '{"error_text":"JTL-Plugin not activated","error":999,"trace_id":"'.((isset($_REQUEST['trace_id']) && !empty($_REQUEST['trace_id'])) ? $_REQUEST['trace_id'] : '').'"}';
  }
}
else {
  echo '{"error_text":"JTL-Plugin not installed","error":999,"trace_id":"'.((isset($_REQUEST['trace_id']) && !empty($_REQUEST['trace_id'])) ? $_REQUEST['trace_id'] : '').'"}';
}
