<?php
use JTL\Shop;
use JTL\Catalog\Category\Kategorie;
use JTL\Cart\Cart;
use JTL\Cart\CartItem;
use JTL\Cart\CartItemProperty;
use JTL\Catalog\Product\Artikel;
use JTL\Catalog\Product\EigenschaftWert;
use JTL\Catalog\Product\Preise;
use JTL\Checkout\Bestellung;
use JTL\Checkout\Eigenschaft;
use JTL\Checkout\Kupon;
use JTL\Checkout\Lieferadresse;
use JTL\Checkout\OrderHandler;
use JTL\Checkout\Rechnungsadresse;
use JTL\Checkout\StockUpdater;
use JTL\Customer\Customer;
use JTL\DB\ReturnType;
use JTL\Extensions\Config\Group AS Konfiggroup;
use JTL\Extensions\Config\Item AS Konfigitem;
use JTL\Firma;
use JTL\Helpers\PaymentMethod;
use JTL\Helpers\ShippingMethod;
use JTL\Helpers\Tax;
use JTL\Helpers\Text;
use JTL\Mail\Mail\Mail;
use JTL\Mail\Mailer;
use JTL\Plugin\Helper AS PluginHelper;

define('SHOPGATE_PLUGIN_VERSION', '1.54.0');

class PluginBase extends ShopgatePlugin {
  
  protected $db = null;
  protected $JTLPluginConf = array();
  protected $URL_SHOP;
  protected $UTF8_SHOP = true;
  
  public $request_shop_number = 0;
  public $kSprache = 1;
  public $cSpracheISO = '';
  public $aSprachen = array();
  
  public $oWaehrung = null;
  
  public $kKundengruppe = 1;
  public $oExportArticle = null;
  //public $shopURLEndSlash = false;
  private $startExecutionTime = 0;
  private $maxExecutionTime = 55;
  private $fallback_steuersatz = 19;
  protected $bAllowKonfiguration = false;
  protected $aTeinstellungenFields = array();
  
  /** Variablen fr Produktexport */
  protected $aVersandart = array();
  protected $cLieferLand = 'DE';
  protected $cNurAbhaengigeVersandart = "N";
  protected $aKundengruppen = array();
  protected $aSteuerklassen = array();
  protected $aAttributNichtExportieren = array();
  protected $aMerkmaleNichtExportieren = array();
  protected $aAddDescAttribute = array();
  protected $aRemoveDescAttribute = array();
  public $cWrapDescriptionAttribute = '<p><strong>%s:</strong> %s</p>';
  private $cTrennzeichen = ';';
  private $tSprache = array('vat'=>'MwSt.');
  private $load_to_article = array('media','description','categories','vpe','attributes');

  
  /**
    * Callback function for initialization by plugin implementations.
    *
    * This method gets called on instantiation of a ShopgatePlugin child class and serves as __construct()
    * replacement.
    *
    * Important: Initialize $this->config here if you have your own config class.
    *
    * @see http://developer.shopgate.com/library
    */
  public function startup() 
  {
   	// Prepare and set up your stuff here. This function is always called before any other.
    $this->startExecutionTime = time();
    $mt = @ini_get('max_execution_time');
    if (!empty($mt) && $mt > -1) {
      $this->maxExecutionTime = $mt-5;
    }
    
    $this->URL_SHOP = substr(URL_SHOP,-1) == '/' ? URL_SHOP : URL_SHOP.'/';
    $this->UTF8_SHOP = strtolower(JTL_CHARSET) == 'utf-8' ? true : false;
    
    $this->db = Shop::Container()->getDB();
    $this->JTLPluginConf = PluginHelper::getConfigByID(AGI_SHOPGATE_CONNECT_KPLUGIN);
    $aRes = $this->db->queryPrepared(
      "SELECT cName,cWert FROM xplugin_agi_shopgate_connect_config",
      [],
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $row) {
      $this->JTLPluginConf[$row->cName] = $row->cWert;
    }
    if (!empty($this->JTLPluginConf['cZahlungsartenZuordnen'])) {
      $this->JTLPluginConf['cZahlungsartenZuordnen'] = json_decode($this->JTLPluginConf['cZahlungsartenZuordnen'], true);
    }
    //var_dump($this->JTLPluginConf);
    
    $oKundengruppe = $this->db->queryPrepared(
      "SELECT kKundengruppe FROM tkundengruppe WHERE cStandard= :cStandard",
      ['cStandard' => 'Y'],
      ReturnType::SINGLE_OBJECT
    );
    $this->kKundengruppe = $oKundengruppe->kKundengruppe;
    
    $oSprache = $this->db->queryPrepared(
      "SELECT kSprache, cISO FROM tsprache WHERE cShopStandard= :cShopStandard",
      ['cShopStandard' => 'Y'],
      ReturnType::SINGLE_OBJECT
    );
    if (!$oSprache) {
      $oSprache = $this->db->queryPrepared(
        "SELECT kSprache, cISO FROM tsprache LIMIT 1",
        [],
        ReturnType::SINGLE_OBJECT
      );
    }
    $this->aSprachen[] = $oSprache;
    $this->kSprache = $oSprache->kSprache;
    $this->cSpracheISO = $oSprache->cISO;
    
    $oWaehrung = $this->db->queryPrepared(
      "SELECT * FROM twaehrung WHERE cStandard= :cStandard",
      ['cStandard' => 'Y'],
      ReturnType::SINGLE_OBJECT
    );
    if (!$oWaehrung) {
      $oWaehrung = $this->db->queryPrepared(
        "SELECT * FROM twaehrung LIMIT 1",
        [],
        ReturnType::SINGLE_OBJECT
      );
    }
    $this->oWaehrung = $oWaehrung;
    Tax::setTaxRates(null);
    
    // If you use Shopgate Library above 2.1 and a extended ShopgateConfig class, you need to set it here
   	//$this->config = new ShopgateConfigMyShoppingSystem();
    if (!isset($_REQUEST['shop_number']) || empty($_REQUEST['shop_number'])) {
      throw new ShopgateLibraryException(
   			ShopgateLibraryException::PLUGIN_API_UNKNOWN_SHOP_NUMBER,
   			''
   		);
      return false;
    }

    $this->request_shop_number = (int)$_REQUEST['shop_number'];
    $this->config = new ShopgateConfig();
    $this->config->loadByShopNumber($this->request_shop_number);
    
    $this->config->setServer('live');
    $this->config->setApiUrl('https://api.shopgate.com/merchant/');
    $this->config->setEncoding('UTF-8');
    $this->config->setPluginName('AGIShopgateConnect');
    $this->config->setEnableAddOrder(1);
    $this->config->setEnableUpdateOrder(1);
    $this->config->setEnableCheckStock(1);
    $this->config->setEnableGetCustomer(1);
    $this->config->setEnableRegisterCustomer(1);
    $this->config->setEnableGetItems(1);
    $this->config->setEnableGetCategories(1);
    $this->config->setEnableCron(1);
    $this->config->setEnableGetSettings(1);
    $this->config->setEnableCheckCart(1);
    $this->config->setEnableRedeemCoupons(0);
    $this->config->setEnableGetReviews(1);
    $this->config->setEnableGetOrders(1);
    
    $this->config->setEnableGetItemsCsv(0);
    $this->config->setEnableGetCategoriesCsv(0);
    $this->config->setEnableGetReviewsCsv(0);
    
    $addtitional = $this->config->returnAdditionalSettings();
    if (array_key_exists('jtlSprache',$addtitional) && !empty($addtitional['jtlSprache'])) {
      if ($addtitional['jtlSprache'] != $this->kSprache) {
        $this->kSprache = $addtitional['jtlSprache'];
        $oSprache = $this->db->queryPrepared(
          "SELECT kSprache, cISO FROM tsprache WHERE kSprache= :kSprache",
          ['kSprache' => $this->kSprache],
          ReturnType::SINGLE_OBJECT
        );
        $this->aSprachen = array($oSprache);
        $this->kSprache = $oSprache->kSprache;
        $this->cSpracheISO = $oSprache->cISO;
        Shop::setLanguage((int)$this->kSprache, $oSprache->cISO);
      }
    }
    
    $fields = array('vat','shippingTimeLP','shippingTime','productInflowing','supplierStockNotice','deliveryStatus','productAvailableFrom','preorderPossible');
    $aRes = $this->db->queryPrepared(
      "SELECT cName,cWert FROM tsprachwerte WHERE cName IN ('".implode("','", $fields)."') AND kSprachISO= :kSprachISO",
      ['kSprachISO' => $this->kSprache],
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $row) {
      $this->tSprache[$row->cName] = $row->cWert;
    }
    
    $fields = array('kundenregistrierung_standardland','bewertung_anzeigen','bewertung_freischalten','kundenregistrierung_nur_lieferlaender','global_erscheinende_kaeuflich');
    $aRes = $this->db->queryPrepared(
      "SELECT cName,cWert FROM teinstellungen WHERE cName IN ('".implode("','", $fields)."')",
      [],
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $row) {
      $this->aTeinstellungenFields[$row->cName] = $row->cWert;
    }
    $this->cLieferLand = $this->aTeinstellungenFields['kundenregistrierung_standardland'];
    
    // lade Steuerklassen
    $aRes = $this->db->query(
      'SELECT * FROM tsteuerklasse',
      ReturnType::ARRAY_OF_ASSOC_ARRAYS
    );
    foreach ($aRes as $row) {
      $this->aSteuerklassen[$row['kSteuerklasse']] = $row;
    }
    
    // Konfigurator
    if (isset($this->JTLPluginConf['cActivateKonfigurator']) && $this->JTLPluginConf['cActivateKonfigurator'] == 'Y' && class_exists('Konfigurator')) {
      $this->bAllowKonfiguration = true;
    }
    $this->aTeinstellungenFields['s360_amazonpay_shop5'] = false;
    $this->aTeinstellungenFields['s360_amazonpay_shop5_active'] = false;
    $mws = $this->db->queryPrepared(
      "SELECT * FROM tplugin WHERE cPluginID = :cPluginID AND nStatus=2",
      ['cPluginID' => 's360_amazonpay_shop5'],
      ReturnType::SINGLE_OBJECT
    );
    if (!empty($mws)) {
      $this->aTeinstellungenFields['s360_amazonpay_shop5'] = true;
    }
    // CSS Gutscheine
    $css = $this->db->queryPrepared(
      "SELECT * FROM tplugin WHERE cPluginID = :cPluginID AND nStatus=2",
      ['cPluginID' => 'css_gutschein'],
      ReturnType::SINGLE_OBJECT
    );
    $this->aTeinstellungenFields['bHasCSS_GUTSCHEIN'] = empty($css) ? false : true;
    
    if (method_exists($this, 'agi_shopgate_connect__startup')) {
      $this->agi_shopgate_connect__startup();
    }
  }

  /**
    * This performs the necessary queries to build a ShopgateCustomer object for the given log in credentials.
    *
    * The method should not abort on soft errors like when the street or phone number of a customer can't be
    * found.
    *
    * @see http://developer.shopgate.com/plugin_api/customers/get_customer
    *
    * @param string $user The user name the customer entered at Shopgate Connect.
    * @param string $pass The password the customer entered at Shopgate Connect.
    * @return ShopgateCustomer A ShopgateCustomer object.
    * @throws ShopgateLibraryException on invalid log in data or hard errors like database failure.
    */
  public function getCustomer($user, $pass)
  {
    $JTL_Client = new Customer();
    $userNotFoundOrPasswordIsWrong = $JTL_Client->holLoginKunde(
      $user, 
      $pass
    ) === 1 ? FALSE : TRUE;
   	if ($userNotFoundOrPasswordIsWrong) {
   		throw new ShopgateLibraryException(
   			ShopgateLibraryException::PLUGIN_WRONG_USERNAME_OR_PASSWORD,
   			'User: '.$user
   		);
   	}

    $cryptoService = Shop::Container()->getCryptoService();
    
    $addresses = array();
    $adress_check = array();
    // load delivery-addresses
    $dbCustomerAddresses = $this->db->queryPrepared(
      "SELECT * FROM tlieferadresse WHERE kKunde= :kKunde",
      ['kKunde' => $JTL_Client->kKunde],
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($dbCustomerAddresses as $dbCustomerAddress) {
      switch ($dbCustomerAddress->cAnrede) {
        case 'w': $gender = 'f'; break;
        case 'm': $gender = 'm'; break;
        default: $gender = '';
      }
      
      $address = new ShopgateAddress();
      $address->setId('D'.$dbCustomerAddress->kLieferadresse);
      $address->setAddressType(ShopgateAddress::DELIVERY);
      if (!empty($gender)) {
        $address->setGender($gender);
      }
      $address->setFirstName($this->_prepare_utf8($dbCustomerAddress->cVorname));
      $address->setLastName($this->_prepare_utf8($cryptoService->decryptXTEA(trim($dbCustomerAddress->cNachname))));
      $address->setBirthday($JTL_Client->dGeburtstag);
      $address->setCompany($this->_prepare_utf8($cryptoService->decryptXTEA(trim($dbCustomerAddress->cFirma))));
      $address->setStreet1($this->_prepare_utf8($cryptoService->decryptXTEA(trim($dbCustomerAddress->cStrasse)).(empty($dbCustomerAddress->cHausnummer) ? '' : ' '.$dbCustomerAddress->cHausnummer)));
      $address->setStreet2($this->_prepare_utf8($dbCustomerAddress->cAdressZusatz));
      $address->setCity($this->_prepare_utf8($dbCustomerAddress->cOrt));
      $address->setZipcode($this->_prepare_utf8($dbCustomerAddress->cPLZ));
      $address->setCountry($this->_prepare_utf8($dbCustomerAddress->cLand));
      $address->setPhone($this->_prepare_utf8($dbCustomerAddress->cTel));
      $address->setMobile($this->_prepare_utf8($dbCustomerAddress->cMobil));
      $address->setMail($this->_prepare_utf8($dbCustomerAddress->cMail));
      $check_String = implode(',', [
        $address->getFirstName(),
        $address->getLastName(),
        $address->getStreet1(),
        $address->getCity(),
        $address->getZipcode()
      ]);
      if (!in_array($check_String, $adress_check)) {
        $addresses[] = $address;
        $adress_check[] = $check_String;
      }
    }
    
    switch ($JTL_Client->cAnrede) {
      case 'w': $gender = 'f'; break;
      case 'm': $gender = 'm'; break;
      default: $gender = '';
    }
    $address = new ShopgateAddress();
    $address->setId('C'.$JTL_Client->kKunde);
    $address->setAddressType(ShopgateAddress::BOTH);
    if (!empty($gender)) {
      $address->setGender($gender);
    }
    $address->setFirstName($this->_prepare_utf8($JTL_Client->cVorname));
    $address->setLastName($this->_prepare_utf8($JTL_Client->cNachname));
    $address->setBirthday($this->_prepare_utf8($JTL_Client->dGeburtstag));
    $address->setCompany($this->_prepare_utf8($JTL_Client->cFirma));
    $address->setStreet1($this->_prepare_utf8($JTL_Client->cStrasse.(empty($JTL_Client->cHausnummer) ? '' : ' '.$JTL_Client->cHausnummer)));
    $address->setStreet2($this->_prepare_utf8($JTL_Client->cAdressZusatz));
    $address->setCity($this->_prepare_utf8($JTL_Client->cOrt));
    $address->setZipcode($this->_prepare_utf8($JTL_Client->cPLZ));
    $address->setCountry($this->_prepare_utf8($JTL_Client->cLand));
    $address->setPhone($this->_prepare_utf8($JTL_Client->cTel));
    $address->setMobile($this->_prepare_utf8($JTL_Client->cMobil));
    $address->setMail($this->_prepare_utf8($JTL_Client->cMail));
    $addresses[] = $address;
    
    $customer = new ShopgateCustomer();
    $customer->setCustomerId($JTL_Client->kKunde);
    $customer->setCustomerToken($JTL_Client->kKunde);
    if (!empty($JTL_Client->cKundenNr)) {
      $customer->setCustomerNumber($JTL_Client->cKundenNr);
    }
    if (!empty($JTL_Client->kKundengruppe)) {
      $customer->setCustomerGroupId($JTL_Client->kKundengruppe);
    }
    $customer->setGender($gender);
    $customer->setFirstName($this->_prepare_utf8($JTL_Client->cVorname));
    $customer->setLastName($this->_prepare_utf8($JTL_Client->cNachname));
    $customer->setBirthday($this->_prepare_utf8($JTL_Client->dGeburtstag));
    $customer->setPhone($this->_prepare_utf8($JTL_Client->cTel));
    $customer->setMobile($this->_prepare_utf8($JTL_Client->cMobil));
    $customer->setMail($this->_prepare_utf8($JTL_Client->cMail));
    
    if (!empty($JTL_Client->kKundengruppe)) {
      $aRes = $this->db->queryPrepared(
        "SELECT cName FROM tkundengruppe WHERE kKundengruppe= :kKundengruppe",
        ['kKundengruppe' => $JTL_Client->kKundengruppe],
        ReturnType::SINGLE_OBJECT
      );
      if ($aRes) {
        $customerGroups = array();
          $group = new ShopgateCustomerGroup();
          $group->setID($JTL_Client->kKundengruppe);
          $group->setName($this->_prepare_utf8($aRes->cName));
        $customerGroups[] = $group;
        $customer->setCustomerGroups($customerGroups);
        $customer->setCustomerGroup($this->_prepare_utf8($aRes->cName));
      }
    }
    
    $customer->setAddresses($addresses);

    if (method_exists($this, 'agi_shopgate_connect__getCustomer')) {
      $customer = $this->agi_shopgate_connect__getCustomer($customer);
    }
    
    return $customer;
  }

  /**
    * Checks the content of a cart to be valid and returns necessary changes if applicable.
    *
    * This currently only supports the validation of coupons.
    *
    * @see http://developer.shopgate.com/plugin_api/cart
    *
    * @param ShopgateCart $cart The ShopgateCart object to be checked and validated.
    * @return array(
    *		'external_coupons' => ShopgateExternalCoupon[], # list of all coupons
    *		'items' => array(...), # list of item changes (not supported yet)
    *		'shippings' => array(...), # list of available shipping services for this cart (not supported yet)
    * )
    * @throws ShopgateLibraryException if an error occurs.
    */
  public function checkCart(ShopgateCart $shopgateCart)
  {
    if (method_exists($this, 'agi_shopgate_connect__checkCart')) {
      $this->agi_shopgate_connect__checkCart($shopgateCart);
    }
    
    $items = $shopgateCart->getItems();
    $this->_AGICreateItemsConfig();
    
    $aItemResponse = array();
    $aPositionen   = array();
    $aWarenWertGet = array('brutto'=>0, 'netto'=>0);
    $aWarenWertSet = array('brutto'=>0, 'netto'=>0);
    $aArtikelMitWertImKorb = array();
    $_SESSION['Warenkorb'] = new Cart();
    $_SESSION['Kunde'] = new Customer();
    
    foreach($items as $checkItem) {
      $itemId = $checkItem->getItemNumber();
      $itemsTaxPercent = $checkItem->getTaxPercent();
      $itemQuantity = $checkItem->getQuantity();
      $itemOptions = $checkItem->getOptions();
      $itemInputs = $checkItem->getInputs();
      $aOptionsWert  = array('brutto'=>0, 'netto'=>0);
      
      $aWarenWertGet['brutto'] += $itemQuantity*$checkItem->getUnitAmountWithTax();
      $aWarenWertGet['netto'] += $itemQuantity*$checkItem->getUnitAmount();
      $bAllowForBasket = true;
      
      // prepare response
      $oItemResponse = new ShopgateCartItem();
      $oItemResponse->setItemNumber($itemId);
      $oItemResponse->setUnitAmount($checkItem->getUnitAmount());
      $oItemResponse->setUnitAmountWithTax($checkItem->getUnitAmountWithTax());
      $oItemResponse->setOptions($itemOptions);
      $oItemResponse->setInputs($itemInputs);
      
      // load Article
      $this->_loadOExportArticle($itemId);
      if (!empty($this->oExportArticle->kArtikel)) {
        $this->_AGIoExportArticleCheckStatusAktiv();
        $this->_AGIoExportArticleAddAttributes();
        
        $artikelInKats = array(); // used later again
        if(count($this->oExportArticle->oKategorie_arr) > 0) {
          foreach ($this->oExportArticle->oKategorie_arr as $oCategorie) {
            if (is_int($oCategorie)) {
              $artikelInKats[] = $oCategorie;
            }
            else {
              $artikelInKats[] = $oCategorie->kKategorie;
            }
          }
        }
        $aPosition = array(
          'kArtikel'    => $this->oExportArticle->kArtikel,
          'cArtNr'      => $this->oExportArticle->cArtNr,
          'aKategorien' => $artikelInKats
        );
        $aPositionen[] = $aPosition;
        
        $oItemResponse->setItemNumber($itemId);
        
        if ($this->oExportArticle->cStatusAktiv == 'inactive') {
          $oItemResponse->setIsBuyable(0);
          $oItemResponse->setQtyBuyable(0);
          $oItemResponse->setStockQuantity(0);
          $oItemResponse->setError(300);
          $oItemResponse->setErrorText('cart item out of stock');
          $bAllowForBasket = false;
        }
        elseif ($this->oExportArticle->cStatusAktiv == 'stock') {
          $oItemResponse->setStockQuantity($this->oExportArticle->fLagerbestand);
          if ($this->oExportArticle->fLagerbestand < $itemQuantity) {
            $oItemResponse->setQtyBuyable($this->oExportArticle->fLagerbestand);
            $oItemResponse->setIsBuyable(0);
            $oItemResponse->setError(302);
            $oItemResponse->setErrorText('cart item requested quantity not available');
            if($this->oExportArticle->fLagerbestand < 1) {
              $oItemResponse->setError(300);
              $oItemResponse->setErrorText('cart item out of stock');
            }
            $bAllowForBasket = false;
          }
          else {
            if ($this->oExportArticle->fMindestbestellmenge > $itemQuantity) {
              $oItemResponse->setQtyBuyable($this->oExportArticle->fMindestbestellmenge);
              $oItemResponse->setIsBuyable(0);
              $oItemResponse->setError(304);
              $oItemResponse->setErrorText('requested quantity is lower than required minimum quantity');
              $bAllowForBasket = false;
            }
            else {
              $oItemResponse->setQtyBuyable($itemQuantity);
              $oItemResponse->setIsBuyable(1);
            }
          }
        }
        else {
          $oItemResponse->setStockQuantity($this->oExportArticle->fLagerbestand);
          $oItemResponse->setQtyBuyable($itemQuantity);
          $oItemResponse->setIsBuyable(1);
        }
        
        $oEigenschaftwerte_arr = array();
        // Konfigurator
        $tmpOptions = array();
        $tmpKonfigurationArticle = array();
        if (!empty($itemOptions)) {
          foreach ($itemOptions as $checkOption) {
            $option_number = $checkOption->getOptionNumber();
            if (substr($option_number,0,10) == 'agiKonfig_') {
              $tmpKonfigurationArticle[] = $checkOption;
            }
            else {
              $tmpOptions[] = $checkOption;
            }
          }
        }
        $itemOptions = $tmpOptions;
        if ($this->bAllowKonfiguration && $this->oExportArticle->bHasKonfig && !empty($this->oExportArticle->oKonfig_arr)) {
          $aGroupsConfiged = array();
          if (!empty($tmpKonfigurationArticle)) {
            foreach ($tmpKonfigurationArticle as $oKonfigurationOption) {
              $option_number = $oKonfigurationOption->getOptionNumber();
              $value_number = $oKonfigurationOption->getValueNumber();
              if ($value_number) {
                //array(kKonfiggruppe ,kKonfigitem, kArtikel, Anzahl);
                $aKonfigKeys = explode('_', $value_number);
                if (count($aKonfigKeys) == 4) {
                  $kItem = new Konfigitem($aKonfigKeys[1]);
                  $kGruppe = $kItem->getKonfiggruppe();
                  if (!array_key_exists($aKonfigKeys[0], $aGroupsConfiged)) {
                    $oGruppe = new Konfiggroup($kGruppe);
                    $aGroupsConfiged[$aKonfigKeys[0]] = array('nMin' => $oGruppe->getMin(), 'nMax' => $oGruppe->getMax(), 'name' => $oGruppe->getSprache()->getName(), 'cnt'=>0);
                  }
                  if ($kGruppe == $aKonfigKeys[0] && !empty($aKonfigKeys[2]) && $kItem->getArtikelKey() == $aKonfigKeys[2]) {
                    if ($aKonfigKeys[3] > 0) {
                      $aGroupsConfiged[$aKonfigKeys[0]]['cnt'] += 1;
                    }
                    $tmpArticle = $kItem->getArtikel();
                    if ($tmpArticle->inWarenkorbLegbar == -1) {
                      $bAllowForBasket = false;
                      $oItemResponse->setIsBuyable(0);
                      $oItemResponse->setError(301);
                      $oItemResponse->setErrorText('cart item product-option not found');
                      $itemerror = true;
                    }
                    else {
                      if ($tmpArticle->cLagerBeachten == 'Y' && $tmpArticle->cLagerKleinerNull == 'N' && $tmpArticle->fLagerbestand <= 0) {
                        $bAllowForBasket = false;
                        $oItemResponse->setIsBuyable(0);
                        $oItemResponse->setError(301);
                        $oItemResponse->setErrorText('cart item product-option not found');
                        $itemerror = true;
                      }
                    }
                  }
                  else {
                    if (!empty($aKonfigKeys[2])) {
                      $bAllowForBasket = false;
                      $oItemResponse->setIsBuyable(0);
                      $oItemResponse->setError(301);
                      $oItemResponse->setErrorText('cart item product-option not found');
                      $itemerror = true;
                    }
                  }
                }
                else {
                  $bAllowForBasket = false;
                  $oItemResponse->setIsBuyable(0);
                  $oItemResponse->setError(301);
                  $oItemResponse->setErrorText('cart item product-option not found');
                  $itemerror = true;
                }
              }
            }
          }
          if (!$itemerror) {
            foreach ($aGroupsConfiged as $value) {
              if ($value['nMin'] > 0 && $value['nMin']>$value['cnt']) {
                $oItemResponse->setIsBuyable(0);
                $oItemResponse->setError(301);
                $oItemResponse->setErrorText($this->_prepare_utf8('Bitte whlen Sie mindestens eine Option "'.$value['name'].'" aus'));
                break;
              }
              if ($value['nMax'] > 0 && $value['nMax']<$value['cnt']) {
                $oItemResponse->setIsBuyable(0);
                $oItemResponse->setError(301);
                $oItemResponse->setErrorText($this->_prepare_utf8('Sie haben zu viele Optionen "'.$value['name'].'" gewhlt'));
                break;
              }
            }
          }
        }
        elseif(!empty($tmpKonfigurationArticle)) {
          $bAllowForBasket = false;
          $oItemResponse->setIsBuyable(0);
          $oItemResponse->setError(301);
          $oItemResponse->setErrorText('cart item product has no options');
        }
        
        
        if (!empty($itemOptions) && !empty($this->oExportArticle->aSelectAttribute)) {
          foreach ($itemOptions as $checkOption) {
            $option_number = $checkOption->getOptionNumber();
            $value_number = $checkOption->getValueNumber();
            if (
              array_key_exists($option_number, $this->oExportArticle->aSelectAttribute) 
              && array_key_exists($value_number, $this->oExportArticle->aSelectAttribute[$option_number]['aWerte']))
            {
              $fOptionPrice = $checkOption->getAdditionalAmountWithTax()/100;
              $aOptionsWert['brutto'] += $fOptionPrice;
              $aOptionsWert['netto']  += round($fOptionPrice*100/(100+$itemsTaxPercent),5);
              $aOption = $this->oExportArticle->aSelectAttribute[$option_number]['aWerte'][$value_number];
              
              $oEigenschaftwerte = new stdClass();
              $oEigenschaftwerte->kEigenschaft = $option_number;
              $oEigenschaftwerte->cFreifeldWert= '';
              $oEigenschaftwerte->kEigenschaftWert=$value_number;
              $oEigenschaftwerte->fAufpreisNetto = round($fOptionPrice*100/(100+$itemsTaxPercent),5);
              $oEigenschaftwerte_arr[] = $oEigenschaftwerte;
              
              if ($this->oExportArticle->cStatusAktiv == 'active' || ($this->oExportArticle->cStatusAktiv == 'stock' && ($aOption['fLagerbestand'] > 0 || $this->oExportArticle->cLagerVariation == 'N'))) {
                if ($this->oExportArticle->cStatusAktiv == 'stock') {
                  if ($this->oExportArticle->cLagerVariation == 'Y' && $aOption['fLagerbestand'] <= 0) {
                    $oItemResponse->setStockQuantity(0);
                    $oItemResponse->setIsBuyable(0);
                    $oItemResponse->setError(300);
                    $oItemResponse->setErrorText('cart item out of stock');
                    $bAllowForBasket = false;
                  }
                  elseif ($this->oExportArticle->cLagerVariation == 'Y' && $aOption['fLagerbestand'] < $itemQuantity) {
                    $oItemResponse->setStockQuantity($aOption['fLagerbestand']);
                    $oItemResponse->setIsBuyable(0);
                    $oItemResponse->setQtyBuyable($aOption['fLagerbestand']);
                    $oItemResponse->setError(302);
                    $oItemResponse->setErrorText('cart item requested quantity not available');
                    $bAllowForBasket = false;
                  }
                }
              }
              else {
                $oItemResponse->setIsBuyable(0);
                $oItemResponse->setStockQuantity(0);
                $oItemResponse->setError(300);
                $oItemResponse->setErrorText('cart item out of stock');
                $bAllowForBasket = false;
              }
            }
            else {
              $oItemResponse->setError(301);
              $oItemResponse->setErrorText('cart item product-option not found');
            }
          }
        }
        elseif (!empty($itemOptions) && empty($this->oExportArticle->aSelectAttribute)) {
          $oItemResponse->setError(301);
          $oItemResponse->setErrorText('cart item product has no options');
        }
        
        if (!empty($itemInputs) && !empty($this->oExportArticle->aInputAttribute)) {
          foreach ($itemInputs as $checkOption) {
            $option_number = $checkOption->getInputNumber();
            if (array_key_exists($option_number, $this->oExportArticle->aInputAttribute)) {
              $oEigenschaftwerte = new stdClass();
              $oEigenschaftwerte->kEigenschaft = $option_number;
              $oEigenschaftwerte->cFreifeldWert= $checkOption->getUserInput();
              $oEigenschaftwerte->kEigenschaftWert=0;
              $oEigenschaftwerte->fAufpreisNetto=0;
              $oEigenschaftwerte_arr[] = $oEigenschaftwerte;
            }
          }
        }
      }
      else {
        $oItemResponse->setError(301);
        $oItemResponse->setErrorText('cart item product not found');
        $bAllowForBasket = false;
      }
      
      if ($bAllowForBasket) {
        $NeuePosition = new CartItem();
        $NeuePosition->Artikel = $this->oExportArticle;
        $NeuePosition->nAnzahl = $itemQuantity;
        $NeuePosition->kArtikel = $NeuePosition->Artikel->kArtikel;
        $NeuePosition->kVersandklasse = $NeuePosition->Artikel->kVersandklasse;
        $NeuePosition->kSteuerklasse = $NeuePosition->Artikel->kSteuerklasse;
        $NeuePosition->fPreisEinzelNetto = $checkItem->getUnitAmount();
        $NeuePosition->fPreis = $checkItem->getUnitAmount()+$aOptionsWert['netto'];		
        $NeuePosition->cArtNr = $NeuePosition->Artikel->cArtNr;
        $NeuePosition->nPosTyp = C_WARENKORBPOS_TYP_ARTIKEL;
        $NeuePosition->cEinheit = $NeuePosition->Artikel->cEinheit;
        $NeuePosition->fGesamtgewicht = $NeuePosition->gibGesamtgewicht();
        $NeuePosition->WarenkorbPosEigenschaftArr = array();
        
        if(is_array($NeuePosition->Artikel->Variationen) && count($NeuePosition->Artikel->Variationen) > 0) {
          foreach ($NeuePosition->Artikel->Variationen as $eWert) {
            foreach ($oEigenschaftwerte_arr as $oEigenschaftwerte) {
              //gleiche Eigenschaft suchen
              if ($oEigenschaftwerte->kEigenschaft == $eWert->kEigenschaft) {
                if ($eWert->cTyp=="FREIFELD" || $eWert->cTyp=="PFLICHT-FREIFELD") {
                  $NeueWarenkorbPosEigenschaft = new CartItemProperty();
                  $NeueWarenkorbPosEigenschaft->kEigenschaft = $oEigenschaftwerte->kEigenschaft;
                  $NeueWarenkorbPosEigenschaft->kEigenschaftWert = 0;
                  $NeueWarenkorbPosEigenschaft->fGewichtsdifferenz = 0;
                  $NeueWarenkorbPosEigenschaft->fAufpreis = 0;
                  $NeuePosition->WarenkorbPosEigenschaftArr[] = $NeueWarenkorbPosEigenschaft;
                }
                elseif ($oEigenschaftwerte->kEigenschaftWert > 0) {
                  $EigenschaftWert = new EigenschaftWert($oEigenschaftwerte->kEigenschaftWert);
                  $Eigenschaft = new Eigenschaft($oEigenschaftwerte->kEigenschaft);
                  // Varkombi Kind?
                  if($NeuePosition->Artikel->kVaterArtikel > 0) {
                    if ($Eigenschaft->kArtikel == $NeuePosition->Artikel->kVaterArtikel) {
                      $NeueWarenkorbPosEigenschaft = new CartItemProperty();
                      $NeueWarenkorbPosEigenschaft->kEigenschaft = $oEigenschaftwerte->kEigenschaft;
                      $NeueWarenkorbPosEigenschaft->kEigenschaftWert = $oEigenschaftwerte->kEigenschaftWert;
                      $NeueWarenkorbPosEigenschaft->fGewichtsdifferenz = $EigenschaftWert->fGewichtDiff;
                      $NeueWarenkorbPosEigenschaft->fAufpreis = $oEigenschaftwerte->fAufpreisNetto;
                      
                      $NeuePosition->WarenkorbPosEigenschaftArr[] = $NeueWarenkorbPosEigenschaft;
                      $NeuePosition->fGesamtgewicht += $itemQuantity*$EigenschaftWert->fGewichtDiff;
                    }
                  }
                  else {
                    if ($Eigenschaft->kArtikel == $NeuePosition->kArtikel) {
                      $NeueWarenkorbPosEigenschaft = new CartItemProperty();
                      $NeueWarenkorbPosEigenschaft->kEigenschaft = $oEigenschaftwerte->kEigenschaft;
                      $NeueWarenkorbPosEigenschaft->kEigenschaftWert = $oEigenschaftwerte->kEigenschaftWert;
                      $NeueWarenkorbPosEigenschaft->fGewichtsdifferenz = $EigenschaftWert->fGewichtDiff;
                      $NeueWarenkorbPosEigenschaft->fAufpreis = $oEigenschaftwerte->fAufpreisNetto;
                      
                      $NeuePosition->WarenkorbPosEigenschaftArr[] = $NeueWarenkorbPosEigenschaft;
                      $NeuePosition->fGesamtgewicht += $itemQuantity*$EigenschaftWert->fGewichtDiff;
                    }
                  }
                }
              }
            }
          }
        }
        
        $_SESSION['Warenkorb']->PositionenArr[] = $NeuePosition;
        
        $tmp_netto  = $itemQuantity*($checkItem->getUnitAmount()+$aOptionsWert['netto']);
        $tmp_brutto = $itemQuantity*($checkItem->getUnitAmountWithTax()+$aOptionsWert['brutto']);
        $cArtNr = $checkItem->getItemNumberPublic();
        if (!empty($cArtNr)) {
          if (!array_key_exists($cArtNr, $aArtikelMitWertImKorb)) {
            $aArtikelMitWertImKorb[$cArtNr] = array(
              'netto' => 0,
              'brutto'=> 0,
              'aKategorien'=>$artikelInKats,
              'kHersteller'=>isset($NeuePosition->Artikel->kHersteller) && !empty($NeuePosition->Artikel->kHersteller) ? $NeuePosition->Artikel->kHersteller : 0
            );
          }
          $aArtikelMitWertImKorb[$cArtNr]['netto'] += $tmp_netto;
          $aArtikelMitWertImKorb[$cArtNr]['brutto'] += $tmp_brutto;
        }
        $aWarenWertSet['brutto'] += $tmp_brutto;
        $aWarenWertSet['netto'] += $tmp_netto;
      }
      $aItemResponse[] = $oItemResponse;
    }
    
    $aCustomerResponse = array('customer_groups'=>array());
    $kKunde = 0;
    $kKundengruppe = $this->kKundengruppe;
    $fKundeGuthabenBrutto = 0;
    $cKundenMail = $shopgateCart->getMail();
    if (!empty($cKundenMail)) {
      $cMail = $this->_prepare_utf8($cKundenMail, false);
      $aRes = $this->db->queryPrepared(
        "SELECT kKunde, kKundengruppe, fGuthaben FROM tkunde WHERE cMail= :cMail AND nRegistriert=1",
        ['cMail' => $cMail],
        ReturnType::SINGLE_OBJECT
      );
      if (!empty($aRes)) {
        $kKundengruppe = $aRes->kKundengruppe;
        $kKunde = $aRes->kKunde;
        $fKundeGuthabenBrutto = $aRes->fGuthaben;
        $_SESSION['Kunde']->loadFromDB($kKunde);
      }
    }
    $group = new ShopgateCartCustomerGroup();
    $group->setId($kKundengruppe);
    $aCustomerResponse = new ShopgateCartCustomer();
    $aCustomerResponse->setCustomerGroups(array($group));
    
    $deliveryCountry = false;
    $oDelivery = $shopgateCart->getDeliveryAddress();
    if ($oDelivery) {
      $tmp = $this->_prepare_utf8($oDelivery->getCountry(), false);
      if (!empty($tmp)) {
        $deliveryCountry = $tmp;
      }
    }
    
    // ermittle Steuersatz
    $fSteuersatz = $this->fallback_steuersatz;
    $cLandSteuersatz = !$deliveryCountry ? $this->cLieferLand : $deliveryCountry;
    $aRes = $this->db->queryPrepared(
      "SELECT kSteuerzone FROM tsteuerzoneland WHERE cISO= :cISO",
      ['cISO' => $cLandSteuersatz],
      ReturnType::SINGLE_OBJECT
    );
    if (!empty($aRes)) {
      $kSteuerzone = $aRes->kSteuerzone;
      $aRes = $this->db->queryPrepared(
        "SELECT tsteuersatz.fSteuersatz FROM tsteuersatz, tsteuerklasse WHERE tsteuersatz.kSteuerklasse=tsteuerklasse.kSteuerklasse AND tsteuerklasse.cStandard='Y' AND tsteuersatz.kSteuerzone= :kSteuerzone",
        ['kSteuerzone' => $kSteuerzone],
        ReturnType::SINGLE_OBJECT
      );
      $fSteuersatz = $aRes->fSteuersatz;
    }
    
    $aCouponsResponse = array();
    $aGuthabenKupon = array();
    $aNeukundenKupon = array();
    $fSummeKupons = array('brutto'=>0, 'netto'=>0);
    $coupons = $shopgateCart->getExternalCoupons();
    $hatVersandkupon = false;
    $cnt_coupons = 0;
    $cnt_validKoupons = 0;
    foreach ($coupons as $checkCoupon) {
      //suche code
      $cCode = $checkCoupon->getCode();
      if (!empty($cCode)) {
        $tmpCode = $this->_prepare_utf8($cCode, false);
        if ($tmpCode == 'GUTHABENVERRECHNUNG') {
          $aGuthabenKupon = array(
            'amount_netto'  => $checkCoupon->getAmountNet()/100,
            'amount'  => $checkCoupon->getAmountGross()/100
          );
        }
        elseif($tmpCode == 'NEUKUNDENRABATT') {
          $aNeukundenKupon = array(
            'amount_netto'  => $checkCoupon->getAmountNet()/100
          );
        }
        else {
          // BOF CSS-Gutscheine 5/8
          if($this->aTeinstellungenFields['bHasCSS_GUTSCHEIN']) {
            // try to find code
            $oCssGutschein = $this->db->queryPrepared(
              "SELECT * FROM css_gutscheinCoupon WHERE voucherCode= :voucherCode AND valid=1 AND DateExpire>".time(),
              ["voucherCode" => $tmpCode],
              ReturnType::SINGLE_OBJECT
            );
            if($oCssGutschein) {
              if(!isset($wertgutscheine)) {
                $wertgutscheine = array();
              }
              $wertgutscheine[$tmpCode] = array(
                'code'  => $tmpCode,
                'rest'  => round($oCssGutschein->restVoucherWorth*100/(100+$fSteuersatz), 5),
                'valid' => true,
                'active'=> $oCssGutschein->restVoucherWorth <= 0 ? false : true
              );
              $this->JTLPluginConf['cGuthabenverrechnung'] = 'N';
              continue;
            }
          }
          // EOF CSS-Gutscheine 5/8
          
          $cnt_coupons++;
          $oKupon = $this->db->queryPrepared(
            "SELECT * FROM tkupon WHERE cCode= :cCode",
            ['cCode' => $tmpCode],
            ReturnType::SINGLE_OBJECT
          );
          $aKuponArtikelList = array();
          if($oKupon && $cnt_coupons <=1 ) {
            $kuponSprache = $this->db->queryPrepared(
              "SELECT * FROM tkuponsprache WHERE kKupon= :kKupon AND cISOSprache= :cISOSprache",
              ['kKupon' => $oKupon->kKupon, 'cISOSprache' => Shop::Lang()->getIsoFromLangID((int)$this->kSprache)->cISO],
              ReturnType::SINGLE_OBJECT
            );
            $checkResult = $this->_checkCoupon($oKupon, array('cMail'=>$cKundenMail, 'kKunde'=>$kKunde), $kKundengruppe, $aWarenWertSet, $aPositionen, $deliveryCountry);
            if ($oKupon->cWertTyp == 'prozent') {
              $kuponwert = array(
                'brutto'  => round($aWarenWertSet['brutto']*$oKupon->fWert/100, 5),
                'netto'   => round($aWarenWertSet['netto']*$oKupon->fWert/100, 5)
              );
              if ((!empty($oKupon->cHersteller) && $oKupon->cHersteller != -1) || (!empty($oKupon->cKategorien) && $oKupon->cKategorien != -1)) {
                $aKouponKategorien = array();
                $aKouponHersteller = array();
                if (!empty($oKupon->cKategorien) && $oKupon->cKategorien != -1) {
                  $cKategorien = $oKupon->cKategorien;
                  if (substr($cKategorien,0,1) == ';') {
                    $cKategorien = substr($cKategorien,1);
                  }
                  if (substr($cKategorien,strlen($cKategorien)-1,1) == ';') {
                    $cKategorien = substr($cKategorien,0,strlen($cKategorien)-1);
                  }
                  $aKouponKategorien = explode(';',$cKategorien);
                }
                if (!empty($oKupon->cHersteller) && $oKupon->cHersteller != -1) {
                  $tmpHersteller = explode(';',$oKupon->cHersteller);
                  foreach ($tmpHersteller as $value) {
                    if (!empty($value)) {
                      $aKouponHersteller[] = $value;
                    }
                  }
                }
                  
                $fNewSumNetto = 0;
                $fNewSumBrutto = 0;
                foreach ($aArtikelMitWertImKorb as $cArtNr => $pos) {
                  $allow_hersteller = empty($pos['kHersteller']) || (!empty($pos['kHersteller']) && in_array($pos['kHersteller'], $aKouponHersteller)) ? true : false;
                  $allow_kategorie = empty($aKouponKategorien) ? true : false;
                  if (!empty($pos['aKategorien'])) {
                    foreach ($pos['aKategorien'] as $tmpKat) {
                      if (in_array($tmpKat, $aKouponKategorien)) {
                        $allow_kategorie = true;
                        break;
                      }
                    }
                  }
                  if ($allow_hersteller && $allow_kategorie) {
                    $fNewSumNetto += $pos['netto'];
                    $fNewSumBrutto += $pos['brutto'];
                    $aKuponArtikelList[] = $cArtNr;
                  }
                }
                $kuponwert = array(
                  'brutto'  => round($fNewSumBrutto*$oKupon->fWert/100, 5),
                  'netto'   => round($fNewSumNetto*$oKupon->fWert/100, 5)
                );
              }
              if (!empty($oKupon->cArtikel)) {
                $aKouponArticle = explode(';',$oKupon->cArtikel);
                if (!empty($aKouponArticle)) {
                  $fNewSumNetto = 0;
                  $fNewSumBrutto = 0;
                  foreach($aKouponArticle as $value) {
                    if (array_key_exists($value, $aArtikelMitWertImKorb)) {
                      $fNewSumNetto += $aArtikelMitWertImKorb[$value]['netto'];
                      $fNewSumBrutto += $aArtikelMitWertImKorb[$value]['brutto'];
                      $aKuponArtikelList[] = $value;
                    }
                  }
                  $kuponwert = array(
                    'brutto'  => round($fNewSumBrutto*$oKupon->fWert/100, 5),
                    'netto'   => round($fNewSumNetto*$oKupon->fWert/100, 5)
                  );
                }
              }
            }
            else {
              $kuponwert = array(
                'brutto'  => $oKupon->fWert,
                'netto'   => round($oKupon->fWert*100/(100+$fSteuersatz), 5)
              );
            }
            $kupon = new ShopgateExternalCoupon();
            $kupon->setIsValid($checkResult > 0 ? 0 : 1);
            $kupon->setNotValidMessage($checkResult > 0 ? $this->_prepare_utf8($this->_getKuponErrorMessage($checkResult)) : '');
            $kupon->setCode($this->_prepare_utf8($oKupon->cCode));
            $kupon->setName($this->_prepare_utf8($kuponSprache ? $kuponSprache->cName : $oKupon->cName));
            $kupon->setDescription('');
            $kupon->setAmountGross($kuponwert['brutto']);
            //$kupon->setAmountNet($kuponwert['netto']);
            $kupon->setTaxType('auto');
            $kupon->setIsFreeShipping($checkResult == 0 && $oKupon->cKuponTyp == 'versandkupon' ? 1 : 0);
            $aInfo = array('kKupon'=>$oKupon->kKupon);
            if ($oKupon->nGanzenWKRabattieren == 0 && $oKupon->cWertTyp == 'prozent') {
              $aInfo['rpo'] = array('percent'=> $oKupon->fWert, 'name'=>$kupon->getName().' ('.number_format($oKupon->fWert,2,',','').'% Rabatt)');
            }
            $kupon->setInternalInfo($checkResult == 0 ? $this->jsonEncode($aInfo) : ''); // ID des Coupons, um ihn bei Add-Order schnell zuordnen zu knnen
            $aCouponsResponse[] = $kupon;
            if ($checkResult == 0 && $oKupon->cKuponTyp == 'versandkupon') {
              $hatVersandkupon = true;
            }
            if ($checkResult == 0) {
              $fSummeKupons['brutto'] += $kuponwert['brutto'];
              $fSummeKupons['netto'] += $kuponwert['netto'];
            }
            if ($checkResult == 0) {
              $cnt_validKoupons++;
            }
          }
          else {
            $kupon = $checkCoupon;
            $kupon->setIsValid(0);
            if ($cnt_coupons > 1) {
              $kupon->setNotValidMessage($this->_prepare_utf8('Sie knnen nur einen Rabattcode verwenden'));
            }
            else {
              $kupon->setNotValidMessage('Der Gutscheincode ist nicht korrekt');
            }
            $aCouponsResponse[] = $kupon;
          }
        }
      }
    }
    
    $sgCoupons = $shopgateCart->getShopgateCoupons();
    foreach ($sgCoupons as $checkCoupon) {
      if ($checkCoupon->getIsFreeShipping()) {
        $hatVersandkupon = true;
      }
    }
    
    $fWarenkorbWarenwert = array(
      'brutto'=>$aWarenWertSet['brutto']-$fSummeKupons['brutto'], 
      'netto'=>$aWarenWertSet['netto']-$fSummeKupons['netto']
    );
    
    // BOF CSS-Gutscheine 6/8
    if(isset($wertgutscheine)) {
      $wertgutschein_warenkorbwert = $fWarenkorbWarenwert['netto'];
    }
    // EOF CSS-Gutscheine 6/8
    
    // fge Guthabenkupon ein
    if (!empty($kKunde) && $fKundeGuthabenBrutto > 0 && $this->JTLPluginConf['cGuthabenverrechnung'] == 'Y') {
      // erlaubtes Guthaben darf nicht hher als der Warenwert sein
      $fErlaubtesGuthaben = $fKundeGuthabenBrutto > $fWarenkorbWarenwert['brutto'] ? $fWarenkorbWarenwert['brutto'] : $fKundeGuthabenBrutto;
      if ($fErlaubtesGuthaben > 0) {
        $kupon = new ShopgateExternalCoupon();
        $kupon->setIsValid(1);
        $kupon->setNotValidMessage('');
        $kupon->setCode('GUTHABENVERRECHNUNG');
        $kupon->setName('Guthabenverrechnung');
        $kupon->setDescription('');
        $kupon->setAmountGross($fErlaubtesGuthaben);
        //$kupon->setAmountNet(round($fErlaubtesGuthaben*100/(100+$fSteuersatz), 5));
        $kupon->setTaxType('auto');
        $kupon->setIsFreeShipping(0);
        $aInfo = array('kundenguthaben'=>$kKunde);
        $kupon->setInternalInfo($this->jsonEncode($aInfo));
        $aCouponsResponse[] = $kupon;
      }
    }
    elseif (!empty($aGuthabenKupon)) {
      $kupon = new ShopgateExternalCoupon();
      $kupon->setIsValid(0);
      $kupon->setNotValidMessage('Guthabenverrechnung nicht mglich');
      $kupon->setCode('GUTHABENVERRECHNUNG');
      $kupon->setName('Guthabenverrechnung');
      $kupon->setDescription('');
      //$kupon->setAmountNet($aGuthabenKupon['amount_netto']);
      $kupon->setAmountGross($aGuthabenKupon['amount']);
      $kupon->setTaxType('auto');
      $kupon->setIsFreeShipping(0);
      $kupon->setInternalInfo('');
      $aCouponsResponse[] = $kupon;
    }
    
    // Fge mglichen neukundencoupon ein
    $delete_newclients_coupon = true;
    if ($kKunde > 0 && $cnt_validKoupons == 0) {
      $aRes = $this->db->queryPrepared(
        "SELECT COUNT(*) AS cnt FROM tbestellung WHERE kKunde= :kKunde",
        ['kKunde' => $kKunde],
        ReturnType::SINGLE_OBJECT
      );
      if ($aRes->cnt == 0) {
        // neukundenkoupon ist mglich!
        $oKupons = $this->db->queryPrepared(
          "SELECT * FROM tkupon WHERE cKuponTyp='neukundenkupon' AND cAktiv='Y' AND (kKundengruppe=-1 OR kKundengruppe= :kKundengruppe)",
          ['kKundengruppe' => $kKundengruppe],
          ReturnType::ARRAY_OF_OBJECTS
        );
        $found = false;
        foreach ($oKupons as $oKupon) {
          $checkResult = $this->_checkCoupon($oKupon, array('cMail'=>$cKundenMail, 'kKunde'=>$kKunde), $kKundengruppe, $aWarenWertSet, $aPositionen, $deliveryCountry);
          if ($checkResult == 0) {
            // Kupon gltig
            $found = true;
            break;
          }
        }
        if ($found) {
          if ($oKupon->cWertTyp == 'prozent') {
            $kuponwert = array(
              'brutto'  => round($aWarenWertSet['brutto']*$oKupon->fWert/100, 5),
              'netto'   => round($aWarenWertSet['netto']*$oKupon->fWert/100, 5)
            );
            if (!empty($oKupon->cKategorien) && $oKupon->cKategorien != -1) {
              $cKategorien = $oKupon->cKategorien;
              if (substr($cKategorien,0,1) == ';') {
                $cKategorien = substr($cKategorien,1);
              }
              if (substr($cKategorien,strlen($cKategorien)-1,1) == ';') {
                $cKategorien = substr($cKategorien,0,strlen($cKategorien)-1);
              }
              $aKouponKategorien = explode(';',$cKategorien);
              if (!empty($aKouponKategorien)) {
                $fNewSumNetto = 0;
                $fNewSumBrutto = 0;
                foreach ($aArtikelMitWertImKorb as $cArtNr => $pos) {
                  if (!empty($pos['aKategorien'])) {
                    foreach ($pos['aKategorien'] as $tmpKat) {
                      if (in_array($tmpKat, $aKouponKategorien)) {
                        $fNewSumNetto += $pos['netto'];
                        $fNewSumBrutto += $pos['brutto'];
                        $aKuponArtikelList[] = $cArtNr;
                        break;
                      }
                    }
                  }
                }
                $kuponwert = array(
                  'brutto'  => round($fNewSumBrutto*$oKupon->fWert/100, 5),
                  'netto'   => round($fNewSumNetto*$oKupon->fWert/100, 5)
                );
              }
            }
            if (!empty($oKupon->cArtikel)) {
              $aKouponArticle = explode(';',$oKupon->cArtikel);
              if (!empty($aKouponArticle)) {
                $fNewSumNetto = 0;
                $fNewSumBrutto = 0;
                foreach($aKouponArticle as $value) {
                  if (array_key_exists($value, $aArtikelMitWertImKorb)) {
                    $fNewSumNetto += $aArtikelMitWertImKorb[$value]['netto'];
                    $fNewSumBrutto += $aArtikelMitWertImKorb[$value]['brutto'];
                  }
                }
                $kuponwert = array(
                  'brutto'  => round($fNewSumBrutto*$oKupon->fWert/100, 5),
                  'netto'   => round($fNewSumNetto*$oKupon->fWert/100, 5)
                );
              }
            }
          }
          else {
            $kuponwert = array(
              'brutto'  => $oKupon->fWert,
              'netto'   => round($oKupon->fWert*100/(100+$fSteuersatz), 5)
            );
          }
          $kupon = new ShopgateExternalCoupon();
          $kupon->setIsValid(1);
          $kupon->setNotValidMessage('');
          $kupon->setCode('NEUKUNDENRABATT');
          $kupon->setName($this->_prepare_utf8($kuponSprache ? $kuponSprache->cName : $oKupon->cName));
          $kupon->setDescription('');
          $kupon->setAmountGross($kuponwert['brutto']);
          //$kupon->setAmountNet($kuponwert['netto']);
          $kupon->setTaxType('auto');
          $kupon->setIsFreeShipping(0);
          $aInfo = array('kKupon'=>$oKupon->kKupon);
          $kupon->setInternalInfo($checkResult == 0 ? $this->jsonEncode($aInfo) : ''); // ID des Coupons, um ihn bei Add-Order schnell zuordnen zu knnen
          $aCouponsResponse[] = $kupon;
          $delete_newclients_coupon = false;
          
          // BOF CSS-Gutscheine 7/8
          if(isset($wertgutscheine)) {
            $wertgutschein_warenkorbwert -= $kuponwert['netto'];
          }
          // EOF CSS-Gutscheine 7/8
        }
      }
    }
    if ($delete_newclients_coupon && !empty($aNeukundenKupon)) {
      $kupon = new ShopgateExternalCoupon();
      $kupon->setIsValid(0);
      $kupon->setNotValidMessage('Neukundenrabatt nicht mglich');
      $kupon->setCode('NEUKUNDENRABATT');
      $kupon->setName('Neukundenrabatt');
      $kupon->setDescription('');
      $kupon->setAmountNet($aNeukundenKupon['amount_netto']);
      $kupon->setTaxType('auto');
      $kupon->setIsFreeShipping(0);
      $kupon->setInternalInfo('');
      $aCouponsResponse[] = $kupon;
    }
    $aShippingMethods = array();
    
    $deliveryZip = $oDelivery ? $oDelivery->getZipcode() : false;
    $oVersandart_arr = $this->_gibMoeglicheVersandarten($deliveryCountry, $deliveryZip, ShippingMethod::getShippingClasses($_SESSION['Warenkorb']), $kKundengruppe);
    
    // Versandarten ausblenden
    $aVersteckeVersandart = array();
    if ($deliveryZip) {
      // BOF agws_freeVA_PLZ - Plugin
      $agws_freeVA = array();
      $aTeinstellungenFields = array("agws_freeVA_PLZ","agws_freeVA_PLZ_vname","agws_freeVA_PLZ_mwert","agws_freeVA_PLZ_plzarr");
      $aRes = $this->db->queryPrepared(
        "SELECT * FROM tplugineinstellungen WHERE cName IN ('".implode("','", $aTeinstellungenFields)."')",
        [],
        ReturnType::ARRAY_OF_ASSOC_ARRAYS
      );
      foreach ($aRes as $row) {
        $agws_freeVA[$row['cName']] = $row['cWert'];
      }
      
      if (array_key_exists('agws_freeVA_PLZ', $agws_freeVA) && $agws_freeVA['agws_freeVA_PLZ'] == 'Y' && !empty($agws_freeVA['agws_freeVA_PLZ_vname'])) {
        // kVersandart ermitteln
        $aRes = $this->db->queryPrepared(
          "SELECT kVersandart FROM tversandart WHERE cName= :cName",
          ['cName' => $agws_freeVA['agws_freeVA_PLZ_vname']],
          ReturnType::SINGLE_OBJECT
        );
        if ($aRes) {
          $kVersandart = $tmp -> kVersandart;
          $tmp = str_replace('-',',', $agws_freeVA['agws_freeVA_PLZ_plzarr']);
          if (!empty($tmp)) {
            $plz = explode(',', $tmp);
            if (in_array($deliveryZip, $plz)) {
              if (!empty($agws_freeVA['agws_freeVA_PLZ_mwert']) && $agws_freeVA['agws_freeVA_PLZ_mwert'] > 0) {
                if ($agws_freeVA['agws_freeVA_PLZ_mwert'] > $fWarenkorbWarenwert['brutto']) {
                  $aVersteckeVersandart[] = $kVersandart;
                }
              }
            }
            else {
              $aVersteckeVersandart[] = $kVersandart;
            }
          }
          else {
            $aVersteckeVersandart[] = $kVersandart;
          }
        }
      }
      // EOF agws_freeVA_PLZ - Plugin
    }
   
    foreach ($oVersandart_arr as $oVersandart) {
      if (in_array($oVersandart->kVersandart, $aVersteckeVersandart)) {
        continue;
      }
      
      $oShippingMethod = new ShopgateShippingMethod();
      $oShippingMethod->setId($oVersandart->kVersandart);
      $oShippingMethod->setTitle($this->_prepare_utf8($oVersandart->angezeigterName));
      $oShippingMethod->setDescription($this->_prepare_utf8($this->_remove_html($oVersandart->angezeigterHinweistext)));
      //$oShippingMethod->setSortOrder($oVersandart->nSort);
      if ($oVersandart->fVersandkostenfreiAbX > 0 && $fWarenkorbWarenwert['brutto'] > $oVersandart->fVersandkostenfreiAbX) {
        $oShippingMethod->setAmountWithTax(0);
      }
      else {
        if ($oVersandart->eSteuer == 'brutto') {
          $oShippingMethod->setAmountWithTax($hatVersandkupon ? 0 : $oVersandart->fEndpreis);
        }
        else {
          $oShippingMethod->setAmount($hatVersandkupon ? 0 : $oVersandart->fEndpreis);
          $oShippingMethod->setTaxPercent($fSteuersatz);
          //$oShippingMethod->setTaxClass('default');
          $oShippingMethod->setAmountWithTax($hatVersandkupon ? 0 : round($oVersandart->fEndpreis*(100+$fSteuersatz)/100,2));
        }
      }
      $aInfo = array('kShipping'=>$oVersandart->kVersandart,'cShipping'=>$this->_prepare_utf8($oVersandart->angezeigterName));
      $oShippingMethod->setInternalShippingInfo($this->jsonEncode($aInfo));
      $aShippingMethods[] = $oShippingMethod;
    }

    
    /* BOF Zahlungsarten */
    $aPaymentMethods = array();
    $checkPossible = array();
    $aRes = $this->db->queryPrepared(
      "SELECT DISTINCT kZahlungsart FROM tversandartzahlungsart",
      [],
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $value) {
      if (!in_array($value->kZahlungsart, $checkPossible)) {
        $checkPossible[] = $value->kZahlungsart;
      }
    }
    $shipping_type = $shopgateCart->getShippingType();
    $add_amazon_payments = true;
    if ($shipping_type == 'PLUGINAPI') {
      $cInfo = $shopgateCart->getShippingInfos()->getInternalShippingInfo();
      $bShippingInfo = false;
      if (!empty($cInfo)) {
        $aInfo = $this->jsonDecode($cInfo, true);
        if (array_key_exists('kShipping', $aInfo) && array_key_exists('cShipping', $aInfo) && !empty($aInfo['kShipping'])) {
          $checkPossible = array();
          // Lese alle Zahlungsarten aus, die zu dieser versandart zugelassen sind
          $aRes = $this->db->queryPrepared(
            "SELECT DISTINCT kZahlungsart FROM tversandartzahlungsart WHERE kVersandart= :kVersandart",
            ['kVersandart' => $aInfo['kShipping']],
            ReturnType::ARRAY_OF_OBJECTS
          );
          foreach ($aRes as $value) {
            $checkPossible[] = $value->kZahlungsart;
          }
          if ($this->aTeinstellungenFields['s360_amazonpay_shop5'] && $this->aTeinstellungenFields['s360_amazonpay_shop5active']) {
            $add_amazon_payments = false;
          }
        }
      }
    }
    
    $Zahlungsarten = $this->db->queryPrepared(
      "SELECT * FROM tzahlungsart WHERE (cKundengruppen IS NULL OR cKundengruppen='' OR cKundengruppen LIKE :kKundengruppe) AND tzahlungsart.nActive = 1 AND tzahlungsart.nNutzbar = 1
       ORDER BY tzahlungsart.nSort",
      ['kKundengruppe' => '%'.$kKundengruppe.';%'],
      ReturnType::ARRAY_OF_OBJECTS
    );
    for ($i=0;$i<count($Zahlungsarten);$i++) {
      if(!$Zahlungsarten[$i]->kZahlungsart) {
        continue;
      }
      if (!in_array($Zahlungsarten[$i]->kZahlungsart,$checkPossible)) {
        continue;
      }
      $tmp = explode('_', $Zahlungsarten[$i]->cModulId);
      
      
      $kPlugin = PluginHelper::getIDByModuleID($Zahlungsarten[$i]->cModulId);
      if ($kPlugin == 0) {
        $tmp = explode('_',$Zahlungsarten[$i]->cModulId);
        $aCheckDir = array('heidelpay','paypal','ipayment','worldpay','sofortueberweisung','wirecard','postfinance','paymentpartner','saferpay','clickandbuy','mbqc','ut','billpay');
        if (in_array($tmp[1], $aCheckDir)) {
          if (!is_dir(PFAD_ROOT . PFAD_INCLUDES_MODULES . $tmp[1])) {
            continue;
          }
        }
      }
      $einstellungen = $this->db->queryPrepared(
        "SELECT cName,cWert FROM teinstellungen WHERE kEinstellungenSektion=".CONF_ZAHLUNGSARTEN." and cModulId= :cModulId",
        ['cModulId' => $Zahlungsarten[$i]->cModulId],
        ReturnType::ARRAY_OF_OBJECTS
      );
      foreach ($einstellungen as $einstellung) {
        $Zahlungsarten[$i]->einstellungen[$einstellung->cName] = $einstellung->cWert;
      }
      
      //Einstellungen beachten
      if (substr($Zahlungsarten[$i]->cModulId, 0, 8) != 'kPlugin_') {
        if (!PaymentMethod::shippingMethodWithValidPaymentMethod($Zahlungsarten[$i])) {
          continue;
        }
      }
      $pay = new ShopgatePaymentMethod();
      $pay -> setId($Zahlungsarten[$i]->kZahlungsart.'_'.$this->_prepare_utf8($Zahlungsarten[$i]->cName));
      $aPaymentMethods[] = $pay;
    }
    // Amazon-Payments
    if ($this->aTeinstellungenFields['s360_amazonpay_shop5'] && $this->aTeinstellungenFields['s360_amazonpay_shop5_active'] && $add_amazon_payments) {
      $pay = new ShopgatePaymentMethod();
      $pay -> setId('s360_mws');
      $aPaymentMethods[] = $pay;
    }
    /* EOF Zahlungsarten */
    $payment_method = $shopgateCart->getPaymentMethod();
    
    // BOF CSS-Gutscheine 8/8
    if(isset($wertgutscheine)) {
      // berechne volle gutscheinsumme
      $wertgutschein_summe = 0;
      foreach($wertgutscheine as $wertgutschein) {
        if($wertgutschein['active']) 
          $wertgutschein_summe += $wertgutschein['rest'];
      }
      // Wenn Betrag 0, dann nur Vorkasse erlauben
      $shipping_amount = $shopgateCart->getAmountShipping();
      if($wertgutschein_warenkorbwert <= $wertgutschein_summe && $shipping_amount == 0) {
        $wertgutschein_payment = '';
        foreach($aPaymentMethods as $tmp) {
          $tmp_id = $tmp->getId();
          if(empty($wertgutschein_payment))
            $wertgutschein_payment = $tmp_id;
          if(substr($tmp_id,0,2) == '1_') {
            $wertgutschein_payment = $tmp_id;
            break;
          }
        }
        if(!empty($wertgutschein_payment)) {
          $aPaymentMethods = array();
          $pay = new ShopgatePaymentMethod();
          $pay -> setId($wertgutschein_payment);
          $aPaymentMethods[] = $pay;
        }
        $payment_method = 'PREPAY';
      }
      $wertgutschein_verwendet = 0;
      $wertgutschein_fully_booked = false;
      foreach($wertgutscheine as $wertgutschein) {
        if($wertgutschein['active']) {
          $wertgutschein_verwendet += $wertgutschein['rest'];
          $kupon = new ShopgateExternalCoupon();
          $kupon->setIsValid(1);
          $kupon->setCode($wertgutschein['code']);
          $kupon->setName('Gutscheinverrechnung');
          if(!$wertgutschein_fully_booked) {
            //if($wertgutschein_warenkorbwert <= $wertgutschein['rest'] && $shipping_amount > 0)
              $kupon->setDescription('Aus technischen Grnden kann der Gutschein nur auf den Warenwert angerechnet werden.');
            if($wertgutschein_warenkorbwert <= $wertgutschein_verwendet) {
              $kupon->setAmountNet(round($wertgutschein_verwendet-$wertgutschein_warenkorbwert,2));
              $wertgutschein_fully_booked = true;
            }
            else
              $kupon->setAmountNet(round($wertgutschein['rest'],2));
          }
          else {
            $kupon->setAmountNet(0);
            $kupon->setIsValid(0);
            $kupon->setNotValidMessage($this->_pss('Das Guthaben zus&auml;tzlicher Gutscheine &uuml;bersteigt den Einkaufswert. Weitere Gutscheine k&ouml;nnen nicht eingel&ouml;st werden.'));
          }
          $kupon->setTaxType('auto');
          $kupon->setIsFreeShipping(0);
          $aInfo = array('css_gutschein'=>$wertgutschein['code']);
          $kupon->setInternalInfo($this->jsonEncode($aInfo));
          $aCouponsResponse[] = $kupon;
        }
        elseif($wertgutschein['rest'] <= 0) {
          $kupon = new ShopgateExternalCoupon();
          $kupon->setIsValid(0);
          $kupon->setCode($wertgutschein['code']);
          $kupon->setName('Gutscheinverrechnung');
          $kupon->setDescription('');
          $kupon->setAmountNet(0);
          $kupon->setNotValidMessage($this->_pss('Das Gutschein-Guthaben ist aufgebraucht.'));
          $kupon->setTaxType('auto');
          $kupon->setIsFreeShipping(0);
          $kupon->setInternalInfo('');
          $aCouponsResponse[] = $kupon;
        }
      }
    }
    // EOF CSS-Gutscheine 8/8
    
    $payment_group = $shopgateCart->getPaymentGroup();
    if (!empty($payment_method)) {
      if (!empty($this->JTLPluginConf['cZahlungsartenZuordnen'])) {
        $shopgate_payments = $this->JTLPluginConf['cZahlungsartenZuordnen'];
        
        if(array_key_exists($payment_method,$shopgate_payments) && $shopgate_payments[$payment_method]['id']>0) {
          $tmpShipping = array();
          $aRes = $this->db->queryPrepared(
            "SELECT DISTINCT kVersandart FROM tversandartzahlungsart WHERE kZahlungsart= :kZahlungsart",
            ['kZahlungsart' => $shopgate_payments[$payment_method]['id']],
            ReturnType::ARRAY_OF_OBJECTS
          );
          foreach ($aRes as $value) {
            $tmpShipping[] = $value->kVersandart;
          }
          $aNewShippingMethods = array();
          foreach ($aShippingMethods as $oShipping)  {
            if (in_array($oShipping->getId(), $tmpShipping)) {
              $aNewShippingMethods[] = $oShipping;
            }
          }
          $aShippingMethods = $aNewShippingMethods;
        }
      }
    }
    
    $response = array(
      'currency'  => $this->oWaehrung->cISO,
      'customer'  => $aCustomerResponse,
      'external_coupons'  => $aCouponsResponse,
      'shipping_methods'  => $aShippingMethods,
      'items'     => $aItemResponse
    );
    
    $response['payment_methods'] = $aPaymentMethods;
    if (method_exists($this, 'agi_shopgate_connect__checkCartcheckPayment')) {
      $response = $this->agi_shopgate_connect__checkCartcheckPayment($response, $shopgateCart);
    }
    
    return $response;
  }

  /**
    * Performs the necessary queries to add an order to the shop system's database.
    *
    * @see http://developer.shopgate.com/merchant_api/orders/get_orders
    * @see http://developer.shopgate.com/plugin_api/orders/add_order
    *
    * @param ShopgateOrder $order The ShopgateOrder object to be added to the shop system's database.
    *
    * @return array(
    *		'external_order_id' => string, # the ID of the order in your shop system's database
    *		'external_order_number' => string, # the number of the order in your shop system
    * )
    * @throws ShopgateLibraryException if an error occurs.
    */
  public function addOrder(ShopgateOrder $order)
  {
    if (method_exists($this, 'agi_shopgate_connect__addOrder')) {
      $this->agi_shopgate_connect__addOrder($order);
    }

    $Einstellungen = Shop::getSettings(array(CONF_GLOBAL));
    $additional_comments = array();
    
    // Check if an order with the number ($order->getOrderNumber())
    // has already been imported from Shopgate
    $aRes = $this->db->queryPrepared(
      "SELECT COUNT(*) AS cnt FROM xplugin_agi_shopgate_connect_orders WHERE kBestellungShopgate= :kBestellungShopgate",
      ['kBestellungShopgate' => $order->getOrderNumber()],
      ReturnType::SINGLE_OBJECT
    );
    $dbOrder = $aRes->cnt;
    // If an order is found with the order number ($order->getOrderNumber()) 
    // abort:
    if (!empty($dbOrder)) {
      throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_DUPLICATE_ORDER);
    }
    
    $cryptoService = Shop::Container()->getCryptoService();
    
    $cMail = $this->_prepare_utf8($order->getMail(), false);
    $aRes = $this->db->queryPrepared(
      "SELECT kKunde FROM tkunde WHERE cMail= :cMail AND nRegistriert=1",
      ['cMail' => $cMail],
      ReturnType::SINGLE_OBJECT
    );
    $invoice = $order->getInvoiceAddress();
    if (empty($aRes)) {
      $JTL_Client = new Customer();
      switch($invoice->getGender()) {
        case 'f': $JTL_Client -> cAnrede  = 'w'; break;
        case 'm': $JTL_Client -> cAnrede  = 'm'; break;
        default: $JTL_Client -> cAnrede  = '';
      }
      
      $JTL_Client -> cVorname = $this->_prepare_utf8($invoice->getFirstName(), false);
      $JTL_Client -> cNachname= $this->_prepare_utf8($invoice->getLastName(), false);
      $JTL_Client -> cFirma   = $this->_prepare_utf8($invoice->getCompany(), false);	
      $JTL_Client -> cStrasse = $this->_prepare_utf8($invoice->getStreetName1(), false);
      $JTL_Client -> cHausnummer  = $this->_prepare_utf8($invoice->getStreetNumber1(), false);
      $JTL_Client -> cAdressZusatz= $this->_prepare_utf8($invoice->getStreet2(), false); // 1.6
      $JTL_Client -> cPLZ     = $this->_prepare_utf8($invoice->getZipcode(), false);
      $JTL_Client -> cOrt     = $this->_prepare_utf8($invoice->getCity(), false);
      //$JTL_Client -> cBundesland  = $this->_prepare_utf8($invoice->getState());
      $JTL_Client -> cLand    = $this->_prepare_utf8($invoice->getCountry(), false);
      $JTL_Client -> cTel     = $this->_prepare_utf8($order->getPhone(), false);
      $JTL_Client -> cMobil   = $this->_prepare_utf8($order->getMobile(), false);
      //$JTL_Client -> cFax     = $invoice->();
      $JTL_Client -> cMail    = $this->_prepare_utf8($order->getMail(), false);
      //$JTL_Client -> cUSTID   = $invoice->();
      //$JTL_Client -> cWWW     = $invoice->();
      $JTL_Client -> nRegistriert = 0;
      $JTL_Client -> kKundengruppe= (int)$this->kKundengruppe;
      $JTL_Client -> kSprache   = $this->kSprache;
      $JTL_Client -> dErstellt = date('Y-m-d');
      $JTL_Client -> cAktiv    = 'Y';
      $JTL_Client -> cAbgeholt = 'N';
      $JTL_Client -> cHerkunft = 'Shopgate';
      $JTL_Client -> insertInDB();
    }
    else {
      $JTL_Client = new Customer((int)$aRes->kKunde);
    }
    
    // create invoice address
    $JTL_Invoice = new Rechnungsadresse();
    $JTL_Invoice->kKunde = $JTL_Client->kKunde;
    switch($invoice->getGender()) {
      case 'f': $JTL_Invoice -> cAnrede  = 'w'; break;
      case 'm': $JTL_Invoice -> cAnrede  = 'm'; break;
      default: $JTL_Invoice -> cAnrede  = '';
    }
    $JTL_Invoice->cVorname    = $this->_prepare_utf8($invoice->getFirstName(), false);
    $JTL_Invoice->cNachname   = $this->_prepare_utf8($invoice->getLastName(), false);
    $JTL_Invoice->cFirma      = $this->_prepare_utf8($invoice->getCompany(), false);
    $JTL_Invoice->cStrasse    = $this->_prepare_utf8($invoice->getStreetName1(), false);
    $JTL_Invoice->cHausnummer = $this->_prepare_utf8($invoice->getStreetNumber1(), false);
    $JTL_Invoice->cAdressZusatz= $this->_prepare_utf8($invoice->getStreet2(), false);
    $JTL_Invoice->cPLZ        = $this->_prepare_utf8($invoice->getZipcode(), false);
    $JTL_Invoice->cOrt        = $this->_prepare_utf8($invoice->getCity(), false);
    //$JTL_Invoice->cBundesland = $this->_prepare_utf8($invoice->getState(), false);
    $JTL_Invoice->cLand       = $this->_prepare_utf8($invoice->getCountry(), false);
    $JTL_Invoice->cTel        = $this->_prepare_utf8($invoice->getPhone(), false);
    $JTL_Invoice->cMobil      = $this->_prepare_utf8($invoice->getMobile(), false);
    $JTL_Invoice->cMail       = $this->_prepare_utf8($invoice->getMail(), false);
    //$JTL_Invoice-> insertInDB(); // don't know, why it does not work
    
    $inv_address = new stdClass();
    $inv_address->kKunde    = $JTL_Client->kKunde;
    $inv_address->cAnrede   = $JTL_Invoice -> cAnrede;
    $inv_address->cVorname  = $JTL_Invoice -> cVorname;
    $inv_address->cNachname = $cryptoService->encryptXTEA(trim($JTL_Invoice -> cNachname));
    $inv_address->cFirma    = $cryptoService->encryptXTEA(trim($JTL_Invoice -> cFirma));
    $inv_address->cStrasse  = $cryptoService->encryptXTEA(trim($JTL_Invoice -> cStrasse));
    $inv_address->cHausnummer = $JTL_Invoice -> cHausnummer;
    $inv_address->cAdressZusatz = $JTL_Invoice -> cAdressZusatz;
    $inv_address->cPLZ      = $JTL_Invoice -> cPLZ;
    $inv_address->cOrt      = $JTL_Invoice -> cOrt;
    $inv_address->cLand     = $JTL_Invoice -> cLand;
    $inv_address->cTel      = $JTL_Invoice -> cTel;
    $inv_address->cMobil    = $JTL_Invoice -> cMobil;
    $inv_address->cMail     = empty($JTL_Invoice -> cMail) ? $JTL_Client -> cMail : $JTL_Invoice -> cMail;
    $JTL_Invoice->kRechnungsadresse = $this->db->insertRow("trechnungsadresse", $inv_address);
    
    
    // check delivery-address
    $equal_address = true;
    $delivery = $order->getDeliveryAddress();
    $JTL_Delivery = new Lieferadresse();
    $JTL_Delivery->kKunde = $JTL_Client->kKunde;
    switch($delivery->getGender()) {
      case 'f': $JTL_Delivery -> cAnrede  = 'w'; break;
      case 'm': $JTL_Delivery -> cAnrede  = 'm'; break;
      default: $JTL_Delivery -> cAnrede  = '';
    }
    $JTL_Delivery->cVorname    = $this->_prepare_utf8($delivery->getFirstName(), false);
    $JTL_Delivery->cNachname   = $this->_prepare_utf8($delivery->getLastName(), false);
    $JTL_Delivery->cFirma      = $this->_prepare_utf8($delivery->getCompany(), false);
    $JTL_Delivery->cStrasse    = $this->_prepare_utf8($delivery->getStreetName1(), false);
    $JTL_Delivery->cHausnummer = $this->_prepare_utf8($delivery->getStreetNumber1(), false);
    $JTL_Delivery->cAdressZusatz = $this->_prepare_utf8($delivery->getStreet2(), false);
    $JTL_Delivery->cPLZ        = $this->_prepare_utf8($delivery->getZipcode(), false);
    $JTL_Delivery->cOrt        = $this->_prepare_utf8($delivery->getCity(), false);
    //$JTL_Delivery->cBundesland = $this->_prepare_utf8($delivery->getState());
    $JTL_Delivery->cLand       = $this->_prepare_utf8($delivery->getCountry(), false);
    $JTL_Delivery->cTel        = $this->_prepare_utf8($delivery->getPhone(), false);
    $JTL_Delivery->cMobil      = $this->_prepare_utf8($delivery->getMobile(), false);
    $JTL_Delivery->cMail       = $this->_prepare_utf8($delivery->getMail(), false);
    if (empty($JTL_Delivery->cMail)) {
      $JTL_Delivery->cMail = $JTL_Client->cMail;
    }
    if (
      $JTL_Delivery->cVorname != $JTL_Client->cVorname ||
      $JTL_Delivery->cNachname != $JTL_Client->cNachname ||
      $JTL_Delivery->cStrasse != $JTL_Client->cStrasse ||
      $JTL_Delivery->cPLZ != $JTL_Client->cPLZ ||
      $JTL_Delivery->cOrt != $JTL_Client->cOrt ||
      $JTL_Delivery->cMail != $JTL_Client->cMail
    )
    {
      $equal_address = false;
      //$JTL_Delivery-> insertInDB(); same like saving invoice-Address
      $del_address = new stdClass();
      $del_address->kKunde    = $JTL_Client->kKunde;
      $del_address->cAnrede   = $JTL_Delivery -> cAnrede;
      $del_address->cVorname  = $JTL_Delivery -> cVorname;
      $del_address->cNachname = $cryptoService->encryptXTEA(trim($JTL_Delivery -> cNachname));
      $del_address->cFirma    = $cryptoService->encryptXTEA(trim($JTL_Delivery -> cFirma));
      $del_address->cStrasse  = $cryptoService->encryptXTEA(trim($JTL_Delivery -> cStrasse));
      $del_address->cHausnummer = $JTL_Delivery -> cHausnummer;
      $del_address->cAdressZusatz = $JTL_Delivery -> cAdressZusatz;
      $del_address->cPLZ      = $JTL_Delivery -> cPLZ;
      $del_address->cOrt      = $JTL_Delivery -> cOrt;
      $del_address->cLand     = $JTL_Delivery -> cLand;
      $del_address->cTel      = $JTL_Delivery -> cTel;
      $del_address->cMobil    = $JTL_Delivery -> cMobil;
      $del_address->cMail     = $JTL_Delivery -> cMail;
      $JTL_Delivery->kLieferadresse = $this->db->insertRow("tlieferadresse", $del_address);
    }
    
    // look for kWaehrung
    $aRes = $this->db->queryPrepared(
      "SELECT kWaehrung FROM twaehrung WHERE cISO= :cISO",
      ['cISO' => $order->getCurrency()],
      ReturnType::SINGLE_OBJECT
    );
    $kWaehrung = $aRes->kWaehrung;
    if (empty($kWaehrung)) {
      $aRes = $this->db->queryPrepared(
        "SELECT kWaehrung FROM twaehrung WHERE cStandard='Y'",
        [],
        ReturnType::SINGLE_OBJECT
      );
      $kWaehrung = $aRes->kWaehrung;
      $additional_comments[] = 'Whrung '.$order->getCurrency().' nicht korrekt zugeordnet, bitte prfen.';
    }
    
    $string_comments = '';
    if (!empty($additional_comments)) {
      $string_comments = chr(10).implode(chr(10), $additional_comments);
    }
    
    // create basket
    $basket = new stdClass();
    $basket->kKunde = $JTL_Client->kKunde;
    $basket->kLieferadresse = 0;
    $basket->kZahlungsInfo  = 0;
    $kWarenkorb = $this->db->insertRow("twarenkorb", $basket);
    $oWarenkorb = new Cart($kWarenkorb);
    
    // get coupons to check for rpo
    $aKupons = $order->getExternalCoupons();
    $direct_coupon = array(
      'sum'     => 0,
      'percent' => 0,
      'name'    => ''
    );
    
    if (!empty($aKupons)) {
      foreach ($aKupons as $oKupon) {
        $cInfo = $oKupon->getInternalInfo();
        if (!empty($cInfo)) {
          $aInfo = $this->jsonDecode($cInfo, true);
          if (array_key_exists('rpo',$aInfo)) {
            $direct_coupon = array(
              'sum'     => 0,
              'percent' => $aInfo['rpo']['percent'],
              'name'    => $aInfo['rpo']['name']
            );
          }
        }
      }
    }
    
    // add items to basket
    $oItems = $order->getItems();
    $verfuegbarkeitResult_arr = array('cArtikelName_arr' => array());
    $aWarenkorbVersandklassen = array();
    $longestDelivery = array('nLongestMinDelivery' => 0, 'nLongestMaxDelivery' => 0);
    foreach($oItems as $orderItem) {
      if ($orderItem->getType() != $orderItem::TYPE_PRODUCT && $orderItem->getType() != $orderItem::TYPE_ITEM) {
        $calcprice = round($orderItem->getUnitAmountWithTax()*100/(100+$orderItem->getTaxPercent()),5);
      
        $warenkorbpos = new stdClass();
        $warenkorbpos->kWarenkorb      = $kWarenkorb;
        $warenkorbpos->kArtikel        = 0;
        $warenkorbpos->kVersandklasse  = 0;
        $warenkorbpos->cName           = $this->_prepare_utf8($orderItem->getName(), false);
        $warenkorbpos->cLieferstatus   = '';
        $warenkorbpos->cArtNr          = $this->_prepare_utf8($orderItem->getItemNumberPublic(), false);
        $warenkorbpos->cEinheit        = '';
        $warenkorbpos->fPreisEinzelNetto = $calcprice;
        $warenkorbpos->fPreis          = $calcprice;
        $warenkorbpos->fMwSt           = $orderItem->getTaxPercent();
        $warenkorbpos->nAnzahl         = 1;
        $warenkorbpos->nPosTyp         = $orderItem->getType() != $orderItem::TYPE_SHOPGATE_COUPON ? C_WARENKORBPOS_TYP_ZAHLUNGSART : C_WARENKORBPOS_TYP_KUPON;
        $warenkorbpos->cHinweis        = '';
        $warenkorbpos->cUnique         = '';
        $warenkorbpos->kKonfigitem     = 0;
        $warenkorbpos->kBestellpos     = 0;
        $kWarenkorbPos = $this->db->insertRow("twarenkorbpos", $warenkorbpos);
        continue;
      }
      
      $set_article_hint = '';
      // try to load article
      $oArtikelOptionen = new stdClass();
      $oArtikelOptionen->nMerkmale = 1;
      $oArtikelOptionen->nAttribute = 1;
      $oArtikelOptionen->nArtikelAttribute = 1;
      $oArtikelOptionen->nKategorie = 0;
      $oArtikelOptionen->nKeinLagerbestandBeachten = 1;//??
      $oArtikelOptionen->nVariationen = 1;
      
      $article = new Artikel();
      $article->fuelleArtikel(
        $orderItem->getItemNumber(),
        $oArtikelOptionen,
        $JTL_Client -> kKundengruppe,
        $JTL_Client -> kSprache
      );
      
      $calcprice = round($orderItem->getUnitAmountWithTax()*100/(100+$orderItem->getTaxPercent()),5);
      $attributeSum = 0;
      $options = $orderItem->getOptions();
      if (!empty($options)) {
        $attributeSum = $this->_get_optionsPriceNetto($options, $orderItem->getTaxPercent());
      }
      $calcpriceEinzel = $calcprice-$attributeSum;
      /*
      if ($direct_coupon['percent'] > 0) {
        $calcpriceEinzel-= round($calcpriceEinzel*$direct_coupon['percent']/100,2);
        $diff = round($calcprice*$direct_coupon['percent']/100,2);
        $calcprice -= $diff;
        $direct_coupon['sum']+=($diff*(($orderItem->getTaxPercent()+100)/100))*$orderItem->getQuantity();
        $set_article_hint = $direct_coupon['name'];
      }
      */
      /* BOF Konfiguration */
      $cUnique = '';
      $tmpOptions = array();
      $tmpKonfigurationArticle = array();
      if (!empty($options)) {
        foreach ($options as $checkOption) {
          $option_number = $checkOption->getOptionNumber();
          if (substr($option_number,0,10) == 'agiKonfig_') {
            $tmpKonfigurationArticle[] = $checkOption;
          }
          else {
            $tmpOptions[] = $checkOption;
          }
        }
      }
      $options = $tmpOptions;
      if ($this->bAllowKonfiguration) {
        if (!empty($tmpKonfigurationArticle)) {
          $cUnique = $this->_gibUID(10);
          $iCountKonfig = 0;
          foreach ($tmpKonfigurationArticle as $oKonfigurationOption) {
            $option_number = $oKonfigurationOption->getOptionNumber();
            $value_number = $oKonfigurationOption->getValueNumber();
            if ($value_number) {
              //array(kKonfiggruppe ,kKonfigitem, kArtikel);
              $aKonfigKeys = explode('_', $value_number);
              if (count($aKonfigKeys) == 4) {
                $kItem = new Konfigitem($aKonfigKeys[1]);
                if ($kItem->getKonfiggruppe() == $aKonfigKeys[0] && !empty($aKonfigKeys[2]) && $kItem->getArtikelKey() == $aKonfigKeys[2]) {
                  $tmpArticle = $kItem->getArtikel();
                  $fKItemCalcprice = round($oKonfigurationOption->getAdditionalAmountWithTax()*100/(100+$tmpArticle->fMwSt),5);
                  /*
                  if ($direct_coupon['percent'] > 0) {
                    $fKItemCalcprice -= round($fKItemCalcprice*$direct_coupon['percent']/100,5);
                  }
                  */
                  $warenkorbpos = new stdClass();
                  $warenkorbpos->kWarenkorb      = $kWarenkorb;
                  $warenkorbpos->kArtikel        = $kItem->getArtikelKey();
                  $warenkorbpos->kVersandklasse  = $tmpArticle->kVersandklasse;
                  $warenkorbpos->cName           = $kItem->getName(); //$this->_prepare_utf8($oKonfigurationOption->getValue(), false);
                  $warenkorbpos->cLieferstatus   = '';
                  $warenkorbpos->cArtNr          = $tmpArticle->cArtNr;
                  $warenkorbpos->cEinheit        = $tmpArticle->cEinheit;
                  $warenkorbpos->fPreisEinzelNetto = $fKItemCalcprice/$aKonfigKeys[3];
                  $warenkorbpos->fPreis          = $fKItemCalcprice/$aKonfigKeys[3];
                  $warenkorbpos->fMwSt           = $tmpArticle->fMwSt;
                  $warenkorbpos->nAnzahl         = $aKonfigKeys[3] > 0 ? $orderItem->getQuantity() * $aKonfigKeys[3] : $orderItem->getQuantity();
                  $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_ARTIKEL;
                  $warenkorbpos->cHinweis        = '';
                  $warenkorbpos->cUnique         = $cUnique;
                  $warenkorbpos->kKonfigitem     = $aKonfigKeys[1];
                  $warenkorbpos->kBestellpos     = 0;
                  $kWarenkorbPos = $this->db->insertRow("twarenkorbpos", $warenkorbpos);
                  //$calcpriceEinzel -= $fKItemCalcprice;
                  $calcprice -= $fKItemCalcprice;
                  $iCountKonfig++;
                }
              }
            }
          }
        }
      }
      /* EOF Konfiguration */
      
      $article->getDeliveryTime($JTL_Delivery->cLand, $warenkorbpos->nAnzahl);
      if ($article->nMinDeliveryDays > $longestDelivery['nLongestMinDelivery']) {
        $longestDelivery['nLongestMinDelivery'] = $article->nMinDeliveryDays;
      }
      if ($article->nMaxDeliveryDays > $longestDelivery['nLongestMaxDelivery']) {
        $longestDelivery['nLongestMaxDelivery'] = $article->nMaxDeliveryDays;
      }
      
      $warenkorbpos = new stdClass();
      $warenkorbpos->kWarenkorb      = $kWarenkorb;
      $warenkorbpos->kArtikel        = $orderItem->getItemNumber();
      $warenkorbpos->kVersandklasse  = $article->kVersandklasse;
      $warenkorbpos->cName           = $this->_prepare_utf8($orderItem->getName(), false);
      $warenkorbpos->cLieferstatus   = $article->cLieferstatus;
      $warenkorbpos->cArtNr          = $this->_prepare_utf8($orderItem->getItemNumberPublic(), false);
      $warenkorbpos->cEinheit        = $article->cEinheit;
      $warenkorbpos->fPreisEinzelNetto = $calcpriceEinzel;
      $warenkorbpos->fPreis          = $calcprice;
      $warenkorbpos->fMwSt           = $orderItem->getTaxPercent();
      $warenkorbpos->nAnzahl         = $orderItem->getQuantity();
      $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_ARTIKEL;
      $warenkorbpos->cHinweis        = $set_article_hint;
      $warenkorbpos->cUnique         = $cUnique;
      $warenkorbpos->kKonfigitem     = 0;
      $warenkorbpos->kBestellpos     = 0;
      $warenkorbpos->nLongestMinDelivery = $article->nMinDeliveryDays;
      $warenkorbpos->nLongestMaxDelivery = $article->nMaxDeliveryDays;
      $kWarenkorbPos = $this->db->insertRow("twarenkorbpos", $warenkorbpos);
      
      if ($article->cLagerBeachten == "Y" && $article->cLagerKleinerNull == "Y" && $Einstellungen['global']['global_lieferverzoegerung_anzeigen'] === "Y") {
        if ($warenkorbpos->nAnzahl > $article->fLagerbestand) {
          $verfuegbarkeitResult_arr['cArtikelName_arr'][] = $article->cName;
        }
      }
      
      // Eigenschaften?
      $wpEigenschaftArr = array();
			if(!empty($options)) {
        foreach($options as $option) {
          // search for attribute in Database
          $kEigenschaft = 0;
          $kEigenschaftWert = 0;
          $aRes = $this->db->queryPrepared(
            "SELECT kEigenschaft FROM teigenschaft WHERE kArtikel= :kArtikel AND kEigenschaft= :kEigenschaft",
            ['kArtikel' => $orderItem->getItemNumber(), 'kEigenschaft' => $option->getOptionNumber()],
            ReturnType::SINGLE_OBJECT
          );
          if (!empty($aRes->kEigenschaft)) {
            $kEigenschaft = $aRes->kEigenschaft;
            $aRes = $this->db->queryPrepared(
              "SELECT kEigenschaftWert FROM teigenschaftwert WHERE kEigenschaft= :kEigenschaft AND kEigenschaftWert= :kEigenschaftWert",
              ['kEigenschaft' => $kEigenschaft, 'kEigenschaftWert' => $option->getValueNumber()],
              ReturnType::SINGLE_OBJECT
            );
            if (!empty($aRes->kEigenschaftWert)) {
              $kEigenschaftWert = $aRes->kEigenschaftWert;
            }
          }
          
          $wpEigenschaft = new stdClass();
          $wpEigenschaft->kWarenkorbPos      = (int)$kWarenkorbPos;
          $wpEigenschaft->kEigenschaft       = (int)$kEigenschaft;
          $wpEigenschaft->kEigenschaftWert   = (int)$kEigenschaftWert;
          $wpEigenschaft->cEigenschaftName   = $this->_prepare_utf8($option->getName(), false);
          $wpEigenschaft->cEigenschaftWertName = $this->_prepare_utf8($option->getValue(), false);
          $wpEigenschaft->cFreifeldWert      = $this->_prepare_utf8($option->getValue(), false);
          $wpEigenschaft->fAufpreis          = round($option->getAdditionalAmountWithTax()*100/(100+$orderItem->getTaxPercent()), 4);
          /*
          if ($direct_coupon['percent'] > 0) {
            $wpEigenschaft->fAufpreis -= round($wpEigenschaft->fAufpreis*$direct_coupon['percent']/100,4);
          }
          */
          $kWarenkorbPosEigenschaft = $this->db->insertRow("twarenkorbposeigenschaft", $wpEigenschaft);
          $wpEigenschaftArr[] = $wpEigenschaft;
        }
      }
      
      $inputs = $orderItem->getInputs();
      if (!empty($inputs)) {
        foreach($inputs as $inputOption) {
          // search for attribute in Database
          $kEigenschaft = 0;
          $kEigenschaftWert = 0;
          $aRes = $this->db->queryPrepared(
            "SELECT kEigenschaft FROM teigenschaft WHERE kArtikel= :kArtikel AND kEigenschaft= :kEigenschaft",
            ['kArtikel' => $orderItem->getItemNumber(), 'kEigenschaft' => $inputOption->getInputNumber()],
            ReturnType::SINGLE_OBJECT
          );
          
          if (!empty($aRes->kEigenschaft)) {
            $kEigenschaft = $aRes->kEigenschaft;
            $kEigenschaftWert = 0;
          }
          
          $wpEigenschaft = new stdClass();
          $wpEigenschaft->kWarenkorbPos      = (int)$kWarenkorbPos;
          $wpEigenschaft->kEigenschaft       = (int)$kEigenschaft;
          $wpEigenschaft->kEigenschaftWert   = (int)$kEigenschaftWert;
          $wpEigenschaft->cEigenschaftName   = $this->_prepare_utf8($inputOption->getLabel(), false);
          $wpEigenschaft->cEigenschaftWertName = $this->_prepare_utf8($inputOption->getUserInput(), false);
          $wpEigenschaft->cFreifeldWert      = $this->_prepare_utf8($inputOption->getUserInput(), false);
          $wpEigenschaft->fAufpreis          = round($inputOption->getAdditionalAmountWithTax()*100/(100+$orderItem->getTaxPercent()), 4);
          /*
          if ($direct_coupon['percent'] > 0) {
            $wpEigenschaft->fAufpreis -= round($wpEigenschaft->fAufpreis*$direct_coupon['percent']/100,4);
          }
          */
          $kWarenkorbPosEigenschaft = $this->db->insertRow("twarenkorbposeigenschaft", $wpEigenschaft);
          $wpEigenschaftArr[] = $wpEigenschaft;
        }
      }
    
      $stockUpdater = new StockUpdater($this->db, $JTL_Client, $oWarenkorb);
      $stockUpdater->updateStock($article, $warenkorbpos->nAnzahl, $wpEigenschaftArr);
      $stockUpdater->updateBestsellers((int)$warenkorbpos->kArtikel, $warenkorbpos->nAnzahl);
      if (!in_array($article->kVersandklasse, $aWarenkorbVersandklassen)) {
        $aWarenkorbVersandklassen[] = $article->kVersandklasse;
      }
    }
    if (count($verfuegbarkeitResult_arr['cArtikelName_arr']) > 0) {
      $cHinweis = str_replace("%s", "", Shop::Lang()->get('orderExpandInventory', 'basket'));
      $verfuegbarkeitResult_arr['cHinweis'] = $cHinweis;
    }
    
    // ermittle Steuersatz
    $fSteuersatz = $this->fallback_steuersatz;
    $aRes = $this->db->queryPrepared(
      "SELECT kSteuerzone FROM tsteuerzoneland WHERE cISO= :cISO",
      ['cISO' => $JTL_Delivery->cLand],
      ReturnType::SINGLE_OBJECT
    );
    if (!empty($aRes)) {
      $kSteuerzone = $aRes->kSteuerzone;
      $aRes = $this->db->queryPrepared(
        "SELECT tsteuersatz.fSteuersatz FROM tsteuersatz, tsteuerklasse WHERE tsteuersatz.kSteuerklasse=tsteuerklasse.kSteuerklasse AND tsteuerklasse.cStandard='Y' AND tsteuersatz.kSteuerzone= :kSteuerzone",
        ['kSteuerzone' => $kSteuerzone],
        ReturnType::SINGLE_OBJECT
      );
      $fSteuersatz = $aRes->fSteuersatz;
    }
    
    // Kupons
    $aKupons = $order->getExternalCoupons();
    $addKuponBestellung = array();
    if (!empty($aKupons)) {
      foreach ($aKupons as $oKupon) {
        $cInfo = $oKupon->getInternalInfo();
        if (!empty($cInfo)) {
          $aInfo = $this->jsonDecode($cInfo, true);
          $kKupon = array_key_exists('kKupon', $aInfo) ? $aInfo['kKupon'] : 0;
          /*
          if (array_key_exists('rpo',$aInfo)) {
            continue; // rabatt on products, dont at coupon to basket
          }
          */
          if (!empty($kKupon)) {
            $fAmount = $oKupon->getAmount();
            $fAmountNet = round($fAmount*100/(100+$fSteuersatz),4);
            $warenkorbpos = new stdClass();
            $warenkorbpos->kWarenkorb      = $kWarenkorb;
            $warenkorbpos->kArtikel        = 0;
            $warenkorbpos->kVersandklasse  = 0;
            $warenkorbpos->cName           = $this->_prepare_utf8($oKupon->getName(), false);
            $warenkorbpos->cLieferstatus   = '';
            $warenkorbpos->cArtNr          = '';
            $warenkorbpos->cEinheit        = '';
            $warenkorbpos->fPreisEinzelNetto = $fAmountNet*-1;
            $warenkorbpos->fPreis          = $fAmountNet*-1;
            $warenkorbpos->fMwSt           = $fSteuersatz;
            $warenkorbpos->nAnzahl         = 1;
            $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_KUPON;
            $warenkorbpos->cHinweis        = '';
            $warenkorbpos->cUnique         = '';
            $warenkorbpos->kKonfigitem     = 0;
            $warenkorbpos->kBestellpos     = 0;
            $kWarenkorbPos = $this->db->insertRow("twarenkorbpos", $warenkorbpos);
            
            $this->db->executeQueryPrepared(
              "UPDATE tkupon SET nVerwendungenBisher=nVerwendungenBisher+1 WHERE kKupon= :kKupon",
              ['kKupon' => $kKupon]
            );

            $KuponKundeBisher = $this->db->queryPrepared(
              "SELECT kKuponKunde,nVerwendungen FROM tkuponkunde WHERE kKupon= :kKupon AND cMail= :cMail",
              ['kKupon' => $kKupon, 'cMail' => Kupon::hash($JTL_Client -> cMail)],
              ReturnType::SINGLE_OBJECT
            );
            if ($KuponKundeBisher) {
              $this->db->updateRow("tkuponkunde", 'kKuponKunde', $KuponKundeBisher->kKuponKunde, (object)['nVerwendungen'=>$KuponKundeBisher->nVerwendungen+1]);
            } else {
              
              $KuponKunde = new stdClass();
              $KuponKunde->kKupon = $kKupon;
              $KuponKunde->cMail =  Kupon::hash($JTL_Client -> cMail);
              $KuponKunde->dErstellt = "now()";
              $KuponKunde->nVerwendungen=1;
              $this->db->insertRow("tkuponkunde", $KuponKunde);
            }
            $addKuponBestellung[$kKupon] = array(
              'amount'  => $oKupon->getAmount(),
              'oKupon'  => new Kupon($kKupon)
            );
          }
          elseif (array_key_exists('kundenguthaben', $aInfo) && $aInfo['kundenguthaben'] == $JTL_Client->kKunde) {
            $fAmount = $oKupon->getAmount();
            $fAmountNet = round($fAmount*100/(100+$fSteuersatz),4);
            $warenkorbpos = new stdClass();
            $warenkorbpos->kWarenkorb      = $kWarenkorb;
            $warenkorbpos->kArtikel        = 0;
            $warenkorbpos->kVersandklasse  = 0;
            $warenkorbpos->cName           = $this->_prepare_utf8($oKupon->getName(), false);
            $warenkorbpos->cLieferstatus   = '';
            $warenkorbpos->cArtNr          = '';
            $warenkorbpos->cEinheit        = '';
            $warenkorbpos->fPreisEinzelNetto = $fAmountNet*-1;
            $warenkorbpos->fPreis          = $fAmountNet*-1;
            $warenkorbpos->fMwSt           = $fSteuersatz;
            $warenkorbpos->nAnzahl         = 1;
            $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_ARTIKEL;
            $warenkorbpos->cHinweis        = '';
            $warenkorbpos->cUnique         = '';
            $warenkorbpos->kKonfigitem     = 0;
            $warenkorbpos->kBestellpos     = 0;
            $kWarenkorbPos = $this->db->insertRow("twarenkorbpos", $warenkorbpos);
            
            // lese Kundenguthaben und speichere neues Guthaben
            $fAmount *= 1;
            $this->db->executeQueryPrepared(
              "UPDATE tkunde SET fGuthaben=fGuthaben- :fGuthaben WHERE kKunde= :kKunde",
              ['fGuthaben' => $fAmount, 'kKunde' => $JTL_Client->kKunde]
            );
          }
          elseif(array_key_exists('css_gutschein', $aInfo) && $aInfo['css_gutschein'] == $oKupon->getCode()) {
            // BOF CSS-Gutscheine 2/8
            if(!isset($wertgutscheine))
              $wertgutscheine = array('used'=>array(),'fSummeGuthaben'=>0);
            
            $warenkorbpos = new stdClass();
            $warenkorbpos->kWarenkorb      = $kWarenkorb;
            $warenkorbpos->kArtikel        = 0;
            $warenkorbpos->kVersandklasse  = 0;
            $warenkorbpos->cName           = $this->_prepare_utf8('Gutschein: '.$aInfo['css_gutschein'].' '.abs($oKupon->getAmount()).' '.$this->oWaehrung->cISO, false);
            $warenkorbpos->cLieferstatus   = '';
            $warenkorbpos->cArtNr          = '';
            $warenkorbpos->cEinheit        = '';
            $warenkorbpos->fPreisEinzelNetto = 0;
            $warenkorbpos->fPreis          = 0;
            $warenkorbpos->fMwSt           = 0;
            $warenkorbpos->nAnzahl         = 1;
            $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_GUTSCHEIN;
            $warenkorbpos->cHinweis        = '';
            $warenkorbpos->cUnique         = '';
            $warenkorbpos->kKonfigitem     = 0;
            $warenkorbpos->kBestellpos     = 0;
            $kWarenkorbPos = $this->db->insertRow("twarenkorbpos", $warenkorbpos);
            
            $wertgutschein = array(
              'fGuthaben'  => $oKupon->getAmount(),
              'kGutschein' => 0,
              'pid'        => 0,
              'cMail'      => ''
            );
            $oCssGutschein = $this->db->queryPrepared(
              "SELECT * FROM css_gutscheinCoupon WHERE voucherCode= :voucherCode AND valid=1 AND DateExpire>".time(),
              ["voucherCode" => $aInfo['css_gutschein']],
              ReturnType::SINGLE_OBJECT
            );
            if($oCssGutschein) {
              $wertgutschein['kGutschein'] = $oCssGutschein->id;
              $this->db->updateRow('css_gutscheinCoupon', 'id', $oCssGutschein->id, (object)['restVoucherWorth' => $oCssGutschein->restVoucherWorth - $wertgutschein['fGuthaben']]);
            }
            $wertgutscheine['used'][] = $wertgutschein;
            $wertgutscheine['fSummeGuthaben'] += $wertgutschein['fGuthaben'];
            // EOF CSS-Gutscheine 2/8
          }
        }
      }
    }
    
    // add shipping
    $cInfo = $order->getShippingInfos()->getInternalShippingInfo();
    $bShippingInfo = false;
    if (!empty($cInfo)) {
      $aInfo = $this->jsonDecode($cInfo, true);
      if (array_key_exists('kShipping', $aInfo) && array_key_exists('cShipping', $aInfo)) {
        if (!empty($aInfo['kShipping'])) {
          $nShippingKey = $aInfo['kShipping'];
          $cShippingName = $this->_prepare_utf8($aInfo['cShipping'], false);
          $bShippingInfo = true;
        }
      }
    }
    if (!$bShippingInfo) {
      $cShippingName = $this->_prepare_utf8($order->getShippingInfos()->getName(), false);
      $nShippingKey = $this->_getKVersandartByName($cShippingName, $aWarenkorbVersandklassen, $JTL_Client->kKundengruppe, $JTL_Delivery->cLand);
    }
    
    $shippingAmountWithoutTax = $order->getAmountShipping();
    $fSteuersatz = $order->getShippingTaxPercent();
    if(!empty($fSteuersatz)) {
      // remove tax from shipping costs
      $shippingAmountWithoutTax /= 1+$fSteuersatz/100;
    }
    $warenkorbpos = new stdClass();
    $warenkorbpos->kWarenkorb      = $kWarenkorb;
    $warenkorbpos->kArtikel        = 0;
    $warenkorbpos->kVersandklasse  = 0;
    $warenkorbpos->cName           = $cShippingName;
    $warenkorbpos->cLieferstatus   = '';
    $warenkorbpos->cArtNr          = '';
    $warenkorbpos->cEinheit        = '';
    $warenkorbpos->fPreisEinzelNetto = $shippingAmountWithoutTax;
    $warenkorbpos->fPreis          = $shippingAmountWithoutTax;
    $warenkorbpos->fMwSt           = $fSteuersatz;
    $warenkorbpos->nAnzahl         = 1;
    $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_VERSANDPOS;
    $warenkorbpos->cHinweis        = '';
    $warenkorbpos->cUnique         = '';
    $warenkorbpos->kKonfigitem     = 0;
    $warenkorbpos->kBestellpos     = 0;
    $kWarenkorbPos = $this->db->insertRow("twarenkorbpos", $warenkorbpos);
    
    // get payment-infos
    $payment_result = $this->_plugin_handle_orders_payment($order);
    $paymentComment = $payment_result['paymentComment'];
    $kZahlart       = $payment_result['kZahlart'];
    $cZahlart       = $payment_result['cZahlart'];
    $cGebuehr       = $payment_result['cGebuehr'];
    if (array_key_exists('tZahlungseingang', $payment_result)) {
      $tZahlungseingang = (object) $payment_result['tZahlungseingang'];
    }
    if (array_key_exists('tZahlungsinfo', $payment_result)) {
      $tZahlungsinfo = (object) $payment_result['tZahlungsinfo'];
    }
    
    if (!empty($paymentComment)) {
      $paymentComment = chr(10).'Payment'.chr(10).$paymentComment;
    }
    
    // add payment
    if($order->getAmountShopPayment() != 0) {
			$paymentInfos = $order->getPaymentInfos();
      $paymentAmountWithoutTax = $order->getAmountShopPayment();
			$fSteuersatz = $order->getPaymentTaxPercent();
      if(!empty($fSteuersatz)) {
        // remove tax from shipping costs
        $paymentAmountWithoutTax /= 1+$fSteuersatz/100;
      }
      
			$warenkorbpos = new stdClass();
      $warenkorbpos->kWarenkorb      = $kWarenkorb;
      $warenkorbpos->kArtikel        = 0;
      $warenkorbpos->kVersandklasse  = 0;
      $warenkorbpos->cName           = $cGebuehr;
      $warenkorbpos->cLieferstatus   = '';
      $warenkorbpos->cArtNr          = '';
      $warenkorbpos->cEinheit        = '';
      $warenkorbpos->fPreisEinzelNetto = $paymentAmountWithoutTax;
      $warenkorbpos->fPreis          = $paymentAmountWithoutTax;
      $warenkorbpos->fMwSt           = $fSteuersatz;
      $warenkorbpos->nAnzahl         = 1;
      $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_ZAHLUNGSART;
      $warenkorbpos->cHinweis        = '';
      $warenkorbpos->cUnique         = '';
      $warenkorbpos->kKonfigitem     = 0;
      $warenkorbpos->kBestellpos     = 0;
      $kWarenkorbPos = $this->db->insertRow("twarenkorbpos", $warenkorbpos);
		}
    
    /* BOF Build order-number */
    $orderHandler = new OrderHandler($this->db, $JTL_Client, $oWarenkorb);
    $tmp_order_number = '';
    if (array_key_exists('cUseShopgateBestellnummer', $this->JTLPluginConf)) {
      $tmp_order_number = $this->JTLPluginConf['cUseShopgateBestellnummer'] == 'Y' ? $order->getOrderNumber() : $orderHandler->createOrderNo();
      $tmp_order_number = $this->JTLPluginConf['cOrdernumberPrefix'].$tmp_order_number.$this->JTLPluginConf['cOrdernumberSuffix'];
    }
    else {
      $tmp_order_number = $orderHandler->createOrderNo().'SG';
    }
    /* EOF Build order-number */
    
    // create Order-object
    $JTL_Order = new Bestellung();
    $JTL_Order->kRechnungsadresse = $JTL_Invoice->kRechnungsadresse;
    $JTL_Order->kLieferadresse    = $equal_address ? 0 : $JTL_Delivery->kLieferadresse;
    $JTL_Order->kKunde            = $JTL_Client->kKunde;
    $JTL_Order->kZahlungsart      = $kZahlart;
    $JTL_Order->kVersandart       = $nShippingKey;
    $JTL_Order->kWaehrung         = $kWaehrung;
    $JTL_Order->kSprache          = $JTL_Client -> kSprache;
    $JTL_Order->fGesamtsumme      = $order->getAmountComplete();
    $JTL_Order->cBestellNr        = $tmp_order_number;
    if ($this->JTLPluginConf['cAddShopgateBestellnummerKommentar'] == 'Y') {
      $JTL_Order->cKommentar        = 'shopgate '.$order->getOrderNumber().$paymentComment;
    }
    else {
      $JTL_Order->cKommentar        = $paymentComment;
    }
    $JTL_Order->cVersandartName   = $cShippingName;
    $JTL_Order->cZahlungsartName  = $cZahlart;
    $JTL_Order->dErstellt         = date('Y-m-d H:i:s');
    if ($payment_result['bSetPayed']) {
      $JTL_Order->cStatus          = BESTELLUNG_STATUS_BEZAHLT;
      $JTL_Order->dBezahltDatum = $order->getIsPaid() ? $order->getPaymentTime('Y-m-d') : date('Y-m-d');
    }
    else {
      $JTL_Order->cStatus          = BESTELLUNG_STATUS_OFFEN;
    }
    if (array_key_exists('cSession',$payment_result)) {
      $JTL_Order->cSession = $payment_result['cSession'];
    }
    $JTL_Order->kWarenkorb        = $kWarenkorb;
    $JTL_Order->cAbgeholt         = 'N';
    $JTL_Order->setEstimatedDelivery($longestDelivery['nLongestMinDelivery'], $longestDelivery['nLongestMaxDelivery']);
    
    
    if (array_key_exists('cPUIZahlungsdaten', $payment_result)) {
      if (property_exists($JTL_Order, 'cPUIZahlungsdaten')) {
        $JTL_Order->cPUIZahlungsdaten = $payment_result['cPUIZahlungsdaten'];
      }
    }
    
    $custom_fields_array = array();
    foreach ($order->getCustomFields() as $row) {
      $custom_fields_array[] = $this->_prepare_utf8($row->getLabel().' ['.$row->getInternalFieldName().']: '.$row->getValue(), false);
    }
    if (!empty($custom_fields_array)) {
      $JTL_Order->cKommentar .= chr(10).chr(10).implode(chr(10), $custom_fields_array);
    }
    
    // BOF CSS-Gutscheine 3/8
    if(isset($wertgutscheine)) {
      $JTL_Order->fGuthaben = $wertgutscheine['fSummeGuthaben']*-1;
    }
    if($JTL_Order->fGesamtsumme <= 0) {
      $JTL_Order->cStatus          = BESTELLUNG_STATUS_BEZAHLT;
      $JTL_Order->dBezahltDatum    = 'now()';
      $JTL_Order->cZahlungsartName = '-';
    }
    // EOF CSS-Gutscheine 3/8

    $kBestellung = $JTL_Order->insertInDB();
    $this->db->executeQueryPrepared(
      "UPDATE tkunde SET cAbgeholt='N' WHERE kKunde= :kKunde",
      ['kKunde' => $JTL_Client->kKunde]
    );
    
    $bestellid  = new stdClass();
    $bestellid->cId = $this->_gibUID(40);
    $bestellid->kBestellung = $kBestellung;
    $bestellid->dDatum = "now()";
    $this->db->insertRow('tbestellid',$bestellid);
    
    $bestellstatus  = new stdClass();
    $bestellstatus->kBestellung = $kBestellung;
    $bestellstatus->dDatum ="now()";
    $bestellstatus->cUID = $this->_gibUID(40);
    $this->db->insertRow('tbestellstatus',$bestellstatus);
    
    foreach ($addKuponBestellung as $kKupon => $aKuponData) {
      $KuponBestellung = new stdClass();
      $KuponBestellung->kKupon = $kKupon;
      $KuponBestellung->kBestellung = $kBestellung;
      $KuponBestellung->kKunde = $JTL_Order->kKunde;
      $KuponBestellung->cBestellNr = $JTL_Order->cBestellNr;
      $KuponBestellung->fGesamtsummeBrutto = $JTL_Order->fGesamtsumme;
      $KuponBestellung->fKuponwertBrutto = $aKuponData['amount'];
      $KuponBestellung->cKuponTyp = $aKuponData['oKupon']->getKuponTyp();
      $KuponBestellung->dErstellt = 'now()';
      $this->db->insertRow('tkuponbestellung', $KuponBestellung);
    }
    
    // safe in seperate table
    $ordersShopgateOrder  = new stdClass();
    $ordersShopgateOrder->kBestellung  = $kBestellung;
    $ordersShopgateOrder->kBestellungShopgate = $order->getOrderNumber();
    $ordersShopgateOrder->created      = date('Y-m-d H:i:s');
    $ordersShopgateOrder->modified     = date('Y-m-d H:i:s');
    $ordersShopgateOrder->sgShopNumber = $this->request_shop_number;
    $this->db->insertRow("xplugin_agi_shopgate_connect_orders", $ordersShopgateOrder);
    
    // BOF CSS-Gutscheine 4/8
    if(isset($wertgutscheine)) {
      foreach($wertgutscheine['used'] as $wertgutschein) {
        $cssGutscheinHistory = new stdClass();
        $cssGutscheinHistory->pid = $wertgutschein['kGutschein'];
        $cssGutscheinHistory->cBestellNr = $JTL_Order->cBestellNr;
        $cssGutscheinHistory->cMail = $wertgutschein['cMail'];
        $cssGutscheinHistory->kBestellung = $kBestellung;
        $cssGutscheinHistory->kWaehrung = $JTL_Order->kWaehrung;
        $cssGutscheinHistory->cStatus = 0;
        $cssGutscheinHistory->usedValue = $wertgutschein['fGuthaben'] *-1;
        $cssGutscheinHistory->tstamp = time();
        $cssGutscheinHistory->kRechnungsadresse = $JTL_Order->kRechnungsadresse;
        $cssGutscheinHistory->kKunde = $JTL_Order->kKunde;
        $this->db->insertRow("css_gutscheinHistory", $cssGutscheinHistory);
      }
    }
    // EOF CSS-Gutscheine 4/8
    
    if (isset($tZahlungseingang)) {
      $tZahlungseingang->kBestellung  = $kBestellung;
      $this->db->insertRow("tzahlungseingang", $tZahlungseingang);
    }
    if (isset($tZahlungsinfo)) {
      $tZahlungsinfo->kBestellung  = $kBestellung;
      $tZahlungsinfo->kKunde  = $JTL_Order->kKunde;
      $this->db->insertRow("tzahlungsinfo", $tZahlungsinfo);
    }
    /* Amazon-Payments */
    if (array_key_exists('insertToTables', $payment_result)) {
      foreach ($payment_result['insertToTables'] as $tableKey => $tableData) {
        $tmpData = array();
        foreach ($tableData as $tmpKey => $tmpValue) {
          $tmpData[$tmpKey] = str_replace('@@ORDER_ID@@',$kBestellung,$tmpValue);
        }
        $this->db->insertRow($tableKey, (object)$tmpData);
      }
    }

    // Bestellmail versenden
    if (array_key_exists('cSendOrderConfirmation',$this->JTLPluginConf) && $this->JTLPluginConf['cSendOrderConfirmation'] == 'Y') {
      require_once PFAD_ROOT . PFAD_INCLUDES . 'mailTools.php';
      $obj = new stdClass();
      $obj->cVerfuegbarkeit_arr = $verfuegbarkeitResult_arr;
      $bestellung = new Bestellung($kBestellung);
      $bestellung->fuelleBestellung(0);
      $obj->tkunde = $JTL_Client;
      $obj->tbestellung = $bestellung;
      $mailer = Shop::Container()->get(Mailer::class);
      $mail   = new Mail();
      $mailer->send($mail->createFromTemplateID(MAILTEMPLATE_BESTELLBESTAETIGUNG, $obj));
    }
    
    // Return the order ID and number of the order saved to the database
    return array(
      'external_order_id' => $kBestellung,        // this is used to inform a developer in case there was a problem
      'external_order_number' => $JTL_Order->cBestellNr // this is used to inform a merchant in case there was a problem
    );
  }

  /**
    * Performs the necessary queries to update an order in the shop system's database.
    *
    * @see http://developer.shopgate.com/merchant_api/orders/get_orders
    * @see http://developer.shopgate.com/plugin_api/orders/update_order
    *
    * @param ShopgateOrder $order The ShopgateOrder object to be updated in the shop system's database.
    * @return array(
    *		'external_order_id' => <i>string</i>, # the ID of the order in your shop system's database
    *		'external_order_number' => <i>string</i> # the number of the order in your shop system
    *	)
    * @throws ShopgateLibraryException if an error occurs.
    */
  public function updateOrder(ShopgateOrder $order)
  {
    if (method_exists($this, 'agi_shopgate_connect__updateOrder')) {
      $this->agi_shopgate_connect__updateOrder($order);
    }
    
    $aRes = $this->db->queryPrepared(
      "SELECT kBestellung, history FROM xplugin_agi_shopgate_connect_orders WHERE kBestellungShopgate= :kBestellungShopgate",
      ['kBestellungShopgate' => $order->getOrderNumber()],
      ReturnType::SINGLE_OBJECT
    );
    
    if (empty($aRes)) {
      throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_ORDER_NOT_FOUND, "Shopgate order number: '{$order->getOrderNumber()}'.");
    }
    
    $dbOrderID = $aRes->kBestellung;
    $dbOrderHistory = $aRes->history;
    
    if (empty($dbOrderID)) {
      throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_ORDER_NOT_FOUND, "Shopgate order number: '{$order->getOrderNumber()}'.");
    }
    
    // load some order-information
    $dbOrder = $this->db->queryPrepared(
      "SELECT cBestellNr, dBezahltDatum, cKommentar FROM tbestellung WHERE kBestellung= :kBestellung",
      ['kBestellung' => $dbOrderID],
      ReturnType::SINGLE_OBJECT
    );
    $something_changed = false;
    $history_comments = array();
    $updateOrder  = new stdClass();
    if ($order->getUpdatePayment() == 1) {
      // Payment status or PaymentInfos has changed
      if ($order->getIsPaid() && $dbOrder->dBezahltDatum == '0000-00-00') {
        $updateOrder->dBezahltDatum = $order->getPaymentTime('Y-m-d');
        $updateOrder->cStatus       = BESTELLUNG_STATUS_BEZAHLT;
        $history_comments[] = 'Bestellstatus von Shopgate gendert: Zahlung erhalten';
        
        // get payment-infos
        $payment_result = $this->_plugin_handle_orders_payment($order);
        $paymentComment = $payment_result['paymentComment'];
        $kZahlart       = $payment_result['kZahlart'];
        $cZahlart       = $payment_result['cZahlart'];
        if (array_key_exists('tZahlungseingang', $payment_result)) {
          $tZahlungseingang = (object) $payment_result['tZahlungseingang'];
        }
        
        if (!empty($paymentComment)) {
          $paymentComment = chr(10).'Payment '.date('Y-m-d').chr(10).$paymentComment;
          if (!isset($updateOrder->cKommentar)) {
            $updateOrder->cKommentar = '';
          }
          $updateOrder->cKommentar .= $paymentComment;
        }
        // add payment information to shop
        if (isset($tZahlungseingang)) {
          $tZahlungseingang->kBestellung  = $dbOrderID;
          $this->db->insertRow("tzahlungseingang", $tZahlungseingang);
        }
        else {
          $objBestellung = new Bestellung($dbOrderID);
          $fBetrag       = $objBestellung->fGesamtsummeKundenwaehrung;
          $objISO = $this->db->queryPrepared(
            "SELECT cISO FROM twaehrung WHERE kWaehrung= :kWaehrung",
            ['kWaehrung' => $objBestellung->kWaehrung],
            ReturnType::SINGLE_OBJECT
          );
          $cISO = $objISO->cISO;
          // safe in seperate table
          $ordersPayment  = new stdClass();
          $ordersPayment->kBestellung       = $dbOrderID;
          $ordersPayment->cZahlungsanbieter = $cZahlart;
          $ordersPayment->fBetrag           = $fBetrag;
          $ordersPayment->cISO              = $cISO;
          $ordersPayment->dZeit             = $order->getPaymentTime('Y-m-d H:i:s');
          $ordersPayment->cAbgeholt         = 'N';
          $this->db->insertRow("tzahlungseingang", $ordersPayment);
          $something_changed = true;
        }
      }
      elseif (!$order->getIsPaid() && $dbOrder->dBezahltDatum != '0000-00-00') {
        $updateOrder->dBezahltDatum = '0000-00-00';
        $updateOrder->cStatus       = BESTELLUNG_STATUS_OFFEN;
        $history_comments[] = 'Bestellstatus von Shopgate gendert: Zahlung noch nicht erhalten';
        $something_changed = true;
      }
    }

    if ($order->getUpdateShipping() == 1) {
      // Released or blocked for shipping
      if ($order->getIsShippingBlocked() == 1) {
        $history_comments[] = 'Versand blockiert!';
        $something_changed = true;
      }
      else {
        $history_comments[] = 'Versand freigegeben';
        if (!isset($updateOrder->cKommentar)) {
          $updateOrder->cKommentar = '';
        }
        $updateOrder->cKommentar = str_replace(chr(10).chr(10).'SHOPGATE : Versand blockiert!','', $dbOrder->cKommentar);
        $something_changed = true;
      }
    }
    
    if ($something_changed) {
      $this->db->updateRow("tbestellung", "kbestellung", $dbOrderID, $updateOrder);
      $aUpdate = array(
        'modified'  => date('Y-m-d H:i:s')
      );
      if (!empty($history_comments)) {
        $aUpdate['history'] = $dbOrderHistory.chr(10).chr(10).'Update '.date('Y-m-d H:i:s').chr(10).implode(chr(10), $history_comments);
      }
        
      if (!empty($aUpdate)) {
        $this->db->updateRow("xplugin_agi_shopgate_connect_orders", "kbestellung", $dbOrderID, (object)$aUpdate);
      }
    }
    
    // Return the order ID and number of the order saved to the database
    return array(
      'external_order_id' => $dbOrderID,        // this is used to inform a developer in case there was a problem
      'external_order_number' => $dbOrder->cBestellNr // this is used to inform a merchant in case there was a problem
    );
  }
  
  /**
    * Loads the products of the shop system's database and passes them to the buffer.
    *
    * @param int $limit pagination limit; if not null, the number of exported items must be <= $limit
    * @param int $offset pagination; if not null, start the export with the item at position $offset
    * @param string[] $uids a list of item UIDs that should be exported
    *
    * @see http://developer.shopgate.com/plugin_api/export/get_items
    *
    * @throws ShopgateLibraryException
    */
  protected function createItems($limit = null, $offset = null, array $uids = array ())
  {
    // Konfiguration laden
    $this->_AGICreateItemsConfig();
    if (method_exists($this, 'agi_shopgate_connect__createItemsStartup')) {
      $this->agi_shopgate_connect__createItemsStartup();
    }
    if (!empty($uids)) {
      $oUids = $uids;
      $uids = array();
      foreach ($oUids as $value) {
        $uids[] = (int)$value;
      }
      $uids = array_unique($uids);
      $query = "SELECT 
          tartikel.kArtikel
        FROM tartikel
        LEFT JOIN tartikelsichtbarkeit ON 
          tartikelsichtbarkeit.kArtikel = tartikel.kArtikel AND tartikelsichtbarkeit.kKundengruppe= :kKundengruppe
        WHERE tartikel.kArtikel IN (".implode(',', $uids).") AND tartikel.kVaterArtikel=0 AND tartikelsichtbarkeit.kArtikel IS NULL
        ORDER BY kArtikel";
      $loaded_products = $this->db->queryPrepared(
        $query,
        ['kKundengruppe' => $this->kKundengruppe],
        ReturnType::ARRAY_OF_OBJECTS
      );
    } 
    else {
      
      $sLimit = '';
      if (!empty($limit)) {
        $sLimit = (int) $limit;
      }
      if (!empty($limit) && !empty($offset)) {
        $sLimit = ((int) $offset).','.$sLimit;
      }
      if (!empty($sLimit)) {
        $sLimit = ' LIMIT '.$sLimit;
      }
      
      $query = "SELECT 
          tartikel.kArtikel
        FROM tartikel
        LEFT JOIN tartikelsichtbarkeit ON 
          tartikelsichtbarkeit.kArtikel = tartikel.kArtikel AND tartikelsichtbarkeit.kKundengruppe= :kKundengruppe
        WHERE tartikel.kArtikel AND tartikel.kVaterArtikel=0 AND tartikelsichtbarkeit.kArtikel IS NULL
        ORDER BY kArtikel".$sLimit;
      $loaded_products = $this->db->queryPrepared(
        $query,
        ['kKundengruppe' => $this->kKundengruppe],
        ReturnType::ARRAY_OF_OBJECTS
      );
    }
    if (method_exists($this, 'agi_shopgate_connect__createItems')) {
      // to manipulate $this->agi_categories
      $this->agi_shopgate_connect__createItems($limit, $offset, $uids);
    }
    
    foreach ($loaded_products as $value) {
      $this->_loadOExportArticle($value->kArtikel);

      if (empty($this->oExportArticle->kArtikel)) {
        continue;
      }
      $this->_AGIBuildItemObject();
      if (time()-$this->startExecutionTime > $this->maxExecutionTime) {
        break;
      }
    }
  }
  
  private function _loadOExportArticle($kArtikel)
  {
    $oArtikelOptionen = new stdClass();
    $oArtikelOptionen->nMerkmale = 1;
    $oArtikelOptionen->nAttribute = 1;
    $oArtikelOptionen->nArtikelAttribute = 1;
    $oArtikelOptionen->nKategorie = 1;
    $oArtikelOptionen->nKeinLagerbestandBeachten = 1;
    $oArtikelOptionen->nVariationen = 1;
    if ($this->bAllowKonfiguration) {
      $oArtikelOptionen->nKonfig = 1;
    }
    
    if (method_exists($this, 'agi_shopgate_connect__oArtikelOptionen')) {
      $oArtikelOptionen = $this->agi_shopgate_connect__oArtikelOptionen($oArtikelOptionen);
    }
    
    $this->oExportArticle = new Artikel();
    $this->oExportArticle->fuelleArtikel(
      $kArtikel,
      $oArtikelOptionen,
      $this->kKundengruppe,
      $this->kSprache,
      false // false:use-cache //true:no-cache
    );
  }
  
  private function _AGIBuildItemObject($thisIsChild = 0)
  {
    //var_dump($this->oExportArticle);
    $aRes = $this->db->queryPrepared(
      "SELECT dLetzteAktualisierung FROM `tartikel` WHERE kArtikel= :kArtikel ",
      ['kArtikel' => $this->oExportArticle->kArtikel],
      ReturnType::SINGLE_OBJECT
    );
    $this->oExportArticle->dLetzteAktualisierung = empty($aRes->dLetzteAktualisierung) ? null : $aRes->dLetzteAktualisierung;
    
    // manipulate here
    $this->oExportArticle->nHasChildren = $this->oExportArticle->nIstVater ? 1 : 0;
    $this->oExportArticle->cParentItemNumber = empty($this->oExportArticle->kVaterArtikel) ? '' : $this->oExportArticle->kVaterArtikel;
    $this->oExportArticle->fLagerbestand = floor($this->oExportArticle->fLagerbestand);
    $this->oExportArticle->fUst = Tax::getSalesTax($this->oExportArticle->kSteuerklasse);
    
    $this->_AGIoExportArticleAddShippingCost();
    $this->_AGIoExportArticleCheckHighlight();
    $this->_AGIoExportArticleCheckStatusAktiv();
    $this->_AGIoExportArticleSetLieferstatus();
    if (in_array('attributes', $this->load_to_article)) {
      $this->_AGIoExportArticleAddAttributes();
    }
    if (in_array('categories', $this->load_to_article)) {
      $this->_AGIoExportArticleAddCategories();
    }
    if (in_array('vpe', $this->load_to_article)) {
      $this->_AGIoExportArticleAddGrundpreis();
    }
    if (in_array('media', $this->load_to_article)) {
      $this->_AGIoExportArticleAddMediaFiles();
    }
    if (in_array('description', $this->load_to_article)) {
      $this->_AGIoExportArticlePrepareDescription();
    }
    
    if (method_exists($this, 'agi_shopgate_connect__AGIModifyExportArticle')) {
      $this->agi_shopgate_connect__AGIModifyExportArticle($thisIsChild);
    }
    
    
    $productModel = new Shopgate_Model_Catalog_Product();
    if ($thisIsChild) {
      $productModel->setIsChild(1);
    }
    $productModel->setUid($this->oExportArticle->kArtikel);
    $productModel->setName($this->_prepare_utf8(Text::htmlentitydecode($this->oExportArticle->cName)));
    $productModel->setLastUpdate($this->oExportArticle->dLetzteAktualisierung);
    $productModel->setTaxPercent($this->oExportArticle->fMwSt);
    $productModel->setTaxClass($this->_prepare_utf8($this->aSteuerklassen[$this->oExportArticle->kSteuerklasse]['cName']));
    $productModel->setCurrency($this->oWaehrung->cISO);
    $productModel->setDescription($this->_prepare_utf8($this->oExportArticle->cBeschreibung));
    $productModel->setDeeplink($this->URL_SHOP.$this->oExportArticle->cURL);
    if ($this->oExportArticle->cTopArtikel == 'Y') {
      $productModel->setPromotionSortOrder(10);
    }
    $productModel->setWeight($this->oExportArticle->fArtikelgewicht);
    $productModel->setWeightUnit(Shopgate_Model_Catalog_Product::DEFAULT_WEIGHT_UNIT_KG);
    //$productModel->setAgeRating(18);
    
    $productModel->setPrice($this->add_Shopgate_Model_Catalog_Price($thisIsChild));
    
    // add images
    $count_images = 0;
    if (isset($this->oExportArticle->Bilder)) {
      foreach ($this->oExportArticle->Bilder as $value) {
        if ($thisIsChild == 0 || ($thisIsChild > 0 && strpos($value->cPfadGross, BILD_KEIN_ARTIKELBILD_VORHANDEN) === false)) {
          $imageItem = new Shopgate_Model_Media_Image();
          $imageItem->setUid($this->oExportArticle->kArtikel.'_'.$value->nNr);
          $imageItem->setSortOrder($value->nNr);
          $imageItem->setAlt($this->_prepare_utf8(Text::htmlentitydecode(empty($value->cAltAttribut) ? $this->oExportArticle->cName : $value->cAltAttribut)));
          $imageItem->setTitle($this->_prepare_utf8(Text::htmlentitydecode(empty($value->cAltAttribut) ? $this->oExportArticle->cName : $value->cAltAttribut)));
          $imageItem->setUrl($this->URL_SHOP.$value->cPfadGross);
          $productModel->addImage($imageItem);
          $count_images++;
        }
      }
    }
    if ($thisIsChild > 0 && $count_images == 0) {
      // add parent images
      $aRes = $this->db->queryPrepared(
        "SELECT * FROM `tartikelpict` WHERE kArtikel= :kArtikel ORDER BY nNr",
        ['kArtikel' => $this->oExportArticle->kArtikel],
        ReturnType::ARRAY_OF_OBJECTS
      );
      foreach ($aRes as $row) {
        $imageItem = new Shopgate_Model_Media_Image();
        $imageItem->setUid($this->oExportArticle->kArtikel.'_'.$row->nNr);
        $imageItem->setSortOrder($row->nNr);
        $imageItem->setAlt($this->oExportArticle->cName);
        $imageItem->setTitle($this->oExportArticle->cName);
        $imageItem->setUrl($this->URL_SHOP.PFAD_PRODUKTBILDER_GROSS.$row->cPfad);
        $productModel->addImage($imageItem);
      }
    }

    foreach ($this->oExportArticle->aAGIKategorien as $value) {
      $categoryPathItem = new Shopgate_Model_Catalog_CategoryPath();
      $categoryPathItem->setUid($value['kKategorie']);
      $categoryPathItem->setSortOrder($this->_getArticleSortInCategory($value['kKategorie']));
      $categoryPathItem->setDeeplink($value['cDeeplink']);
      foreach($value['paths'] as $level => $path) {
        $categoryPathItem->addItem($level, $this->_prepare_utf8($path));
      }
      if (!empty($value['image'])) {
        $imageItem = new Shopgate_Model_Media_Image();
        $imageItem->setUid($value['kKategorie']);
        $imageItem->setAlt($this->_prepare_utf8($value['cName']));
        $imageItem->setTitle($this->_prepare_utf8($value['cName']));
        $imageItem->setUrl($value['image']);
        $categoryPathItem->addImage($imageItem);
      }
      $productModel->addCategoryPath($categoryPathItem);
    }
    if (array_key_exists('cUseNewArticleCategory', $this->JTLPluginConf) && $this->JTLPluginConf['cUseNewArticleCategory'] == 'Y' && $this->oExportArticle->cNeu == 'Y' && $this->oExportArticle->dErstellt>=$this->max_date_new_article) {
      $categoryPathItem = new Shopgate_Model_Catalog_CategoryPath();
      $categoryPathItem->setUid('1neu');
      $categoryPathItem->setSortOrder(1000);
      //$categoryPathItem->setDeeplink($value['cDeeplink']);
      $productModel->addCategoryPath($categoryPathItem);
    }
    
    $shippingItem = new Shopgate_Model_Catalog_Shipping();
    $shippingItem->setIsFree($this->oExportArticle->nOhneVersandkosten);
    $shippingItem->setCostsPerOrder($this->oExportArticle->fVersandkosten);
    $productModel->setShipping($shippingItem);
    
    if (isset($this->oExportArticle->kHersteller) && !empty($this->oExportArticle->kHersteller)) {
      $manufacturerItem = new Shopgate_Model_Catalog_Manufacturer();
      $manufacturerItem->setUid($this->oExportArticle->kHersteller);
      $manufacturerItem->setTitle($this->_prepare_utf8($this->oExportArticle->cHersteller));
      $manufacturerItem->setItemNumber($this->oExportArticle->cHAN);
      $productModel->setManufacturer($manufacturerItem);
    }
    
    $stockItem = new Shopgate_Model_Catalog_Stock();
    $stockItem->setStockQuantity($this->oExportArticle->fLagerbestand);
    $stockItem->setAvailabilityText($this->_prepare_utf8($this->_remove_html($this->oExportArticle->agiLieferstatus)));
    $stockItem->setBackorders(1);
    $stockItem->setIsSaleable(1);
    $stockItem->setUseStock($this->oExportArticle->nUseStock);
    if ($this->oExportArticle->cStatusAktiv == 'inactive') {
      $stockItem->setStockQuantity(0);
      $stockItem->setBackorders(0);
      $stockItem->setUseStock(1);
      $stockItem->setIsSaleable(0);
    }
    $stockItem->setMinimumOrderQuantity(empty($this->oExportArticle->fMindestbestellmenge) ? 1 : ceil($this->oExportArticle->fMindestbestellmenge));
    if (defined('FKT_ATTRIBUT_MAXBESTELLMENGE') && array_key_exists(FKT_ATTRIBUT_MAXBESTELLMENGE,$this->oExportArticle->FunktionsAttribute) && $this->oExportArticle->FunktionsAttribute[FKT_ATTRIBUT_MAXBESTELLMENGE] > 0) {
      $stockItem->setMaximumOrderQuantity($this->oExportArticle->FunktionsAttribute[FKT_ATTRIBUT_MAXBESTELLMENGE]);
    }
    $productModel->setStock($stockItem);
    
    if (isset($this->oExportArticle->Attribute) && !empty($this->oExportArticle->Attribute)) {
      foreach ($this->oExportArticle->Attribute as $value) {
        if (!in_array(strtolower($value->cName), $this->aAttributNichtExportieren)) {
          $propertyItem = new Shopgate_Model_Catalog_Property();
          $propertyItem->setUid($value->cName);
          $propertyItem->setLabel($this->_prepare_utf8($value->cName));
          $propertyItem->setValue($this->_prepare_utf8($value->cWert));
          $productModel->addProperty($propertyItem);
        }
      }
    }
    if (isset($this->oExportArticle->oMerkmale_arr) && !empty($this->oExportArticle->oMerkmale_arr)) {
      foreach ($this->oExportArticle->oMerkmale_arr as $value) {
        if (!in_array(strtolower($value->cName), $this->aMerkmaleNichtExportieren)) {
          foreach ($value->oMerkmalWert_arr AS $wert) {
            $propertyItem = new Shopgate_Model_Catalog_Property();
            $propertyItem->setUid($value->kMerkmal.'_'.$wert->kMerkmalWert);
            $propertyItem->setLabel($this->_prepare_utf8($value->cName));
            $propertyItem->setValue($this->_prepare_utf8($wert->cWert));
            $productModel->addProperty($propertyItem);
          }
        }
      }
    }
    
    /* BOF GPSR 1/2 */
    $gpsr_export = array();
    if (isset($this->JTLPluginConf['cGPSROption'])) {
      switch($this->JTLPluginConf['cGPSROption']) {
        case 'M': $gpsr_export[] = 'manufacturer'; break;
        case 'P': $gpsr_export[] = 'responsibleperson'; break;
        case 'B': $gpsr_export[] = 'manufacturer'; $gpsr_export[] = 'responsibleperson'; break;
      }
    }
    else {
      $gpsr_export[] = 'manufacturer';
    }
      
    $p_gpsr = PluginHelper::getPluginById('jtl_gpsr');
    if ($p_gpsr) {
      $gpsr_lang_heading = array(
        'manufacturer'      => $p_gpsr->getLocalization()->getTranslation('jtl_gpsr_ManufacturerLabel'),
        'responsibleperson' => $p_gpsr->getLocalization()->getTranslation('jtl_gpsr_RespPersonLabel')
      );
    } else {
      $gpsr_lang_heading = array(
        'manufacturer'      => 'Herstellerinformationen',
        'responsibleperson' => 'verantwortliche Person'
      );
    }
    $gpsr = array('manufacturer' => array(), 'responsibleperson' => array());
    $gpsr_fields = array('name', 'street', 'housenumber', 'postalcode', 'city', 'state', 'country', 'email', 'homepage');
    foreach ($gpsr as $group => $empty) {
      foreach ($gpsr_fields as $field) {
        $idx = 'gpsr_' . $group . '_' . $field;
        if (array_key_exists($idx, $this->oExportArticle->FunktionsAttribute) && !empty($this->oExportArticle->FunktionsAttribute[$idx])) {
          $gpsr[$group][$field] = $this->oExportArticle->FunktionsAttribute[$idx];
        }
      }
    }
    $export_gpsr_property = array();
    foreach ($gpsr as $group => $address) {
      if (count($gpsr[$group])) {
        if (!empty($gpsr[$group]['name'])) {
          $propertyItem = new Shopgate_Model_Catalog_Property();
          $propertyItem->setUid('gpsr_'.$group);
          $propertyItem->setLabel($this->_prepare_utf8($gpsr_lang_heading[$group]));
          $propertyItem->setValue($this->_build_gpsr_address($address, $group));
          $export_gpsr_property[$group] = $propertyItem;
        }
      }
    }
    foreach ($export_gpsr_property as $group => $propertyItem) {
      if (count($export_gpsr_property) == 2 && !in_array($group, $gpsr_export)) {
        continue;
      }
      $productModel->addProperty($propertyItem);
    }
    /* EOF GPSR 1/2 */
    
    // Grundpreis als Merkmal
    if ($this->oExportArticle->cVPE == 'Y') {
      $cVpeEinheit = $this->oExportArticle->cVPEEinheit;
      $fVpeWert = $this->oExportArticle->fVPEWert;
      if (preg_match('/^[0-9]{1,}/',$cVpeEinheit, $matches)) {
        $multiplice = $matches[0];
        $vpe_string = preg_replace('/^[0-9]{1,}/', '', $cVpeEinheit, 1);
        $vpe_string = trim($vpe_string);
        
        $lowwer = strtolower($vpe_string);
        //if (in_array($lowwer, array('g','ml','gramm','milliliter')) && $multiplice > 10)
          //$multiplice *= 10;
        
        $cVpeEinheit = $vpe_string;
        $fVpeWert *= $multiplice;
      }
      if (round($fVpeWert) == $fVpeWert) {
        $tmp = round($fVpeWert);
      }
      elseif (round($fVpeWert, 2) == $fVpeWert) {
        $tmp = number_format($fVpeWert,2,',','');
      }
      else {
        $tmp = number_format($fVpeWert,3,',','');
      }
      $propertyItem = new Shopgate_Model_Catalog_Property();
      $propertyItem->setUid('vpe_inhalt');
      $propertyItem->setLabel($this->_prepare_utf8('Inhalt'));
      $propertyItem->setValue($this->_prepare_utf8($tmp.' '.$cVpeEinheit));
      $productModel->addProperty($propertyItem);
    }
    
    // Identifier
    if (isset($this->oExportArticle->cArtNr) && !empty($this->oExportArticle->cArtNr) && $this->oExportArticle->cArtNr != 'n/a') {
      $identifierItem = new Shopgate_Model_Catalog_Identifier();
      $identifierItem->setUid('cArtNr');
      $identifierItem->setType($this->_prepare_utf8('sku'));
      $identifierItem->setValue($this->_prepare_utf8($this->oExportArticle->cArtNr));
      $productModel->addIdentifier($identifierItem);
    }
    if (isset($this->oExportArticle->cISBN) && !empty($this->oExportArticle->cISBN) && $this->oExportArticle->cISBN != 'n/a') {
      if (!preg_match('/[^\d\-]/', $this->oExportArticle->cISBN) && strlen($this->oExportArticle->cISBN) <= 16) {
        $identifierItem = new Shopgate_Model_Catalog_Identifier();
        $identifierItem->setUid('cISBN');
        $identifierItem->setType($this->_prepare_utf8('isbn'));
        $identifierItem->setValue($this->_prepare_utf8($this->oExportArticle->cISBN));
        $productModel->addIdentifier($identifierItem);
      }
    }
    if (isset($this->oExportArticle->cBarcode) && !empty($this->oExportArticle->cBarcode) && $this->oExportArticle->cBarcode != 'n/a') {
      if (!preg_match('/[^\d]/', $this->oExportArticle->cBarcode)) {
        $identifierItem = new Shopgate_Model_Catalog_Identifier();
        $identifierItem->setUid('cEAN');
        $identifierItem->setType($this->_prepare_utf8('ean'));
        $identifierItem->setValue($this->_prepare_utf8($this->oExportArticle->cBarcode));
        $productModel->addIdentifier($identifierItem);
      }
    }
    if (isset($this->oExportArticle->cUPC) && !empty($this->oExportArticle->cUPC) && $this->oExportArticle->cUPC != 'n/a') {
      if (!preg_match('/[^\d]/', $this->oExportArticle->cUPC) && strlen($this->oExportArticle->cUPC) <= 12) {
        $identifierItem = new Shopgate_Model_Catalog_Identifier();
        $identifierItem->setUid('cUPC');
        $identifierItem->setType($this->_prepare_utf8('upc'));
        $identifierItem->setValue($this->_prepare_utf8($this->oExportArticle->cUPC));
        $productModel->addIdentifier($identifierItem);
      }
    }
    
    if (isset($this->oExportArticle->cSuchbegriffe) && !empty($this->oExportArticle->cSuchbegriffe)) {
      $tmp = explode(' ', $this->oExportArticle->cSuchbegriffe);
      foreach ($tmp as $ttag) {
        $ttag = trim($ttag);
        if (!empty($ttag)) {
          $tagItem = new Shopgate_Model_Catalog_Tag();
          //$tagItem->setUid($ttag);
          $tagItem->setValue($this->_prepare_utf8($ttag));
          $productModel->addTag($tagItem);
        }
      }
    }
    
    // Crossselling
    if ($this->aTeinstellungenFields['artikeldetails_xselling_standard_anzeigen'] == 'Y') {
      $lagerFilter = '';
      if ($this->aTeinstellungenFields['artikel_artikelanzeigefilter'] == EINSTELLUNGEN_ARTIKELANZEIGEFILTER_LAGER) {
        $lagerFilter = "AND (NOT (tartikel.fLagerbestand <= 0 AND tartikel.cLagerBeachten = 'Y') OR tartikel.cLagerVariation = 'Y')";
      }
      if ($this->aTeinstellungenFields['artikel_artikelanzeigefilter'] == EINSTELLUNGEN_ARTIKELANZEIGEFILTER_LAGERNULL) {
        $lagerFilter = "AND (NOT (tartikel.fLagerbestand <= 0 AND tartikel.cLagerBeachten = 'Y') OR tartikel.cLagerKleinerNull = 'Y' OR tartikel.cLagerVariation = 'Y')";
      }
      $aRes = $this->db->queryPrepared(
        "SELECT txsell.kXSellArtikel from txsell, tartikel 
        WHERE txsell.kXSellArtikel = tartikel.kArtikel $lagerFilter AND txsell.kArtikel = :kArtikel ORDER BY tartikel.cName",
        ['kArtikel' => $this->oExportArticle->kArtikel],
        ReturnType::ARRAY_OF_OBJECTS
      );
      foreach ($aRes as $xs) {
        $relationModel = new Shopgate_Model_Catalog_Relation();
        $relationModel->setType('upsell');
        $relationModel->addValue($xs->kXSellArtikel);
        $productModel->addRelation($relationModel);
      }
    }
    if ($this->aTeinstellungenFields['artikeldetails_xselling_kauf_anzeigen'] == 'Y') {
      $aRes = $this->db->queryPrepared(
        "SELECT kXSellArtikel FROM txsellkauf 
        WHERE kArtikel= :kArtikel ORDER BY nAnzahl DESC, RAND() LIMIT ".$this->aTeinstellungenFields['artikeldetails_xselling_kauf_anzahl'],
        ['kArtikel' => $this->oExportArticle->kArtikel],
        ReturnType::ARRAY_OF_OBJECTS
      );
      foreach ($aRes as $xs) {
        $relationModel = new Shopgate_Model_Catalog_Relation();
        $relationModel->setType('ordered_with');
        $relationModel->addValue($xs->kXSellArtikel);
        $productModel->addRelation($relationModel);
      }
    }
    
    $visibilityItem = new Shopgate_Model_Catalog_Visibility();
    if ($this->oExportArticle->cStatusAktiv == 'inactive') {
      $visibilityItem->setLevel(Shopgate_Model_Catalog_Visibility::DEFAULT_VISIBILITY_NOTHING);
      $visibilityItem->setMarketplace(0);
    } else {
      $visibilityItem->setLevel(Shopgate_Model_Catalog_Visibility::DEFAULT_VISIBILITY_CATALOG_AND_SEARCH);
      $visibilityItem->setMarketplace(1);
    }
    $productModel->setVisibility($visibilityItem);
    
    if (isset($this->oExportArticle->aInputAttribute) && !empty($this->oExportArticle->aInputAttribute)) {
      foreach ($this->oExportArticle->aInputAttribute as $value) {
        $inputItem = new Shopgate_Model_Catalog_Input();
        $inputItem->setType(Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_TEXT);
        $inputItem->setUid($value['kEigenschaft']);
        $inputItem->setRequired($value['nRequired']);
        $inputItem->setLabel($this->_prepare_utf8($value['cName']));
        $inputItem->setInfoText($this->_prepare_utf8($value['cInfoText']));
        $inputItem->setAdditionalPrice(0);
        
        $validationItem = new Shopgate_Model_Catalog_Validation();
        $validationItem->setValidationType(Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_STRING);
        $inputItem->setValidation($validationItem);
        $productModel->addInput($inputItem);
      }
    }
    
    if (isset($this->oExportArticle->aSelectAttribute) && !empty($this->oExportArticle->aSelectAttribute)) {
      foreach ($this->oExportArticle->aSelectAttribute as $value) {
        $inputItem = new Shopgate_Model_Catalog_Input();
        $inputItem->setType(Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT);
        $inputItem->setUid($value['kEigenschaft']);
        $inputItem->setRequired($value['nRequired']);
        $inputItem->setLabel($this->_prepare_utf8($value['cName']));
        //$inputItem->setInfoText($this->_prepare_utf8($value['cInfoText']));// Habe ich einen Infotext fr die Attribute? in aArtikel prfen!
        $sold_out = count($value['aWerte']);
        foreach ($value['aWerte'] as $row) {
          if ($this->oExportArticle->cStatusAktiv == 'active' || ($this->oExportArticle->cStatusAktiv == 'stock' && ($row['fLagerbestand'] > 0 || $this->oExportArticle->cLagerVariation == 'N'))) {
            if (!empty($row['kEigenschaftWert'])) {
              $optionModel = new Shopgate_Model_Catalog_Option();
              $optionModel->setUid($row['kEigenschaftWert']);
              $optionModel->setLabel($this->_prepare_utf8($row['cName']));
              $optionModel->setValue($row['kEigenschaftWert']);
              $optionModel->setAdditionalPrice($row['fAufpreisNetto']);
              $inputItem->addOption($optionModel);
            }
            else {
              $sold_out--;
            }
          }
          else {
            $sold_out--;
          }
        }
        
        $validationItem = new Shopgate_Model_Catalog_Validation();
        $validationItem->setValidationType(Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_INT);
        $inputItem->setValidation($validationItem);
        if ($sold_out > 0) {
          $productModel->addInput($inputItem);
        }
      }
    }
    
    /* BOF Konfigurator */
    if ($this->bAllowKonfiguration && $this->oExportArticle->bHasKonfig && !empty($this->oExportArticle->oKonfig_arr)) {
      //var_dump($this->oExportArticle->oKonfig_arr); exit;
      foreach ($this->oExportArticle->oKonfig_arr as $oKonfigGruppe) {
        $nKonfigType = $oKonfigGruppe->getAnzeigeTyp();
        /*
          $nKonfigType => 0 == checkbox
            Kunde aktiviert checkbox und kann dann einen oder mehrere Artikel daraus whlen (getMin,getMax), mit unterschiedlichen Mengen je Artikel (Mengen je Auswahl sind bei Shopgate nicht abbildbar
            
          $nKonfigType => 1 == radio
            Kunde aktiviert checkbox und kann dann genau einen Artikel daraus whlen, mit unterschiedlichen Mengen je Artikel (Mengen je Auswahl sind bei Shopgate nicht abbildbar
            
          $nKonfigType => 2 == select
            Kunde aktiviert checkbox und kann dann genau einen Artikel daraus whlen, mit unterschiedlichen Mengen je Artikel (Mengen je Auswahl sind bei Shopgate nicht abbildbar
            
          $nKonfigType => 3 == multi-select
            Kunde aktiviert checkbox und kann dann einen oder mehrere Artikel daraus whlen (getMin,getMax), mit unterschiedlichen Mengen je Artikel (Mengen je Auswahl sind bei Shopgate nicht abbildbar
        */
        if ($nKonfigType == 0 || $nKonfigType == 3) {
          // mehrfachauswahl
          $oGruppeSprache = $oKonfigGruppe->getSprache();
          $cnt_oKonfigGruppe = count($oKonfigGruppe->oItem_arr);
          foreach ($oKonfigGruppe->oItem_arr as $oKonfigItem) {
            $inputItem = new Shopgate_Model_Catalog_Input();
            $inputItem->setType(Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT);
            $inputItem->setUid('agiKonfig_'.$oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem());
            $inputItem->setRequired(1);
            $inputItem->setLabel($this->_prepare_utf8($oGruppeSprache->getName()));
            if ($oGruppeSprache->hatBeschreibung()) {
              $inputItem->setInfoText($this->_prepare_utf8($oGruppeSprache->getBeschreibung()));
            }
            if ($cnt_oKonfigGruppe > 1 || ($oKonfigGruppe->getMin() == 0 && $cnt_oKonfigGruppe)) {
              // add 0-Option
              $optionModel = new Shopgate_Model_Catalog_Option();
              $optionModel->setUid($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_0_0');
              $optionModel->setLabel('---');
              $optionModel->setValue($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_0_0');
              $optionModel->setAdditionalPrice(0);
              $inputItem->addOption($optionModel);
            }
            $cnt = 0;
            for ($min = $oKonfigItem->getMin(), $max = $oKonfigItem->getMax(); $min<=$max; $min++) {
              $oItemSprache = $oKonfigItem->getSprache();
              $optionModel = new Shopgate_Model_Catalog_Option();
              $optionModel->setUid($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_'.$oKonfigItem->getArtikelKey().'_'.$min);
              $optionModel->setLabel($this->_prepare_utf8($min.'x '.$oKonfigItem->getName()));
              $optionModel->setValue($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_'.$oKonfigItem->getArtikelKey().'_'.$min);
              $fItemPrice = $oKonfigItem->getPreis()*$min;
              if($fItemPrice <> 0) {
                $optionModel->setAdditionalPrice(round($fItemPrice*100/(100+$this->oExportArticle->fMwSt) *$this->oWaehrung->fFaktor,4));
              }
              else {
                $optionModel->setAdditionalPrice(0);
              }
              $inputItem->addOption($optionModel);
              $cnt++;
              if ($cnt > 10) {
                break;
              }
            }
            /* add validation */
            $validationItem = new Shopgate_Model_Catalog_Validation();
            $validationItem->setValidationType(Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_STRING);
            //$validationItem->setValue('^\d+_\d+_\d+$');
            $inputItem->setValidation($validationItem);
            
            $productModel->addInput($inputItem);
          }
        }
        else {
          // einfachauswahl select
          $oGruppeSprache = $oKonfigGruppe->getSprache();
          $inputItem = new Shopgate_Model_Catalog_Input();
          $inputItem->setType(Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT);
          $inputItem->setUid('agiKonfig_'.$oKonfigGruppe->getKonfiggruppe());
          $inputItem->setRequired($oKonfigGruppe->getMin() > 0 ? 1 : 0);
          $inputItem->setLabel($this->_prepare_utf8($oGruppeSprache->getName()));
          if ($oGruppeSprache->hatBeschreibung()) {
            $inputItem->setInfoText($this->_prepare_utf8($oGruppeSprache->getBeschreibung()));
          }
          foreach ($oKonfigGruppe->oItem_arr as $oKonfigItem) {
            $oItemSprache = $oKonfigItem->getSprache();
            $min = $oKonfigItem->getMin();
            $max = $oKonfigItem->getMax();
            if ($max > $min) {
              $cnt = 0;
              while ($min<=$max) {
                $optionModel = new Shopgate_Model_Catalog_Option();
                $optionModel->setUid($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_'.$oKonfigItem->getArtikelKey().'_'.$min);
                $optionModel->setLabel($min.'x '.$this->_prepare_utf8($oKonfigItem->getName()));
                $optionModel->setValue($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_'.$oKonfigItem->getArtikelKey().'_'.$min);
                $fItemPrice = $oKonfigItem->getPreis()*$min;
                if($fItemPrice <> 0) {
                  $optionModel->setAdditionalPrice(round($fItemPrice*100/(100+$this->oExportArticle->fMwSt) *$this->oWaehrung->fFaktor,4));
                }
                else {
                  $optionModel->setAdditionalPrice(0);
                }
                $inputItem->addOption($optionModel);
                $min++;
                $cnt++;
                if ($cnt > 10) {
                  break;
                }
              }
            } else {
              $optionModel = new Shopgate_Model_Catalog_Option();
              $optionModel->setUid($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_'.$oKonfigItem->getArtikelKey().'_'.$min);
              $optionModel->setLabel($this->_prepare_utf8($oKonfigItem->getName()));
              $optionModel->setValue($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_'.$oKonfigItem->getArtikelKey().'_'.$min);
              $fItemPrice = $oKonfigItem->getPreis()*$min;
              if($fItemPrice <> 0) {
                $optionModel->setAdditionalPrice(round($fItemPrice*100/(100+$this->oExportArticle->fMwSt) *$this->oWaehrung->fFaktor,4));
              }
              else {
                $optionModel->setAdditionalPrice(0);
              }
              $inputItem->addOption($optionModel);
            }
          }
          $validationItem = new Shopgate_Model_Catalog_Validation();
          $validationItem->setValidationType(Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_STRING);
          //$validationItem->setValue('^\d+_\d+_\d+$');
          $inputItem->setValidation($validationItem);
          
          $productModel->addInput($inputItem);
        }
      }
    }
    /* BOF Konfigurator */
    
    // Jetzt schauen, ob es Vater-Kind-Artikel gibt, wenn ja, die Attribute abgreifen und ausgeben
    if (isset($this->oExportArticle->nIstVater) && $this->oExportArticle->nIstVater == 1 && !$thisIsChild) {
      if (isset($this->oExportArticle->oVariationenNurKind_arr) && !empty($this->oExportArticle->oVariationenNurKind_arr)) {
        foreach ($this->oExportArticle->oVariationenNurKind_arr as $row) {
          if ($this->oExportArticle->nIstVater) {
            $attributeGroupItem = new Shopgate_Model_Catalog_AttributeGroup();
            $attributeGroupItem->setUid($row->kEigenschaft);
            $attributeGroupItem->setLabel($this->_prepare_utf8($row->cName));
            $productModel->addAttributeGroup($attributeGroupItem);
          }
        }
      }
      
      // Kinder auslesen
      $query = "SELECT DISTINCT tartikel.kArtikel
        FROM teigenschaftkombiwert
        JOIN tartikel ON tartikel.kEigenschaftKombi = teigenschaftkombiwert.kEigenschaftKombi
            AND tartikel.kVaterArtikel = :kArtikel
        LEFT JOIN tartikelsichtbarkeit ON tartikel.kArtikel = tartikelsichtbarkeit.kArtikel
            AND tartikelsichtbarkeit.kKundengruppe= :kKundengruppe
        WHERE tartikelsichtbarkeit.kArtikel IS NULL
        ORDER BY tartikel.kArtikel, teigenschaftkombiwert.kEigenschaft";
      $aRes = $this->db->queryPrepared(
        $query,
        ['kArtikel' => $this->oExportArticle->kArtikel, 'kKundengruppe'=>$this->kKundengruppe],
        ReturnType::ARRAY_OF_OBJECTS
      );
      if(is_array($aRes) && count($aRes) > 0) {
        foreach ($aRes as $row) {
          $this->_loadOExportArticle($row->kArtikel);
          $childItem = $this->_AGIBuildItemObject($row->kArtikel);
          $productModel->addChild($childItem);
        }
      }
    }
    elseif ($thisIsChild) {
      if (isset($this->oExportArticle->oVariationenNurKind_arr) && !empty($this->oExportArticle->oVariationenNurKind_arr)) {
        foreach ($this->oExportArticle->oVariationenNurKind_arr as $row) {
          if(!empty($this->oExportArticle->kVaterArtikel) && !empty($row->Werte[0])) {
            $attributeItem = new Shopgate_Model_Catalog_Attribute();
            $attributeItem->setUid($row->Werte[0]->kEigenschaftWert);
            $attributeItem->setGroupUid($row->kEigenschaft);
            $attributeItem->setLabel($this->_prepare_utf8($this->_remove_html($row->Werte[0]->cName)));
            $productModel->addAttribute($attributeItem);
          }
        }
      }
    }
    if (!$thisIsChild) {
      $this->addItemModel($productModel);
    }
    else {
      return $productModel;
    }
  }
  
  private function add_Shopgate_Model_Catalog_Price($thisIsChild)
  {
    $allow_tier_prices = true;
    if (isset($this->oExportArticle->nIstVater) && $this->oExportArticle->nIstVater == 1 && !$thisIsChild) {
      $allow_tier_prices = false;
    }
    
    $priceItem = new Shopgate_Model_Catalog_Price();
    $priceItem->setType(Shopgate_Model_Catalog_Price::DEFAULT_PRICE_TYPE_NET);
    
    $agi_calc_price = $this->oExportArticle->Preise->fVK[1];
    if (!empty($this->oExportArticle->Preise->alterVK[1]) && $this->oExportArticle->Preise->alterVK[1] > $this->oExportArticle->Preise->fVK[1] && isset($this->oExportArticle->Preise->Sonderpreis_aktiv) && $this->oExportArticle->Preise->Sonderpreis_aktiv) {
      $priceItem->setPrice($this->oExportArticle->Preise->alterVK[1]);
      $priceItem->setSalePrice($this->oExportArticle->Preise->fVK[1]);
    }
    elseif(!empty($this->oExportArticle->Preise->alterVK[1]) && $this->oExportArticle->Preise->alterVK[1] > $this->oExportArticle->Preise->fVK[1]) {
      $priceItem->setPrice($this->oExportArticle->Preise->alterVK[1]);
      $priceItem->setSalePrice($this->oExportArticle->Preise->fVK[1]);
    }
    else {
      $priceItem->setPrice($this->oExportArticle->Preise->fVK[1]);
    }
    if (isset($this->oExportArticle->fUVP)) {
      $priceItem->setMsrp(round($this->oExportArticle->fUVP*100/(100+$this->oExportArticle->fMwSt) *$this->oWaehrung->fFaktor,4));
    }
    $priceItem->setMinimumOrderAmount(0);
    
    if ($this->oExportArticle->cVPE == 'Y') {
      $priceItem->setBasePrice($this->_prepare_utf8(str_replace('&euro;', 'EURO',$this->oExportArticle->cLocalizedVPE[0])));
    }
    
    /* BOF Staffelpreis */
    $hatPreisstaffel = false;
    if (!empty($this->oExportArticle->Preise->fStaffelpreis_arr)) {
      $hatPreisstaffel = true;
    }
    
    if (!$hatPreisstaffel && count($this->aKundengruppen) > 1) {
      foreach ($this->aKundengruppen as $kKundengruppe) {
        $preise = new Preise($kKundengruppe, $this->oExportArticle->kArtikel);
        $preise->rabbatierePreise($this->oExportArticle->getDiscount($kKundengruppe, $this->oExportArticle->kArtikel));
        if (!empty($preise->fStaffelpreis_arr)) {
          $hatPreisstaffel = true;
          break;
        }
      }
    }
    if ($allow_tier_prices && (count($this->aKundengruppen) > 1 || $hatPreisstaffel)) {
      foreach ($this->aKundengruppen as $kKundengruppe) {
        $preise = new Preise($kKundengruppe, $this->oExportArticle->kArtikel);
        $preise->rabbatierePreise($this->oExportArticle->getDiscount($kKundengruppe, $this->oExportArticle->kArtikel));
        $preise->localizePreise();
      
        if (!empty($preise->fVK[1]) && $agi_calc_price-$preise->fVK[1] <> 0) {
          $tierPriceItem = new Shopgate_Model_Catalog_TierPrice();
          $tierPriceItem->setAggregateChildren(0);
          $tierPriceItem->setCustomerGroupUid($kKundengruppe);
          $tierPriceItem->setReduction($agi_calc_price-$preise->fVK[1]);
          $tierPriceItem->setFromQuantity(1);
          $tierPriceItem->setReductionType(Shopgate_Model_Catalog_TierPrice::DEFAULT_TIER_PRICE_TYPE_FIXED);
          $priceItem->addTierPriceGroup($tierPriceItem);
        }
        if ($hatPreisstaffel) {
          foreach($preise->nAnzahl_arr as $kAnz => $nAnz) {
            if (!empty($nAnz)) {
              if ($agi_calc_price-$preise->fStaffelpreis_arr[$kAnz][1] <> 0) {
                $tierPriceItem = new Shopgate_Model_Catalog_TierPrice();
                $tierPriceItem->setAggregateChildren(0);
                $tierPriceItem->setCustomerGroupUid($kKundengruppe);
                $tierPriceItem->setFromQuantity($nAnz);
                //$tierPriceItem->setToQuantity(20);
                $tierPriceItem->setReduction($agi_calc_price-$preise->fStaffelpreis_arr[$kAnz][1]);
                $tierPriceItem->setReductionType(Shopgate_Model_Catalog_TierPrice::DEFAULT_TIER_PRICE_TYPE_FIXED);
                $priceItem->addTierPriceGroup($tierPriceItem);
              }
            }
          }
        }
      }
    }
    /* EOF Staffelpreis */
    return $priceItem;
  }
  
  protected function _AGIoExportArticleAddShippingCost()
  {
    $this->oExportArticle->cLieferland        = $this->cLieferLand;
    $this->oExportArticle->nOhneVersandkosten = 0;
    $this->oExportArticle->fVersandkosten     = 0;
    
    if ($this->oExportArticle->kVersandklasse != $this->JTLPluginConf['nStandardVersandklasse']) {
      if (!array_key_exists($this->oExportArticle->kVersandklasse, $this->aVersandart))
      {
        $this->aVersandart[$this->oExportArticle->kVersandklasse] = $this->db->queryPrepared(
          "SELECT * FROM tversandart WHERE cNurAbhaengigeVersandart= :cNurAbhaengigeVersandart AND cLaender LIKE :cLieferLandLike AND (
            cVersandklassen='-1' OR 
            (cVersandklassen LIKE :cVersandklassenLike1 OR cVersandklassen LIKE :cVersandklassenLike2)
          ) AND (cKundengruppen='-1' or cKundengruppen LIKE :cKundengruppenLike) ORDER BY nSort",
          ['cNurAbhaengigeVersandart' => $this->cNurAbhaengigeVersandart,'cLieferLandLike' => '%'.$this->cLieferLand.'%', 'cVersandklassenLike1' => '% '.$this->oExportArticle->kVersandklasse.' %', 'cVersandklassenLike2' => '% '.$this->oExportArticle->kVersandklasse, 'cKundengruppenLike' => '%;'.(int)$this->kKundengruppe.';%'],
          ReturnType::ARRAY_OF_OBJECTS
        );
      }
      $price = 0;
      $fStandardVersand = 0;
      foreach ($this->aVersandart[$this->JTLPluginConf['nStandardVersandklasse']] as $shippingMethod) {
        $tmp = ShippingMethod::calculateShippingFees($shippingMethod, $this->cLieferLand, null, $this->oExportArticle);
        if ($tmp != -1) {
          $fStandardVersand = $tmp;
          break;
        }
      }
      
      $fArtikelVersand = 0;
      foreach ($this->aVersandart[$this->oExportArticle->kVersandklasse] as $shippingMethod) {
        $tmp = ShippingMethod::calculateShippingFees($shippingMethod, $this->cLieferLand, null, $this->oExportArticle);
        if ($tmp != -1) {
          if ($tmp > 0 && $tmp < $fArtikelVersand) {
            $fArtikelVersand = $tmp;
          }
          elseif ($fArtikelVersand == 0 && $tmp > 0) {
            $fArtikelVersand = $tmp;
          }
          break;
        }
      }
      $differenz = $fArtikelVersand - $fStandardVersand;
      if ($differenz > 0) {
        $differenz = $differenz*100/(100+$this->oExportArticle->fUst);
      }
      $this->oExportArticle->fVersandkosten = $differenz < 0 ? 0 : round($differenz,3);
    }
  }
  
  protected function _AGIoExportArticleAddCategories()
  {
    $this->oExportArticle->aAGIKategorien = array();
    if(count($this->oExportArticle->oKategorie_arr) > 0) {
      foreach ($this->oExportArticle->oKategorie_arr as $oCategorie)
      {
        if (is_int($oCategorie)) {
          $oKat = new Kategorie($oCategorie, $this->kSprache, $this->kKundengruppe);
          $kat = array(
            'kKategorie'    => $oKat->kKategorie,
            'cName'         => $oKat->cName,
            'nSort'         => $oKat->nSort,
            'kOberKategorie'=> $oKat->kOberKategorie,
            'cDeeplink'     => $oKat->cURL,
            'paths'         => array(),
            'image'         => $oKat->nBildVorhanden ? $oKat->cBild : ''
          );
          $i = 1;
          foreach ($oKat->cKategoriePfad_arr as $value) {
            $kat['paths'][$i] = $value;
            $i++;
          }
        }
        else {
          $kat = array(
            'kKategorie'    => $oCategorie->kKategorie,
            'cName'         => $oCategorie->cName,
            'nSort'         => $oCategorie->nSort,
            'kOberKategorie'=> $oCategorie->kOberKategorie,
            'cDeeplink'     => $oCategorie->cURL,
            'paths'         => array(),
            'image'         => $oCategorie->nBildVorhanden ? $oCategorie->cBild : ''
          );
          $i = 1;
          foreach ($oCategorie->cKategoriePfad_arr as $value) {
            $kat['paths'][$i] = $value;
            $i++;
          }
        }
        $this->oExportArticle->aAGIKategorien[] = $kat;
      }
    }
  }
  
  protected function _AGIoExportArticleCheckHighlight()
  {
    if (isset($this->JTLPluginConf['cTopAtFirst']) && $this->JTLPluginConf['cTopAtFirst'] == 'Y') {
      $this->oExportArticle->nTopArtikel = $this->oExportArticle->cTopArtikel == 'Y' ? 1 : 0;
    }
    else {
      $this->oExportArticle->nTopArtikel = 0;
    }
    
    $aRes = $this->db->queryPrepared(
      "SELECT COUNT(*) AS cnt FROM tartikelsonderpreis WHERE kArtikel= :kArtikel AND cAktiv='Y' AND dStart <= DATE(NOW()) AND (dEnde >= DATE(NOW()) OR dEnde='0000-00-00')",
      ['kArtikel' => $this->oExportArticle->kArtikel],
      ReturnType::SINGLE_OBJECT
    );
    if (!empty($aRes->icount)) {
      $this->oExportArticle->nTopArtikel = 1;
    }
  }
  
  protected function _AGIoExportArticleCheckStatusAktiv()
  {
    if (!empty($this->oExportArticle->FunktionsAttribute)) {
      if (array_key_exists('shopgate_active',$this->oExportArticle->FunktionsAttribute) && $this->oExportArticle->FunktionsAttribute['shopgate_active'] == 'no') {
        $this->oExportArticle->cStatusAktiv = 'inactive';
        return true;
      }
    }
    if (isset($this->oExportArticle->nErscheinendesProdukt) && $this->oExportArticle->nErscheinendesProdukt) {
      if ($this->aTeinstellungenFields['global_erscheinende_kaeuflich'] == 'Y') {
        $this->oExportArticle->cStatusAktiv = 'active';
        $this->oExportArticle->fLagerbestand= 0;
        $this->oExportArticle->nUseStock = 0;
      }
      else {
        $this->oExportArticle->cStatusAktiv = 'active';
        $this->oExportArticle->fLagerbestand= 0;
        $this->oExportArticle->nUseStock = 1;
      }
      return true;
    }
    if (!empty($this->oExportArticle->kLieferstatus) && !empty($this->JTLPluginConf['aInaktivStatus'])) {
      if (in_array($this->oExportArticle->kLieferstatus, $this->JTLPluginConf['aInaktivStatus'])) {
        $this->oExportArticle->cStatusAktiv = 'inactive';
        $this->oExportArticle->fLagerbestand= 0;
        $this->oExportArticle->nUseStock = 1;
        return true;
      }
    }
    if (isset($this->oExportArticle->inWarenkorbLegbar) && $this->oExportArticle->inWarenkorbLegbar == -1) {
      $this->oExportArticle->cStatusAktiv = 'inactive';
      $this->oExportArticle->fLagerbestand= 0;
      $this->oExportArticle->nUseStock = 1;
      return true;
    }
    if (isset($this->oExportArticle->cLagerBeachten)) {
      if ($this->oExportArticle->cLagerBeachten == 'Y') {
        if (isset($this->oExportArticle->cLagerKleinerNull) && $this->oExportArticle->cLagerKleinerNull == 'Y') {
          $this->oExportArticle->cStatusAktiv = 'active';
          $this->oExportArticle->nUseStock = 0;
        }
        else {
          $this->oExportArticle->cStatusAktiv = 'stock';
          $this->oExportArticle->nUseStock = 1;
        }
      }
      else {
        $this->oExportArticle->cStatusAktiv = 'active';
        $this->oExportArticle->nUseStock = 0;
      }
    }
    else {
      $this->oExportArticle->cStatusAktiv = 'stock';
      $this->oExportArticle->nUseStock = 1;
    }
    
    if ($this->oExportArticle->fLagerbestand < 0) {
      $this->oExportArticle->fLagerbestand = 0;
    }
    return true;
  }
  
  protected function _AGIoExportArticleAddGrundpreis()
  {
    $this->oExportArticle->Grundpreis = '';
    if ($this->oExportArticle->cVPE == 'Y' && $this->oExportArticle->fVPEWert > 0) {
      $this->oExportArticle->Grundpreis = $this->oExportArticle->cLocalizedVPE[1];
    }
  }
  
  protected function _AGIoExportArticleAddAttributes()
  {
    $this->oExportArticle->aInputAttribute = array();
    $this->oExportArticle->aSelectAttribute = array();
    $aVaterKindEigenschaftenKeys = array();
    if (isset($this->oExportArticle->oVariationenNurKind_arr) && !empty($this->oExportArticle->oVariationenNurKind_arr)) {
      foreach ($this->oExportArticle->oVariationenNurKind_arr as $row) {
        $aVaterKindEigenschaftenKeys[] = $row->kEigenschaft;
      }
    }
    if (isset($this->oExportArticle->Variationen) && !empty($this->oExportArticle->Variationen) && !$this->oExportArticle->nIstVater) {
      foreach ($this->oExportArticle->Variationen as $variation) {
        if ($variation->nLieferbareVariationswerte <= 0 || in_array($variation->kEigenschaft, $aVaterKindEigenschaftenKeys)) {
          continue;
        }
        
        if (strpos(strtolower($variation->cTyp), 'freifeld') !== FALSE) {
          $data = array(
            'kEigenschaft'  => $variation->kEigenschaft,
            'cType'         => 'text',
            'cName'         => $variation->cName,
            'cInfoText'     => '',
            'nRequired'     => strpos(strtolower($variation->cTyp), 'pflicht-freifeld') !== FALSE ? 1 : 0
          );
          $this->oExportArticle->aInputAttribute[$variation->kEigenschaft] = $data;
        }
        else {
          $data = array(
            'kEigenschaft'  => $variation->kEigenschaft,
            'cType'         => 'select',
            'cName'         => $variation->cName,
            'aWerte'        => array(),
            'nRequired'     => 1
          );
          foreach ($variation->Werte as $value) {
            $dataWert = array(
              'kEigenschaftWert'  => $value->kEigenschaftWert,
              'cName'             => $value->cName,
              'fAufpreisNetto'    => isset($value->fAufpreis) ? $value->fAufpreis[1] *$this->oWaehrung->fFaktor : 0,
              'fAufpreisBrutto'   => isset($value->fAufpreis) ? $value->fAufpreis[0] *$this->oWaehrung->fFaktor : 0,
              'cArtNr'            => empty($value->cArtNr) ? '' : $value->cArtNr,
              'fGewichtDiff'      => empty($value->fGewichtDiff) ? 0 : $value->fGewichtDiff,
              'fLagerbestand'     => empty($value->fLagerbestand) ? 0 : $value->fLagerbestand
            );
            $data['aWerte'][$value->kEigenschaftWert] = $dataWert;
          }
          if (!empty($data['aWerte'])) {
            $this->oExportArticle->aSelectAttribute[$variation->kEigenschaft] = $data;
          }
        }
      }
    }
  }
  
  protected function _AGIoExportArticleSetLieferstatus()
  {
    $this->oExportArticle->agiLieferstatus = $this->oExportArticle->cLieferstatus;
    if (isset($this->oExportArticle->nErscheinendesProdukt) && $this->oExportArticle->nErscheinendesProdukt) 
    {
      $this->oExportArticle->agiLieferstatus = $this->tSprache['productAvailableFrom'].': '.date('d.m.Y',strtotime($this->oExportArticle->dErscheinungsdatum));
      if ($this->aTeinstellungenFields['global_erscheinende_kaeuflich'] == 'Y') {
        $this->oExportArticle->agiLieferstatus .= ' ('.$this->tSprache['preorderPossible'].')';
      }
      if (isset($this->oExportArticle->cEstimatedDelivery) && !empty($this->oExportArticle->cEstimatedDelivery)) {
        if (array_key_exists('shippingTime', $this->tSprache)) {
          $this->oExportArticle->agiLieferstatus .= ', '.$this->tSprache['shippingTime'].': '.$this->oExportArticle->cEstimatedDelivery;
        }
        else {
          $this->oExportArticle->agiLieferstatus .= ', '.$status_text[] = $this->oExportArticle->cEstimatedDelivery;
        }
      }
      return;
    }
    
    if (isset($this->JTLPluginConf['cNehmeLieferzeit']) && $this->JTLPluginConf['cNehmeLieferzeit'] == 'kombiampelstatuszeit')
    {
      $status_text = array();
      if (!$this->oExportArticle->nErscheinendesProdukt) {
        $anzeige = $this->aTeinstellungenFields['artikel_lagerbestandsanzeige'];
        if (
          $anzeige != "nichts" && $this->oExportArticle->cLagerBeachten == 'Y' && 
          ($this->oExportArticle->cLagerKleinerNull == 'N' || $this->aTeinstellungenFields['artikeldetails_lieferantenbestand_anzeigen'] == 'U') && 
          $this->oExportArticle->fLagerbestand <= 0 && $this->oExportArticle->fZulauf > 0 && isset($this->oExportArticle->dZulaufDatum_de)
        ) {
          if (array_key_exists('productInflowing', $this->tSprache)) {
            $s_count = substr_count($this->tSprache['productInflowing'], '%s');
            if ($s_count == 2) {
              $status_text[] = sprintf($this->tSprache['productInflowing'], $this->oExportArticle->fZulauf, $this->oExportArticle->dZulaufDatum_de);
            }
            elseif ($s_count == 1) {
              $status_text[] = sprintf($this->tSprache['productInflowing'], $this->oExportArticle->dZulaufDatum_de);
            }
            else {
              $status_text[] = $this->tSprache['productInflowing'];
            }
          }
        }
        elseif (
          $anzeige != "nichts" && $this->aTeinstellungenFields['artikeldetails_lieferantenbestand_anzeigen'] != 'N' &&
          $this->oExportArticle->cLagerBeachten == 'Y' && $this->oExportArticle->fLagerbestand <= 0 && 
          $this->oExportArticle->fLieferantenlagerbestand > 0 && $this->oExportArticle->fLieferzeit > 0 &&
          ($this->oExportArticle->cLagerKleinerNull == 'N' || $this->aTeinstellungenFields['artikeldetails_lieferantenbestand_anzeigen'] == 'U')
        ) {
          if (array_key_exists('supplierStockNotice', $this->tSprache)) {
            $s_count = substr_count($this->tSprache['supplierStockNotice'], '%s');
            if ($s_count == 1) {
              $status_text[] = sprintf($this->tSprache['supplierStockNotice'], $this->oExportArticle->fLieferzeit);
            }
            else {
              $status_text[] = $this->tSprache['supplierStockNotice'];
            }
          }
        }
        elseif ($anzeige=='verfuegbarkeit' || $anzeige=='genau') {
          $status_text[] = $this->oExportArticle->Lageranzeige->cLagerhinweis[$anzeige];
        }
        elseif ($anzeige=='ampel') {
          $status_text[] = $this->oExportArticle->Lageranzeige->AmpelText;
        }
        
        if (
          !empty($this->oExportArticle->cLieferstatus) &&
          (
            $this->aTeinstellungenFields['artikeldetails_lieferstatus_anzeigen'] == 'Y' || 
            (
              $this->aTeinstellungenFields['artikeldetails_lieferstatus_anzeigen'] == 'L' && $this->oExportArticle->fLagerbestand == 0 && $this->oExportArticle->cLagerBeachten == 'Y' 
            ) || 
            (
              $this->aTeinstellungenFields['artikeldetails_lieferstatus_anzeigen'] == 'A' && 
              (
                $this->oExportArticle->fLagerbestand > 0 || $this->oExportArticle->cLagerKleinerNull == 'Y' || $this->oExportArticle->cLagerBeachten != 'Y'
              )
            )
          )
        )
        {
          if (array_key_exists('deliveryStatus', $this->tSprache)) {
            $status_text[] = $this->tSprache['deliveryStatus'].': '.$this->oExportArticle->cLieferstatus;
          }
          elseif (array_key_exists('shippingTimeLP', $this->tSprache)) {
            $status_text[] = $this->tSprache['shippingTimeLP'].': '.$this->oExportArticle->cLieferstatus;
          }
          else {
            $status_text[] = $this->oExportArticle->cLieferstatus;
          }
        }
        if (isset($this->oExportArticle->cEstimatedDelivery) && !empty($this->oExportArticle->cEstimatedDelivery)) {
          if (array_key_exists('shippingTime', $this->tSprache)) {
            $status_text[] = $this->tSprache['shippingTime'].': '.$this->oExportArticle->cEstimatedDelivery;
          }
          else {
            $status_text[] = $this->oExportArticle->cEstimatedDelivery;
          }
        }
        if (!empty($status_text)) {
          $this->oExportArticle->agiLieferstatus = implode(', ', $status_text);
        }
      }
    }
    elseif (isset($this->JTLPluginConf['cNehmeLieferzeit']) && $this->JTLPluginConf['cNehmeLieferzeit'] == 'kombistatuszeit')
    {
      //shippingTimeLP,shippingTime
      $status_text = array();
      if (!empty($this->oExportArticle->cLieferstatus))
      {
        if (array_key_exists('deliveryStatus', $this->tSprache)) {
          $status_text[] = $this->tSprache['deliveryStatus'].': '.$this->oExportArticle->cLieferstatus;
        }
        elseif (array_key_exists('shippingTimeLP', $this->tSprache)) {
          $status_text[] = $this->tSprache['shippingTimeLP'].': '.$this->oExportArticle->cLieferstatus;
        }
        else {
          $status_text[] = $this->oExportArticle->cLieferstatus;
        }
      }
      if (isset($this->oExportArticle->cEstimatedDelivery) && !empty($this->oExportArticle->cEstimatedDelivery))
      {
        if (array_key_exists('shippingTime', $this->tSprache)) {
          $status_text[] = $this->tSprache['shippingTime'].': '.$this->oExportArticle->cEstimatedDelivery;
        }
        else {
          $status_text[] = $this->oExportArticle->cEstimatedDelivery;
        }
      }
      if (!empty($status_text)) {
        $this->oExportArticle->agiLieferstatus = implode(', ', $status_text);
      }
    }
    elseif (isset($this->JTLPluginConf['cNehmeLieferzeit']) && $this->JTLPluginConf['cNehmeLieferzeit'] == 'ampel')
    {
      if (isset($this->oExportArticle->Lageranzeige) && isset($this->oExportArticle->Lageranzeige->AmpelText) && !empty($this->oExportArticle->Lageranzeige->AmpelText)) {
        $this->oExportArticle->agiLieferstatus = $this->oExportArticle->Lageranzeige->AmpelText;
      }
      elseif (isset($this->oExportArticle->Lageranzeige) && isset($this->oExportArticle->Lageranzeige->cLagerhinweis) && !empty($this->oExportArticle->Lageranzeige->cLagerhinweis)) {
        $this->oExportArticle->agiLieferstatus = $this->oExportArticle->Lageranzeige->cLagerhinweis['verfuegbarkeit'];
      }
    }
    elseif (isset($this->JTLPluginConf['cNehmeLieferzeit']) && $this->JTLPluginConf['cNehmeLieferzeit'] == 'zeit')
    {
      if (isset($this->oExportArticle->cEstimatedDelivery) && !empty($this->oExportArticle->cEstimatedDelivery)) {
        if (array_key_exists('shippingTime', $this->tSprache)) {
          $this->oExportArticle->agiLieferstatus = $this->tSprache['shippingTime'].': '.$this->oExportArticle->cEstimatedDelivery;
        }
        else {
          $this->oExportArticle->agiLieferstatus = $this->oExportArticle->cEstimatedDelivery;
        }
      }
    }
    
    if (empty($this->oExportArticle->agiLieferstatus))
    {
      if (isset($this->oExportArticle->Lageranzeige) && isset($this->oExportArticle->Lageranzeige->AmpelText) && !empty($this->oExportArticle->Lageranzeige->AmpelText)) {
        $this->oExportArticle->agiLieferstatus = $this->oExportArticle->Lageranzeige->AmpelText;
      }
      elseif (isset($this->oExportArticle->Lageranzeige) && isset($this->oExportArticle->Lageranzeige->cLagerhinweis) && !empty($this->oExportArticle->Lageranzeige->cLagerhinweis)) {
        $this->oExportArticle->agiLieferstatus = $this->oExportArticle->Lageranzeige->cLagerhinweis['verfuegbarkeit'];
      }
      elseif (isset($this->oExportArticle->cEstimatedDelivery) && !empty($this->oExportArticle->cEstimatedDelivery)) {
        if (array_key_exists('shippingTime', $this->tSprache)) {
          $this->oExportArticle->agiLieferstatus = $this->tSprache['shippingTime'].': '.$this->oExportArticle->cEstimatedDelivery;
        }
        else {
          $this->oExportArticle->agiLieferstatus = $this->oExportArticle->cEstimatedDelivery;
        }
      }
    }
  }
  
  protected function _AGIoExportArticleAddMediaFiles()
  {
    $query = "SELECT 
        tmediendatei.kMedienDatei, tmediendatei.cPfad, tmediendatei.cURL, tmediendatei.cTyp, tmediendatei.nSort,
        tmediendateisprache.cName, tmediendateisprache.cBeschreibung, tmediendateisprache.kSprache
      FROM tmediendatei
      JOIN tmediendateisprache ON tmediendateisprache.kMedienDatei = tmediendatei.kMedienDatei
      WHERE tmediendatei.kArtikel= :kArtikel
        AND tmediendateisprache.kSprache = :kSprache
      ORDER BY tmediendatei.nSort ASC, tmediendateisprache.cName";
    $this->oExportArticle->oMedienDatei_arr = $this->db->queryPrepared(
      $query,
      ['kArtikel' => $this->oExportArticle->kArtikel, 'kSprache' => $this->kSprache],
      ReturnType::ARRAY_OF_OBJECTS
    );
    if(is_array($this->oExportArticle->oMedienDatei_arr) && count($this->oExportArticle->oMedienDatei_arr) > 0) {
      $cMedienTyp_arr = array();  // Wird im Template gebraucht um Tabs aufzubauen
      foreach($this->oExportArticle->oMedienDatei_arr as $i => $oMedienDatei) {
        $this->oExportArticle->oMedienDatei_arr[$i]->nErreichbar = 0;
        $this->oExportArticle->oMedienDatei_arr[$i]->nExtern = 0;
        if((strlen($oMedienDatei->cPfad) > 0) && file_exists(PFAD_ROOT . PFAD_MEDIAFILES . $oMedienDatei->cPfad)) {
          $this->oExportArticle->oMedienDatei_arr[$i]->nErreichbar = 1;
          $this->oExportArticle->oMedienDatei_arr[$i]->cURL = $this->URL_SHOP.PFAD_MEDIAFILES;
        }
        elseif(strlen($oMedienDatei->cURL) > 0) {
          $this->oExportArticle->oMedienDatei_arr[$i]->nErreichbar = 1;
          $this->oExportArticle->oMedienDatei_arr[$i]->nExtern = 1;
        }
      }
    }
  }
  
  protected function _AGIoExportArticlePrepareDescription()
  {
    $this->oExportArticle->bAgiBenutzeKurzbeschreibung = false;
    if (empty($this->oExportArticle->cBeschreibung)) {
      $this->oExportArticle->bAgiBenutzeKurzbeschreibung = true;
      $this->oExportArticle->cBeschreibung = $this->oExportArticle->cKurzBeschreibung;
    }
    
    if (method_exists($this, 'agi_shopgate_connect__oExportArticlePrepareDescription_pos1')) {
      $this->agi_shopgate_connect__oExportArticlePrepareDescription_pos1();
    }
    
    $count_ps = substr_count($this->cWrapDescriptionAttribute, '%s');
    if (array_key_exists('artikeldetails_attribute_anhaengen', $this->aTeinstellungenFields) && $this->aTeinstellungenFields['artikeldetails_attribute_anhaengen'] == 'Y') {
      if (isset($this->oExportArticle->Attribute) && !empty($this->oExportArticle->Attribute)) {
        foreach ($this->oExportArticle->Attribute as $attribut) {
          if (in_array(strtolower($attribut->cName), $this->aRemoveDescAttribute)) {
            continue;
          }
          if ($count_ps === 2) {
            $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, Text::htmlentities($attribut->cName), ' '.$attribut->cWert).chr(10);
          }
          elseif ($count_ps === 1) {
            $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, $attribut->cWert).chr(10);
          }
          else {
            $this->oExportArticle->cBeschreibung .= Text::htmlentities($attribut->cName).': '.$attribut->cWert.chr(10);
          }
        }
      }
    }
    elseif (!empty($this->aAddDescAttribute)) {
      if (isset($this->oExportArticle->Attribute) && !empty($this->oExportArticle->Attribute)) {
        foreach ($this->oExportArticle->Attribute as $attribut) {
          if (in_array(strtolower($attribut->cName), $this->aAddDescAttribute)) {
            if ($count_ps === 2) {
              $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, Text::htmlentities($attribut->cName), ' '.$attribut->cWert).chr(10);
            }
            elseif ($count_ps === 1) {
              $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, $attribut->cWert).chr(10);
            }
            else {
              $this->oExportArticle->cBeschreibung .= Text::htmlentities($attribut->cName).': '.$attribut->cWert.chr(10);
            }
          }
        }
      }
    }
    if (method_exists($this, 'agi_shopgate_connect__addAttributeAssocToDesc')) {
      $this->agi_addAttributeAssocToDesc();
    }
    else {
      if (!empty($this->oExportArticle->AttributeAssoc) && !empty($this->aAddDescAttribute)) {
        $add_description = '';
        foreach ($this->aAddDescAttribute as $check_key) {
          if (array_key_exists($check_key, $this->oExportArticle->AttributeAssoc)) {
            $add_description .= $this->oExportArticle->AttributeAssoc[$check_key].chr(10);
          }
        }
        if (!empty($add_description)) {
          $this->oExportArticle->cBeschreibung .= $add_description;
        }
      }
    }
    
    if (method_exists($this, 'agi_shopgate_connect__oExportArticlePrepareDescription_pos2')) {
      $this->agi_shopgate_connect__oExportArticlePrepareDescription_pos2();
    }
    
    // Javascript entfernen
    $this->oExportArticle->cBeschreibung = preg_replace('/<script\b[^>]*>(.*?)<\/script>/is', "", $this->oExportArticle->cBeschreibung); // 2.06
    
    // URLs vervollstndigen
    $result = preg_match_all('/(src|href)=([\'"].*?[\'"])/i', $this->oExportArticle->cBeschreibung, $matches);
    if ($result && !empty($matches[2])) {
      $replaces = array('from'=>array(),'to'=>array());
      foreach ($matches[2] as $string) {
        $nSlash = strpos($string, '//');
        if (!preg_match("~https?://~i", $string) && ($nSlash === FALSE || $nSlash > 1)) {
          $replaces['from'][] = $string;
          if (substr($string,1,1) == '/') {
            $replaces['to'][] = substr($string,0,1).substr($this->URL_SHOP, 0, -1).substr($string,1);
          }
          else {
            if ($pos = strrpos($string,'./') !== false) {
              $string = substr($string,0,1).substr($string, $pos+2);
              $replaces['to'][] = substr($string,0,1).substr($this->URL_SHOP, 0, -1).substr($string,1);
            }
            else {
              $replaces['to'][] = substr($string,0,1).$this->URL_SHOP.substr($string,1);
            }
          }
        }
      }
      $this->oExportArticle->cBeschreibung = str_replace($replaces['from'], $replaces['to'], $this->oExportArticle->cBeschreibung);
    }
    $this->oExportArticle->cBeschreibung = str_replace(
      '<object type="application/x-shockwave-flash" data="/includes/',
      '<object type="application/x-shockwave-flash" data="'.$this->URL_SHOP.'includes/',
      $this->oExportArticle->cBeschreibung
    );
    $this->oExportArticle->cBeschreibung = str_replace(
      '<param name="movie" value="/includes/',
      '<param name="movie" value="'.$this->URL_SHOP.'includes/',
      $this->oExportArticle->cBeschreibung
    );
    $this->oExportArticle->cBeschreibung = str_replace(
      '<param name="FlashVars" value="flv=/mediafiles/',
      '<param name="FlashVars" value="flv='.$this->URL_SHOP.'mediafiles/',
      $this->oExportArticle->cBeschreibung
    );
    
    if (isset($this->oExportArticle->oMedienDatei_arr) && !empty($this->oExportArticle->oMedienDatei_arr)) {
      foreach($this->oExportArticle->oMedienDatei_arr as $i => $oMedienDatei) {
        if ($oMedienDatei->nErreichbar) {
          $link = '<a href="'.$oMedienDatei->cURL.'">'.$oMedienDatei->cURL.'</a>';
          if ($count_ps === 2)
            $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, Text::htmlentities($oMedienDatei->cName), ' '.$link).chr(10);
          elseif ($count_ps === 1)
            $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, $link).chr(10);
          else
            $this->oExportArticle->cBeschreibung .= Text::htmlentities($oMedienDatei->cName).': '.$link.chr(10);
        }
      }
    }
    if (method_exists($this, 'agi_shopgate_connect__oExportArticlePrepareDescription_pos3')) {
      $this->agi_shopgate_connect__oExportArticlePrepareDescription_pos3();
    }
  }
  
  protected function _getArticleSortInCategory($kKategorie)
  {
    $query = "SELECT ta.kArtikel AS id FROM tkategorieartikel AS tk LEFT JOIN tartikel AS ta ON ta.kArtikel=tk.kArtikel WHERE tk.kKategorie= :kKategorie ORDER BY ";
    if ($this->JTLPluginConf['cTopAtFirst'] == 'Y') {
      $query .= 'ta.cTopArtikel DESC, ';
    }
    $query .= 'ta.'.$this->JTLPluginConf['cSortArticle'].' '.$this->JTLPluginConf['cSortArticleOrder'].', ta.cName ASC';
    $aRes = $this->oExportArticle->oMedienDatei_arr = $this->db->queryPrepared(
      $query,
      ['kKategorie' => $kKategorie],
      ReturnType::ARRAY_OF_OBJECTS
    );
    $nSort = 999999;
    foreach ($aRes as $row) {
      if ($row->id == $this->oExportArticle->kArtikel) {
        break;
      }
      $nSort--;
    }
    return $nSort;
  }
  
  private function _AGICreateItemsConfig()
  {
    // standard-versandart
    $query =  "SELECT 
        *
      FROM 
        tversandart
      WHERE 
        cNurAbhaengigeVersandart= :cNurAbhaengigeVersandart
        AND cLaender LIKE :cLaender
        AND (
          cVersandklassen='-1' OR (cVersandklassen LIKE :cVersandklassen_1 or cVersandklassen LIKE :cVersandklassen_2)
        )
        AND (cKundengruppen='-1' or cKundengruppen LIKE :cKundengruppen)
      ORDER BY nSort
    ";
    $this->aVersandart[$this->JTLPluginConf['nStandardVersandklasse']] = $this->db->queryPrepared(
      $query,
      ['cNurAbhaengigeVersandart' => $this->cNurAbhaengigeVersandart, 'cLaender' => '%'.$this->cLieferLand.'%', 'cVersandklassen_1' => '% '.$this->JTLPluginConf['nStandardVersandklasse'].' %', 'cVersandklassen_2' => '% '.$this->JTLPluginConf['nStandardVersandklasse'], 'cKundengruppen' => '%;'.$this->kKundengruppe.';%'],
      ReturnType::ARRAY_OF_OBJECTS
    );
    
    // attribute, die nicht exportiert werden
    if (array_key_exists('cAttributNichtExportieren', $this->JTLPluginConf) && !empty($this->JTLPluginConf['cAttributNichtExportieren'])) {
      $tmp = explode(',',$this->JTLPluginConf['cAttributNichtExportieren']);
      foreach ($tmp as $value) {
        $value = trim($value);
        if (!empty($value)) {
          $this->aAttributNichtExportieren[] = strtolower($value);
        }
      }
    }
    
    // merkmale, die nicht exportiert werden
    if (array_key_exists('cMerkmaleNichtExportieren', $this->JTLPluginConf) && !empty($this->JTLPluginConf['cMerkmaleNichtExportieren'])) {
      $tmp = explode(',',$this->JTLPluginConf['cMerkmaleNichtExportieren']);
      foreach ($tmp as $value) {
        $value = trim($value);
        if (!empty($value)) {
          $this->aMerkmaleNichtExportieren[] = strtolower($value);
        }
      }
    }
    
    if (array_key_exists('cAttributeAnBeschreibung', $this->JTLPluginConf) && !empty($this->JTLPluginConf['cAttributeAnBeschreibung'])) {
      $tmp = explode(',',$this->JTLPluginConf['cAttributeAnBeschreibung']);
      foreach ($tmp as $value) {
        $value = trim($value);
        if (!empty($value)) {
          $this->aAddDescAttribute[] = strtolower($value);
        }
      }
    }
    if (array_key_exists('cAttributeNichtAnBeschreibung', $this->JTLPluginConf) && !empty($this->JTLPluginConf['cAttributeNichtAnBeschreibung'])) {
      $tmp = explode(',',$this->JTLPluginConf['cAttributeNichtAnBeschreibung']);
      foreach ($tmp as $value) {
        $value = trim($value);
        if (!empty($value)) {
          $this->aRemoveDescAttribute[] = strtolower($value);
        }
      }
    }
    
    if (array_key_exists('cWrapAttributeAnBeschreibung', $this->JTLPluginConf) && !empty($this->JTLPluginConf['cWrapAttributeAnBeschreibung'])) {
      $tmp = str_replace(array('[wert]', '[inhalt]'), array('%s', '%s'), $this->JTLPluginConf['cWrapAttributeAnBeschreibung']);
      $tcount = array(
        'wert'  => substr_count($this->JTLPluginConf['cWrapAttributeAnBeschreibung'], '[wert]'),
        'inhalt'=> substr_count($this->JTLPluginConf['cWrapAttributeAnBeschreibung'], '[inhalt]')
      );
      if ($tcount['wert'] === 1 || $tcount['inhalt'] === 1) {
        $this->cWrapDescriptionAttribute = $tmp;
      }
    }
    
    // lade KundengruppenIds
    $aRes = $this->db->query(
      'SELECT kKundengruppe FROM tkundengruppe',
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $row) {
      $this->aKundengruppen[] = $row->kKundengruppe;
    }
    
    // lade teinstellungen
    $aTeinstellungenFields = array("artikeldetails_attribute_anhaengen","artikel_lagerbestandsanzeige","artikeldetails_lieferantenbestand_anzeigen","artikeldetails_lieferstatus_anzeigen","artikeldetails_xselling_standard_anzeigen","artikeldetails_xselling_kauf_anzeigen","artikeldetails_xselling_kauf_anzahl","artikel_artikelanzeigefilter");
    $aRes = $this->db->queryPrepared(
      "SELECT cName,cWert FROM teinstellungen WHERE cName IN ('".implode("','", $aTeinstellungenFields)."')",
      [],
      ReturnType::ARRAY_OF_ASSOC_ARRAYS
    );
    foreach ($aRes as $row) {
      $this->aTeinstellungenFields[$row['cName']] = $row['cWert'];
    }
    
    $this->max_date_new_article = strtotime('- '.$this->JTLPluginConf['cUseNewArticleCategoryAge'].' DAYS');
    
    if (method_exists($this, 'agi_shopgate_connect__AGICreateItemsConfig')) {
      $this->agi_shopgate_connect__AGICreateItemsConfig();
    }
  }
  
  /**
    * Loads the product categories of the shop system's database and passes them to the buffer.
    *
    * @param int $limit pagination limit; if not null, the number of exported categories must be <= $limit
    * @param int $offset pagination; if not null, start the export with the categories at position $offset
    * @param string[] $uids a list of categories UIDs that should be exported
    *
    * @see http://developer.shopgate.com/plugin_api/export/get_categories
    *
    * @throws ShopgateLibraryException
    */
  protected function createCategories($limit = null, $offset = null, array $uids = array())
  {
    $this->agi_categories = array();
    
    if (!empty($uids)) {
      $oUids = $uids;
      $uids = array();
      foreach ($oUids as $value) {
        $uids[] = (int)$value;
      }
      $uids = array_unique($uids);
      $query =  "SELECT 
          tkategorie.kKategorie
        FROM tkategorie
        LEFT JOIN tkategoriesichtbarkeit ON 
          tkategoriesichtbarkeit.kKategorie = tkategorie.kKategorie AND tkategoriesichtbarkeit.kKundengruppe=:kKundengruppe
        WHERE tkategorie.kKategorie IN (".implode(',', $uids).") AND tkategoriesichtbarkeit.kKategorie IS NULL
        ORDER BY tkategorie.kOberKategorie
      ";
      $loaded_categories = $this->db->queryPrepared(
        $query,
        ['kKundengruppe' => $this->kKundengruppe],
        ReturnType::ARRAY_OF_OBJECTS
      );
    } 
    else {
      
      $sLimit = '';
      if (!empty($limit)) {
        $sLimit = (int) $limit;
      }
      if (!empty($limit) && !empty($offset)) {
        $sLimit = ((int) $offset).','.$sLimit;
      }
      if (!empty($sLimit)) {
        $sLimit = ' LIMIT '.$sLimit;
      }
      
      $query =  "SELECT 
          tkategorie.kKategorie
        FROM tkategorie
        LEFT JOIN tkategoriesichtbarkeit ON 
          tkategoriesichtbarkeit.kKategorie = tkategorie.kKategorie AND tkategoriesichtbarkeit.kKundengruppe=:kKundengruppe
        WHERE tkategoriesichtbarkeit.kKategorie IS NULL
        ORDER BY tkategorie.kOberKategorie".$sLimit;
      $loaded_categories = $this->db->queryPrepared(
        $query,
        ['kKundengruppe' => $this->kKundengruppe],
        ReturnType::ARRAY_OF_OBJECTS
      );
      if (empty($limit) || count($loaded_categories) < $limit) {
        // Kategorie fr neue Artikel, wenn so konfiguriert
        if (array_key_exists('cUseNewArticleCategory', $this->JTLPluginConf) && $this->JTLPluginConf['cUseNewArticleCategory'] == 'Y') {
          $tmp_name = $this->JTLPluginConf['cUseNewArticleCategoryName'];
          if (empty($tmp_name)) {
            $tmp = $this->db->queryPrepared(
              "SELECT cSeo FROM tseo WHERE kKey= :kKey AND kSprache= :kSprache",
              ['kKey' => SEARCHSPECIALS_NEWPRODUCTS, 'kSprache' => $this->kSprache],
              ReturnType::SINGLE_OBJECT
            );
            $tmp_name = $tmp->cSeo;
          }
          $loaded_categories[] = (object) [
            'nID'       => '1neu',
            'name'      => $tmp_name,
            'active'    => 1,
            'parentID'  => 0,
            'cDeeplink' => $this->URL_SHOP.$tmp->cSeo,
            'nSort'     => 100000,
            'hasImage'  => 0,
            'cBild'     => ''
          ];
        }
      }
    }
    foreach ($loaded_categories as $value) {
      if (isset($value->kKategorie)) {
        $cKey = $value->kKategorie;
        $oKategorie = new Kategorie($value->kKategorie, $this->kSprache, $this->kKundengruppe);
        $item = array(
          'nID'       => $oKategorie->kKategorie,
          'name'      => $oKategorie->cName,
          'active'    => 1,
          'parentID'  => $oKategorie->kOberKategorie,
          'cDeeplink' => $this->URL_SHOP.$oKategorie->cURL,
          'nSort'     => 100000-$oKategorie->nSort,
          'hasImage'  => $oKategorie->nBildVorhanden,
          'cBild'     => $oKategorie->cBild
        );
      } elseif(isset($value->nID)) {
        $cKey = $value->nID;
        $item = array(
          'nID'       => '1neu',
          'name'      => $value->name,
          'active'    => $value->active,
          'parentID'  => $value->parentID,
          'cDeeplink' => $value->cDeeplink,
          'nSort'     => $value->nSort,
          'hasImage'  => $value->hasImage,
          'cBild'     => $value->cBild
        );
      }
      
      if(property_exists($oKategorie,'categoryAttributes')) {
        if (array_key_exists('shopgate_active',$oKategorie->categoryAttributes) && $oKategorie->categoryAttributes['shopgate_active']->cWert == 'no') {
          $item['active'] = 0;
        }
      }
      if(property_exists($oKategorie,'categoryFunctionAttributes')) {
        if (array_key_exists('shopgate_active',$oKategorie->categoryFunctionAttributes) && $oKategorie->categoryFunctionAttributes['shopgate_active'] == 'no') {
          $item['active'] = 0;
        }
      }
      $this->agi_categories[$cKey] = $item;
    }
    
    if (method_exists($this, 'agi_shopgate_connect__createCategories')) {
      // to manipulate $this->agi_categories
      $this->agi_shopgate_connect__createCategories($limit, $offset, $uids);
    }
    
    foreach ($this->agi_categories as $key => $value) {
      $categoryObj = new Shopgate_Model_Catalog_Category();
      $utf8_name = $this->_prepare_utf8($value['name']);
      // Fill in basic data:
   		$categoryObj->setUid($value['nID']);
   		$categoryObj->setIsActive($value['active']);
   		$categoryObj->setSortOrder($value['nSort']);

   		$categoryObj->setName($utf8_name);
      // Set up the link to the category on the desktop website
   		$categoryObj->setDeeplink($this->_prepare_utf8($value['cDeeplink']));
   		
   		// Set a parent category
   		if(!empty($value['parentID'])) {
   			$categoryObj->setParentUid($value['parentID']);
   		}
      
      if ($value['hasImage']) {
        $categoryImage = new Shopgate_Model_Media_Image();
   			$categoryImage->setUid('c'.$value['nID']);
   			$categoryImage->setSortOrder(1);
   			$categoryImage->setUrl($this->_prepare_utf8($value['cBild']));
        $categoryImage->setTitle($utf8_name);
        $categoryImage->setAlt($utf8_name);
   			
   			// Insert the image into the category [optional]
   			$categoryObj->setImage($categoryImage);
      }
      
      $this->addCategoryModel($categoryObj);

      if (time()-$this->startExecutionTime > $this->maxExecutionTime) {
        break;
      }
    }
  }
  
  /**
    * Loads the product reviews of the shop system's database and passes them to the buffer.
    *
    * @param int      $limit  pagination limit; if not null, the number of exported reviews must be <= $limit
    * @param int      $offset pagination; if not null, start the export with the reviews at position $offset
    * @param string[] $uids   A list of products that should be fetched for the reviews.
    *
    * @see http://developer.shopgate.com/plugin_api/export/get_reviews
    *
    * @throws ShopgateLibraryException
    */
  protected function createReviews($limit = null, $offset = null, array $uids = array())
  {
    if (!empty($uids)) {
      $oUids = $uids;
      $uids = array();
      foreach ($oUids as $value) {
        $uids[] = (int)$value;
      }
      $uids = array_unique($uids);
      $query =  "SELECT * FROM tbewertung WHERE kArtikel IN (".implode(',', $uids).")";
      if ($this->aTeinstellungenFields['bewertung_freischalten'] == 'Y') {
        $query .= " AND nAktiv=1";
      }
      $query .= " AND kBewertung";
      $loaded_reviews = $this->db->queryPrepared(
        $query,
        [],
        ReturnType::ARRAY_OF_OBJECTS
      );
    } 
    else {
      
      $sLimit = '';
      if (!empty($limit)) {
        $sLimit = (int) $limit;
      }
      if (!empty($limit) && !empty($offset)) {
        $sLimit = ((int) $offset).','.$sLimit;
      }
      if (!empty($sLimit)) {
        $sLimit = ' LIMIT '.$sLimit;
      }
      
      $query = "SELECT * FROM `tbewertung`";
      if ($this->aTeinstellungenFields['bewertung_freischalten'] == 'Y') {
        $query .= " WHERE nAktiv=1";
      }
      $query .= " ORDER BY kBewertung".$sLimit;
      
      $loaded_reviews = $this->db->queryPrepared(
        $query,
        [],
        ReturnType::ARRAY_OF_OBJECTS
      );
    }
    if (!empty($loaded_reviews)) {
      foreach ($loaded_reviews as $review) {
        $oReview = new Shopgate_Model_Review();
        $oReview->setUid($review->kBewertung);
        $oReview->setItemUid($review->kArtikel);
        $oReview->setTitle($this->_prepare_utf8($review->cTitel));
        $oReview->setText($this->_prepare_utf8($this->_remove_html($review->cText)));
        $oReview->setScore($review->nSterne);
        $oReview->setReviewerName($this->_prepare_utf8($review->cName));
        $oReview->setDate($review->dDatum.' 00:00:00');
        $this->addReviewModel($oReview);
      }
    }
  }
  
  /**
    * Returns an array of certain settings of the shop. (Currently mainly tax settings.)
    *
    * @see http://developer.shopgate.com/plugin_api/system_information/get_settings
    *
    * @return array(
    *		'tax' => array(
    *			'tax_classes_products' => A list of product tax class identifiers.
    *			'tax_classes_customers' => A list of customer tax classes.
    *			'tax_rates' => A list of tax rates.
    *			'tax_rules' => A list of tax rule containers.
    *		)
    * )
    * @throws ShopgateLibraryException on invalid log in data or hard errors like database failure.
    */
  public function getSettings()
  {
    $settings = array(
      'payment_methods'           => array(),
      'allowed_address_countries' => array(),
      'allowed_shipping_countries'=> array(),
			'customer_groups'       => array(),
      'tax'                   => array(
        'product_tax_classes'   => array(),
        'customer_tax_classes'  => array(array('id' => 1, 'key' => "default")),
        'tax_rates'             => array(),
        'tax_rules'             => array()
      )
		);
    
    $checkPossible = array();
    $aRes = $this->db->query(
      'SELECT DISTINCT kZahlungsart FROM tversandartzahlungsart',
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $value) {
      if (!in_array($value->kZahlungsart, $checkPossible)) {
        $checkPossible[] = $value->kZahlungsart;
      }
    }
    $aRes = $this->db->query(
      'SELECT kZahlungsart, cName FROM tzahlungsart WHERE nActive=1 AND nNutzbar=1',
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $row) {
      if (in_array($row->kZahlungsart, $checkPossible)) {
        $pay = array(
          'id' => $row->kZahlungsart.'_'.$this->_prepare_utf8($row->cName)
        );
        $settings['payment_methods'][] = $pay;
      }
    }

    if ($this->aTeinstellungenFields['s360_amazonpay_shop5'] && $this->aTeinstellungenFields['s360_amazonpay_shop5_active']) {
      $pay = array(
        'id' => 's360_mws'
      );
      $settings['payment_methods'][] = $pay;
    }
    
    // erlaubte Lnder
    $aLaender = ShippingMethod::getPossibleShippingCountries($this->kKundengruppe, $this->aTeinstellungenFields['kundenregistrierung_nur_lieferlaender'] == 'Y');
    
    if (empty($aLaender)) {
      $aLaender = ShippingMethod::getPossibleShippingCountries($this->kKundengruppe);
    }
    foreach ($aLaender as $oLand) {
      $settings['allowed_address_countries'][] = array(
        'country' => $oLand->cISO,
        'state'   => array('all')
      );
      $settings['allowed_shipping_countries'][] = array(
        'country' => $oLand->cISO,
        'state'   => array('all')
      );
    }

    // steuerstze
    // https://developer.shopgate.com/references/cart-integration/plugin-api/system-info/get-settings
    foreach ($this->aSteuerklassen as $key => $row) {
      $settings['tax']['product_tax_classes'][] = array(
        'id'  => $row['kSteuerklasse'],
        'key' => $this->_prepare_utf8($row['cName'])
      );
    }
    $rules_set = array();
    $aRes = $this->db->query(
      'SELECT 
        ts.kSteuersatz, ts.kSteuerzone, ts.kSteuerklasse, ts.fSteuersatz, ts.nPrio,
        tz.cName AS cNameSteuerzone,
        tk.cName AS cNameSteuerklasse, tk.cStandard,
        tl.cISO 
      FROM tsteuersatz AS ts
      LEFT JOIN tsteuerzone AS tz ON ts.kSteuerzone=tz.kSteuerzone
      LEFT JOIN tsteuerklasse AS tk ON ts.kSteuerklasse=tk.kSteuerklasse
      LEFT JOIN tsteuerzoneland AS tl ON ts.kSteuerzone=tl.kSteuerzone
      WHERE ts.fSteuersatz > 0',
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $tax) {
      $settings['tax']['tax_rates'][] = array(
        'id'            => $tax->kSteuersatz.'_'.$tax->kSteuerzone.'_'.$tax->cISO,
        'key'           => $this->_prepare_utf8($tax->fSteuersatz.' '.$tax->cNameSteuerklasse.' '.$tax->cNameSteuerzone.' '.$tax->cISO),
        'display_name'  => $this->_prepare_utf8($this->tSprache['vat']),
        'tax_percent'   => $tax->fSteuersatz,
        'country'       => $this->_prepare_utf8($tax->cISO),
        'state'         => '',
        'zipcode_type'  => 'all'
      );
      $cKeyTmp = $tax->kSteuerzone.'_'.$tax->kSteuerklasse;
      if (!in_array($cKeyTmp, $rules_set)) {
        $settings['tax']['tax_rules'][] = array(
          'id'        => $cKeyTmp,
          'name'      => $this->_prepare_utf8($tax->cNameSteuerzone.' '.$tax->cNameSteuerklasse),
          'priority'  => 100-$tax->nPrio,
          'product_tax_classes'   => array(array('id'=> $tax->kSteuerklasse, 'key' => $this->_prepare_utf8($tax->cNameSteuerklasse))),
          'customer_tax_classes'  => array(array('id'=> 1, 'key' => 'default')),
          'tax_rates' => array()
        );
        $rules_key = count($rules_set);
        $rules_set[] = $cKeyTmp;
      }
      else {
        $rules_key = array_search($cKeyTmp, $rules_set);
      }
      
      $settings['tax']['tax_rules'][$rules_key]['tax_rates'][] = array(
        'id'  => $tax->kSteuersatz.'_'.$tax->kSteuerzone.'_'.$tax->cISO,
        'key' => $this->_prepare_utf8($tax->fSteuersatz.' '.$tax->cNameSteuerklasse.' '.$tax->cNameSteuerzone.' '.$tax->cISO)
      );
    }
    
    $aRes = $this->db->query(
      'SELECT kKundengruppe, cName, cStandard, nNettoPreise FROM tkundengruppe',
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($aRes as $row) {
      $settings['customer_groups'][] = array(
        'id'  => $row->kKundengruppe,
        'name'=> $this->_prepare_utf8($row->cName),
        'is_default'  => $row->cStandard == 'Y' ? 1 : 0
      );
    }
    
    return $settings;
  }

  /**
    * Executes a cron job with parameters.
    *
    * @param string $jobname The name of the job to execute.
    * @param <string => mixed> $params Associative list of parameter names and values.
    * @param string $message A reference to the variable the message is appended to.
    * @param int $errorcount A reference to the error counter variable.
    * @post $message contains a message of success or failure for the job.
    * @post $errorcount contains the number of errors that occurred during execution.
    */
  public function cron($jobname, $params, &$message, &$errorcount)
  {
   	switch($jobname) {
			case "set_shipping_completed":
				$aProcessLog = $this->cronSetShippingCompleted($message, $errorcount);
				break;
      case "cancel_orders":
				$aProcessLog = $this->cronGetCancelOrder($message, $errorcount);
				break;
			default:
				throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_CRON_UNSUPPORTED_JOB, "{$jobname}", true);
		}
  }
  
  protected function cronSetShippingCompleted(&$message, &$errorcount)
  {
    $cQuery = "SELECT 
        xp.kBestellung, xp.kBestellungShopgate, tb.dVersandDatum
      FROM
        `xplugin_agi_shopgate_connect_orders` AS xp, `tbestellung` AS tb
      WHERE
        xp.kBestellung=tb.kBestellung AND xp.sgVersandstatusGesendet=0 AND tb.dVersandDatum>'0000-00-00'
    ";
    if (!empty($this->request_shop_number)) {
      $cQuery .= " AND `sgShopNumber`=".$this->request_shop_number;
    }
    
    $builder = new ShopgateBuilder($this->config);
    $api = $builder->buildMerchantApi();
    
    $aRes = $this->db->queryPrepared($cQuery, [], ReturnType::ARRAY_OF_ASSOC_ARRAYS);
    foreach ($aRes as $row) {
      try {
        $resp = $api->setOrderShippingCompleted($row['kBestellungShopgate']);
        if ($resp->error == 0) {
          $aUpdate = (object) ['sgVersandstatusGesendet' => 1];
          if (!empty($aUpdate)) {
            $keyName = array('kBestellung');
            $keyValue = array($row['kBestellung']);
            if (!empty($this->request_shop_number)) {
              $keyName[] = 'sgShopNumber';
              $keyValue[] = $this->request_shop_number;
            }
            $this->db->updateRow('xplugin_agi_shopgate_connect_orders', $keyName, $keyValue, $aUpdate);
          }
        }
        else {
          $this->log('cronSetShippingCompleted(): Code '.$resp->error.': '.$resp->error_text, ShopgateLogger::LOGTYPE_DEBUG);
        }
      }
      catch (ShopgateMerchantApiException $e) {
        $eCode = $e->getCode();
        if ($eCode == 204 || $eCode == 201)
        {
          $aUpdate = (object) ['sgVersandstatusGesendet' => 1];
          if (!empty($aUpdate)) {
            $keyName = array('kBestellung');
            $keyValue = array($row['kBestellung']);
            if (!empty($this->request_shop_number)) {
              $keyName[] = 'sgShopNumber';
              $keyValue[] = $this->request_shop_number;
            }
            $this->db->updateRow('xplugin_agi_shopgate_connect_orders', $keyName, $keyValue, $aUpdate);
            $message .= 'Bestellung '.$row['kBestellungShopgate'].' war bereits manuell auf versendet gesetzt. JTL-Datenbank wurde aktualisiert.';
            $errorcount++;
          }
        }
        else {
          $message .= $e->getMessage();
          $errorcount++;
        }
      }
    }
  }

  protected function cronGetCancelOrder(&$message, &$errorcount)
  {
    $cQuery = "SELECT 
        xp.kBestellung, xp.kBestellungShopgate, tb.dVersandDatum
      FROM
        `xplugin_agi_shopgate_connect_orders` AS xp, `tbestellung` AS tb
      WHERE
        xp.kBestellung=tb.kBestellung AND xp.sgBestellungStorniert=0 AND tb.cStatus='".BESTELLUNG_STATUS_STORNO."'
    ";
    if (!empty($this->request_shop_number)) {
      $cQuery .= " AND `sgShopNumber`=".$this->request_shop_number;
    }
    
    $builder = new ShopgateBuilder($this->config);
    $api = $builder->buildMerchantApi();
    
    $aRes = $this->db->queryPrepared($cQuery, [], ReturnType::ARRAY_OF_ASSOC_ARRAYS);
    foreach ($aRes as $row) {
      try {
        $resp = $api->cancelOrder($row['kBestellungShopgate']);
        if ($resp->error == 0) {
          $aUpdate = (object) ['sgBestellungStorniert' => 1];
          if (!empty($aUpdate)) {
            $keyName = array('kBestellung');
            $keyValue = array($row['kBestellung']);
            if (!empty($this->request_shop_number)) {
              $keyName[] = 'sgShopNumber';
              $keyValue[] = $this->request_shop_number;
            }
            $this->db->updateRow('xplugin_agi_shopgate_connect_orders', $keyName, $keyValue, $aUpdate);
          }
        }
        else {
          $this->log('cronSetShippingCompleted(): Code '.$resp->error.': '.$resp->error_text, ShopgateLogger::LOGTYPE_DEBUG);
        }
      }
      catch (Exception $e) {
        $eCode = $e->getCode();
        if ($eCode == 222 || $eCode == 201)
        {
          $aUpdate = (object) ['sgBestellungStorniert' => 1];
          if (!empty($aUpdate)) {
            $keyName = array('kBestellung');
            $keyValue = array($row['kBestellung']);
            if (!empty($this->request_shop_number)) {
              $keyName[] = 'sgShopNumber';
              $keyValue[] = $this->request_shop_number;
            }
            $this->db->updateRow('xplugin_agi_shopgate_connect_orders', $keyName, $keyValue, $aUpdate);
            $message = 'Bestellung '.$row['kBestellungShopgate'].' war bereits manuell storniert gesetzt. JTL-Datenbank wurde aktualisiert.';
            $errorcount++;
          }
        }
        else {
          echo 'Exception abgefangen: ',  $e->getMessage(), "\n";
        }
      }
    }
  }
  

  /**
    * This method creates a new user account / user addresses for a customer in the shop system's database
    *
    * The method should not abort on soft errors like when the street or phone number of a customer is not set.
    *
    * @see http://developer.shopgate.com/plugin_api/customers/register_customer
    *
    * @param string $user The user name the customer entered at Shopgate.
    * @param string $pass The password the customer entered at Shopgate.
    * @param ShopgateCustomer $shopgateCustomer A ShopgateCustomer object to be added to the shop system's database.
    * @throws ShopgateLibraryException if an error occurred
    */
  public function registerCustomer($user, $pass, ShopgateCustomer $shopgateCustomer)
  {
    if (method_exists($this, 'agi_shopgate_connect__registerCustomer')) {
      $this->agi_shopgate_connect__registerCustomer($user, $pass, $shopgateCustomer);
    }
    
    $user = Text::filterXSS($this->_prepare_utf8($user, false));
    $aRes = $this->db->queryPrepared(
      "SELECT COUNT(*) AS cnt FROM tkunde WHERE cMail= :cMail AND nRegistriert=1",
      ['cMail' => $user],
      ReturnType::SINGLE_OBJECT
    );
    if ($aRes->cnt > 0) {
      throw new ShopgateLibraryException(
   			ShopgateLibraryException::REGISTER_USER_ALREADY_EXISTS,
   			'Username: '.$user
   		);
    }
    
    $kKundengruppe = $this->kKundengruppe;
    $kSprache = $this->kSprache;
    
    $JTL_Client = new Customer();
    switch($shopgateCustomer->getGender()) {
      case 'f': $JTL_Client -> cAnrede  = 'w'; break;
      case 'm': $JTL_Client -> cAnrede  = 'm'; break;
      default: $JTL_Client -> cAnrede  = '';
    }
    $passwordService = Shop::Container()->getPasswordService();
    
    $JTL_Client -> cVorname = $this->_prepare_utf8($shopgateCustomer->getFirstName(), false);
    $JTL_Client -> cNachname= $this->_prepare_utf8($shopgateCustomer->getLastName(), false);
    $JTL_Client -> dGeburtstag = $this->_prepare_utf8($shopgateCustomer->getBirthday(), false);
    $JTL_Client -> cTel     = $this->_prepare_utf8($shopgateCustomer->getPhone(), false);
    $JTL_Client -> cMobil   = $this->_prepare_utf8($shopgateCustomer->getMobile(), false);
    $JTL_Client -> cMail    = $this->_prepare_utf8($shopgateCustomer->getMail(), false);
    $JTL_Client -> cPasswort = $passwordService->hash($this->_prepare_utf8($pass, false));
    $JTL_Client -> nRegistriert = 1;
    $JTL_Client -> kKundengruppe= $kKundengruppe;
    $JTL_Client -> kSprache   = $kSprache;
    $JTL_Client -> dErstellt = date('Y-m-d');
    $JTL_Client -> cAktiv    = 'Y';
    $JTL_Client -> cAbgeholt = 'N';
    $JTL_Client -> cHerkunft = 'Shopgate';
    
    $adressen = array();
    $adressList = $shopgateCustomer->getAddresses();
    foreach ($adressList as $adress) {
      if ($adress->getIsInvoiceAddress()) {
        $adressen[] = $adress;
      }
    }
    if (empty($adressen)) {
      foreach ($adressList as $adress) {
        $adressen[] = $adress;
      }
    }
    $first = reset($adressen);
    if ($first) {
      $JTL_Client -> cFirma   = $this->_prepare_utf8($first->getCompany(), false);	
      $JTL_Client -> cStrasse = $this->_prepare_utf8($first->getStreetName1(), false);
      $JTL_Client -> cHausnummer  = $this->_prepare_utf8($first->getStreetNumber1(), false);
      $JTL_Client -> cAdressZusatz= $this->_prepare_utf8($first->getStreet2(), false);
      $JTL_Client -> cPLZ     = $this->_prepare_utf8($first->getZipcode(), false);
      $JTL_Client -> cOrt     = $this->_prepare_utf8($first->getCity(), false);
      $JTL_Client -> cLand    = $this->_prepare_utf8($first->getCountry(), false);
    }
    $JTL_Client -> insertInDB();
    
    $customer = new ShopgateCustomer();
    $customer->setCustomerId($JTL_Client->kKunde);
    $customer->setCustomerToken($JTL_Client->kKunde);
    if (!empty($JTL_Client->cKundenNr)) {
      $customer->setCustomerNumber($JTL_Client->cKundenNr);
    }
    if (!empty($JTL_Client->kKundengruppe)) {
      $customer->setCustomerGroupId($JTL_Client->kKundengruppe);
    }
    $customer->setGender($gender);
    $customer->setFirstName($this->_prepare_utf8($JTL_Client->cVorname));
    $customer->setLastName($this->_prepare_utf8($JTL_Client->cNachname));
    $customer->setBirthday($this->_prepare_utf8($JTL_Client->dGeburtstag));
    $customer->setPhone($this->_prepare_utf8($JTL_Client->cTel));
    $customer->setMobile($this->_prepare_utf8($JTL_Client->cMobil));
    $customer->setMail($this->_prepare_utf8($JTL_Client->cMail));
    
    if (!empty($JTL_Client->kKundengruppe)) {
      $aRes = $this->db->queryPrepared(
        "SELECT cName FROM tkundengruppe WHERE kKundengruppe= :kKundengruppe",
        ['kKundengruppe' => $JTL_Client->kKundengruppe],
        ReturnType::SINGLE_OBJECT
      );
      if ($aRes) {
        $customerGroups = array();
          $group = new ShopgateCustomerGroup();
          $group->setID($JTL_Client->kKundengruppe);
          $group->setName($this->_prepare_utf8($aRes->cName));
        $customerGroups[] = $group;
        $customer->setCustomerGroups($customerGroups);
        $customer->setCustomerGroup($this->_prepare_utf8($aRes->cName));
      }
    }
    
    $customer->setAddresses($adressen);
    
    return $customer;
  }

  /**
    * Exports orders from the shop system's database to Shopgate.
    *
    * @see http://developer.shopgate.com/plugin_api/orders/get_orders
    *
    * @param string $customerToken
    * @param string $customerLanguage
    * @param int $limit
    * @param int $offset
    * @param string $orderDateFrom
    * @param string $sortOrder
    *
    * @return ShopgateExternalOrder[] A list of ShopgateExternalOrder objects
    *
    * @throws ShopgateLibraryException
    */
  public function getOrders(
   			$customerToken,
   			$customerLanguage,
   			$limit = 10,
   			$offset = 0,
   			$orderDateFrom = '',
   			$sortOrder = 'created_desc'
   ) 
  {
    
    $JTL_Client = new Customer($customerToken);
   	// validate customer token, e.g.:
   	if (empty($JTL_Client->kKunde)) {
   		throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_CUSTOMER_TOKEN_INVALID, 'token expired or invalid');
   	}
    
    $str_where = 'kKunde= :kKunde';
    if (!empty($orderDateFrom)) {
      $str_where = 'AND dErstellt>= :dErstellt';
    }
    $orders = $this->db->queryPrepared(
      "SELECT kBestellung FROM tbestellung WHERE $str_where ORDER BY kBestellung DESC LIMIT ".(empty($offset) ? $limit : $offset.','.$limit),
      !empty($orderDateFrom) ? ['kKunde' => $JTL_Client->kKunde, 'dErstellt' => $orderDateFrom] : ['kKunde' => $JTL_Client->kKunde],
      ReturnType::ARRAY_OF_OBJECTS
    );
    $aWaehrung = array();

    $shopgateOrders = array();
    foreach ($orders as $dbOrder) {
      $JTL_Order = new Bestellung($dbOrder->kBestellung, true);
      
      $oSGOrderRes = $this->db->queryPrepared(
        "SELECT kBestellungShopgate FROM xplugin_agi_shopgate_connect_orders WHERE kBestellung= :kBestellung",
        ['kBestellung' => $dbOrder->kBestellung],
        ReturnType::SINGLE_OBJECT
      );
      $sg_id = empty($oSGOrderRes) ? 0 : $oSGOrderRes->kBestellungShopgate;
      
      if (!array_key_exists($JTL_Order->kWaehrung, $aWaehrung)) {
        $oWaehrung = $this->db->queryPrepared(
          "SELECT cISO FROM twaehrung WHERE kWaehrung= :kWaehrung",
          ['kWaehrung' => $JTL_Order->kWaehrung],
          ReturnType::SINGLE_OBJECT
        );
        $aWaehrung[$JTL_Order->kWaehrung] = empty($oWaehrung) ? $this->oWaehrung->cISO : $oWaehrung->cISO;
      }

      Tax::setTaxRates(null);
      
   		$_tmpOrder = new ShopgateExternalOrder();

      if (!empty($sg_id)) {
        $_tmpOrder->setOrderNumber($sg_id);
      }
   		$_tmpOrder->setExternalOrderId($JTL_Order->kBestellung);
   		$_tmpOrder->setExternalOrderNumber($this->_prepare_utf8($JTL_Order->cBestellNr));
      $_tmpOrder->setCurrency($aWaehrung[$JTL_Order->kWaehrung]);
   		$_tmpOrder->setIsPaid(!empty($JTL_Order->dBezahltDatum) ? 1 : 0);
   		$_tmpOrder->setIsShippingCompleted(!empty($JTL_Order->dVersandDatum) ? 1 : 0);
      if (!empty($JTL_Order->dVersandDatum)) {
        $_tmpOrder->setShippingCompletedTime($JTL_Order->dVersandDatum);
      }
   		$_tmpOrder->setCreatedTime($JTL_Order->dErstellt);
      $_tmpOrder->setStatusName($this->_prepare_utf8($JTL_Order->Status));
      $_tmpOrder->setPaymentMethod($this->_prepare_utf8($JTL_Order->cZahlungsartName));
      $_tmpOrder->setAmountCompleteGross($JTL_Order->fGesamtsumme);
      
      // Rechnungsanschrift
      $address = new ShopgateAddress();
      $address->setId('R'.$JTL_Order->oRechnungsadresse->kRechnungsadresse);
      $address->setAddressType(ShopgateAddress::INVOICE);
      switch ($JTL_Order->oRechnungsadresse->cAnrede) {
        case 'w': $gender = 'f'; break;
        case 'm': $gender = 'm'; break;
        default: $gender = '';
      }
      if (!empty($gender)) {
        $address->setGender($gender);
      }
      $address->setFirstName($this->_prepare_utf8($JTL_Order->oRechnungsadresse->cVorname));
      $address->setLastName($this->_prepare_utf8($JTL_Order->oRechnungsadresse->cNachname));
      $address->setCompany($this->_prepare_utf8($JTL_Order->oRechnungsadresse->Firma));
      $address->setStreet1($this->_prepare_utf8($JTL_Order->oRechnungsadresse->cStrasse).(empty($JTL_Order->oRechnungsadresse->cHausnummer) ? '' : ' '.$JTL_Order->oRechnungsadresse->cHausnummer));
      $address->setStreet2($this->_prepare_utf8($JTL_Order->oRechnungsadresse->cAdressZusatz));
      $address->setCity($this->_prepare_utf8($JTL_Order->oRechnungsadresse->cOrt));
      $address->setZipcode($this->_prepare_utf8($JTL_Order->oRechnungsadresse->cPLZ));
      $address->setCountry($this->_prepare_utf8($JTL_Order->oRechnungsadresse->cLand));
      $address->setPhone($this->_prepare_utf8($JTL_Order->oRechnungsadresse->cTel));
      $address->setMobile($this->_prepare_utf8($JTL_Order->oRechnungsadresse->cMobil));
      $address->setMail($this->_prepare_utf8($JTL_Order->oRechnungsadresse->cMail));
      $_tmpOrder->setInvoiceAddress($address);
      
      // Lieferanschrift
      if (empty($JTL_Order->kLieferadresse)) {
        $address->setAddressType(ShopgateAddress::DELIVERY);
        $_tmpOrder->setDeliveryAddress($address);
      } else {
        $address = new ShopgateAddress();
        $address->setId('R'.$JTL_Order->Lieferadresse->kRechnungsadresse);
        $address->setAddressType(ShopgateAddress::DELIVERY);
        switch ($JTL_Order->Lieferadresse->cAnrede) {
          case 'w': $gender = 'f'; break;
          case 'm': $gender = 'm'; break;
          default: $gender = '';
        }
        if (!empty($gender)) {
          $address->setGender($gender);
        }
        $address->setFirstName($this->_prepare_utf8($JTL_Order->Lieferadresse->cVorname));
        $address->setLastName($this->_prepare_utf8($JTL_Order->Lieferadresse->cNachname));
        $address->setCompany($this->_prepare_utf8($JTL_Order->Lieferadresse->Firma));
        $address->setStreet1($this->_prepare_utf8($JTL_Order->Lieferadresse->cStrasse).(empty($JTL_Order->Lieferadresse->cHausnummer) ? '' : ' '.$JTL_Order->Lieferadresse->cHausnummer));
        $address->setStreet2($this->_prepare_utf8($JTL_Order->Lieferadresse->cAdressZusatz));
        $address->setCity($this->_prepare_utf8($JTL_Order->Lieferadresse->cOrt));
        $address->setZipcode($this->_prepare_utf8($JTL_Order->Lieferadresse->cPLZ));
        $address->setCountry($this->_prepare_utf8($JTL_Order->Lieferadresse->cLand));
        $address->setPhone($this->_prepare_utf8($JTL_Order->Lieferadresse->cTel));
        $address->setMobile($this->_prepare_utf8($JTL_Order->Lieferadresse->cMobil));
        $address->setMail($this->_prepare_utf8($JTL_Order->Lieferadresse->cMail));
        $_tmpOrder->setDeliveryAddress($address);
      }
      
   		//$_tmpOrder->setExternalCoupons($order['external_coupons']);
      $extraCosts = array();
      $externalCoupons = array();
      $shopgateItems = array();
      foreach ($JTL_Order->Positionen as $item) {
        if ($item->nPosTyp == C_WARENKORBPOS_TYP_ARTIKEL) {
          $eigenschaften = array();
          if (!empty($item->WarenkorbPosEigenschaftArr)) {
            foreach ($item->WarenkorbPosEigenschaftArr as $row) {
              $eigenschaften[] = $row->cEigenschaftName.': '.$row->cFreifeldWert;
            }
          }
          $shopgateExternalOrderItem = new ShopgateExternalOrderItem();
          $shopgateExternalOrderItem->setItemNumber($item->kArtikel);
          $shopgateExternalOrderItem->setItemNumberPublic($this->_prepare_utf8($item->cArtNr));
          $shopgateExternalOrderItem->setQuantity($item->nAnzahl);
          $shopgateExternalOrderItem->setUnitAmount($item->fPreisEinzelNetto);
          $shopgateExternalOrderItem->setUnitAmountWithTax(round($item->fPreis*(100+$item->fMwSt)/100, 2));
          $shopgateExternalOrderItem->setTaxPercent($item->fMwSt);
          $shopgateExternalOrderItem->setCurrency($aWaehrung[$JTL_Order->kWaehrung]);
          $shopgateExternalOrderItem->setName($this->_prepare_utf8($item->cName));
          if (!empty($eigenschaften)) {
            $shopgateExternalOrderItem->setDescription($this->_prepare_utf8(implode(chr(10), $eigenschaften)));
          }
          $shopgateItems[] = $shopgateExternalOrderItem;
        } 
        elseif ($item->nPosTyp == C_WARENKORBPOS_TYP_KUPON) {
          $ec = new ShopgateExternalCoupon();
          $ec->setOrderIndex(count($externalCoupons)+1);
          $ec->setName($this->_prepare_utf8($item->cName));
          $ec->setAmount(round($item->fPreis*(100+$item->fMwSt)/100, 2));
          $ec->setCurrency($aWaehrung[$JTL_Order->kWaehrung]);
          $externalCoupons[] = $ec;
        } 
        else {
          $ec = new ShopgateExternalOrderExtraCost();
          
          if ($item->nPosTyp == C_WARENKORBPOS_TYP_VERSANDPOS) {
            $ec->setType(ShopgateExternalOrderExtraCost::TYPE_SHIPPING);
          } elseif ($item->nPosTyp == C_WARENKORBPOS_TYP_ZAHLUNGSART) {
            $ec->setType(ShopgateExternalOrderExtraCost::TYPE_PAYMENT);
          } else {
            $ec->setType(ShopgateExternalOrderExtraCost::TYPE_MISC);
          }
          $ec->setLabel($this->_prepare_utf8($item->cName));
          $ec->setTaxPercent($item->fMwSt);
          $ec->setAmount(round($item->fPreis*(100+$item->fMwSt)/100, 2));
          $extraCosts[] = $ec;
        }
        
      }
      $_tmpOrder->setItems($shopgateItems);
      if (!empty($extraCosts)) {
        $_tmpOrder->setExtraCosts($extraCosts);
      }
      if (!empty($externalCoupons)) {
        $_tmpOrder->setExternalCoupons($externalCoupons);
      }
      /*
      foreach ($JTL_Order->Positionen as $item) {
        if ($item->nPosTyp === 1) {
          continue;
        }
        $shopgateExternalOrderItem = new ShopgateExternalOrderItem();
   			$shopgateExternalOrderItem->setItemNumber($item->kArtikel);
   			$shopgateExternalOrderItem->setItemNumberPublic($this->_prepare_utf8($item->cArtNr));
   			$shopgateExternalOrderItem->setQuantity($item->nAnzahl);
   			$shopgateExternalOrderItem->setUnitAmount($item->fPreisEinzelNetto);
   			$shopgateExternalOrderItem->setUnitAmountWithTax(round($item->fPreis*(100+$item->fMwSt)/100, 2));
   			$shopgateExternalOrderItem->setTaxPercent($item->fMwSt);
        $shopgateExternalOrderItem->setCurrency($aWaehrung[$JTL_Order->kWaehrung]);
   			$shopgateExternalOrderItem->setName($this->_prepare_utf8($item->cName));
        $shopgateItems[] = $shopgateExternalOrderItem;
      }
      $_tmpOrder->setItems($shopgateItems);
      */
      $shopgateTaxItems = array();
      foreach ($JTL_Order->Steuerpositionen AS $item) {
   			$shopgateOrderTaxItem = new ShopgateExternalOrderTax();
   			$shopgateOrderTaxItem->setLabel($item->cName);
   			$shopgateOrderTaxItem->setTaxPercent($item->fUst);
   			$shopgateOrderTaxItem->setAmount($item->fBetrag);
   			$shopgateTaxItems[] = $shopgateOrderTaxItem;
   		}
   		$_tmpOrder->setOrderTaxes($shopgateTaxItems);

   		$shopgateOrders[] = $_tmpOrder;
   	}
   	return $shopgateOrders;
  }

  /**
    * Check items of a cart, validates stock quantity of the the products and returns available amount.
    *
    * @see http://developer.shopgate.com/plugin_api/stock
    *
    * @param ShopgateCart $cart The ShopgateCart object to be checked and validated.
    * @return ShopgateCartItem[], # list of cartItems with validated stock
    *
    * @throws ShopgateLibraryException if an error occurs.
    */
  public function checkStock(ShopgateCart $shopgateCart)
  {
    if (method_exists($this, 'agi_shopgate_connect__checkStock')) {
      $this->agi_shopgate_connect__checkStock($shopgateCart);
    }
    $items = $shopgateCart->getItems();
    
    $this->_AGICreateItemsConfig();
    
    $aItemResponse = array();
    foreach($items as $checkItem) {
      $itemId = $checkItem->getItemNumber();
      $itemQuantity = 100;
      $itemOptions = $checkItem->getOptions();
      $itemInputs = $checkItem->getInputs();
      
      // prepare response
      $oItemResponse = new ShopgateCartItem();
      $oItemResponse->setItemNumber($itemId);
      $oItemResponse->setOptions($itemOptions);
      $oItemResponse->setInputs($itemInputs);
      
      // load Article
      $this->_loadOExportArticle($itemId);
      if (!empty($this->oExportArticle->kArtikel)) {
        $this->_AGIoExportArticleCheckStatusAktiv();
        $this->_AGIoExportArticleAddAttributes();
        
        $oItemResponse = new ShopgateCartItem();
        $oItemResponse->setItemNumber($itemId);
        
        if ($this->oExportArticle->cStatusAktiv == 'inactive') {
          $oItemResponse->setIsBuyable(0);
          $oItemResponse->setStockQuantity(0);
          $oItemResponse->setError(300);
          $oItemResponse->setErrorText('item out of stock');
        }
        elseif ($this->oExportArticle->cStatusAktiv == 'stock') {
          $oItemResponse->setStockQuantity($this->oExportArticle->fLagerbestand);
          if($this->oExportArticle->fLagerbestand < 1) {
            $oItemResponse->setError(300);
            $oItemResponse->setErrorText('item out of stock');
          }
          else {
            $oItemResponse->setIsBuyable(1);
          }
        }
        else {
          if (!$this->oExportArticle->nUseStock) {
            $oItemResponse->setStockQuantity(100);
          }
          else {
            $oItemResponse->setStockQuantity($this->oExportArticle->fLagerbestand);
          }
          $oItemResponse->setIsBuyable(1);
        }
        /* BOF Konfiguration */
        $tmpOptions = array();
        $tmpKonfigurationArticle = array();
        if (!empty($itemOptions)) {
          foreach ($itemOptions as $checkOption) {
            $option_number = $checkOption->getOptionNumber();
            if (substr($option_number,0,10) == 'agiKonfig_')
              $tmpKonfigurationArticle[] = $checkOption;
            else
              $tmpOptions[] = $checkOption;
          }
        }
        $itemOptions = $tmpOptions;
        if ($this->bAllowKonfiguration && $this->oExportArticle->bHasKonfig && !empty($this->oExportArticle->oKonfig_arr)) {
          $aGroupsConfiged = array();
          $itemerror = false;
          if (!empty($tmpKonfigurationArticle)) {
            foreach ($tmpKonfigurationArticle as $oKonfigurationOption) {
              $option_number = $oKonfigurationOption->getOptionNumber();
              $value_number = $oKonfigurationOption->getValueNumber();
              if ($value_number) {
                //array(kKonfiggruppe ,kKonfigitem, kArtikel, Anzahl);
                $aKonfigKeys = explode('_', $value_number);
                if (count($aKonfigKeys) == 4) {
                  $kItem = new Konfigitem($aKonfigKeys[1]);
                  $kGruppe = $kItem->getKonfiggruppe();
                  if (!array_key_exists($aKonfigKeys[0], $aGroupsConfiged)) {
                    $oGruppe = new Konfiggroup($kGruppe);
                    $aGroupsConfiged[$aKonfigKeys[0]] = array('nMin' => $oGruppe->getMin(), 'nMax' => $oGruppe->getMax(), 'name' => '', 'cnt'=>0);
                  }
                  if ($kGruppe == $aKonfigKeys[0] && !empty($aKonfigKeys[2]) && $kItem->getArtikelKey() == $aKonfigKeys[2]) {
                    if ($aKonfigKeys[3] > 0) {
                      $aGroupsConfiged[$aKonfigKeys[0]]['cnt'] += 1;
                    }
                    $tmpArticle = $kItem->getArtikel();
                    if ($tmpArticle->inWarenkorbLegbar == -1) {
                      $oItemResponse->setIsBuyable(0);
                      $oItemResponse->setError(301);
                      $oItemResponse->setErrorText('item product-option not found');
                      $itemerror = true;
                    }
                    else {
                      if ($tmpArticle->cLagerBeachten == 'Y' && $tmpArticle->cLagerKleinerNull == 'N' && $tmpArticle->fLagerbestand <= 0) {
                        $oItemResponse->setIsBuyable(0);
                        $oItemResponse->setError(301);
                        $oItemResponse->setErrorText('item product-option not found');
                        $itemerror = true;
                      }
                    }
                  }
                  else {
                    if (!empty($aKonfigKeys[2])) {
                      $oItemResponse->setIsBuyable(0);
                      $oItemResponse->setError(301);
                      $oItemResponse->setErrorText('item product-option not found');
                      $itemerror = true;
                    }
                  }
                }
                else {
                  $oItemResponse->setIsBuyable(0);
                  $oItemResponse->setError(301);
                  $oItemResponse->setErrorText('item product-option not found');
                  $itemerror = true;
                }
              }
            }
          }
          if (!$itemerror) {
            foreach ($aGroupsConfiged as $value) {
              if ($value['nMin'] > 0 && $value['nMin']>$value['cnt']) {
                $oItemResponse->setIsBuyable(0);
                $oItemResponse->setError(301);
                $oItemResponse->setErrorText($this->_prepare_utf8('Bitte whlen Sie mindestens eine Option aus'));
                break;
              }
              if ($value['nMax'] > 0 && $value['nMax']<$value['cnt']) {
                $oItemResponse->setIsBuyable(0);
                $oItemResponse->setError(301);
                $oItemResponse->setErrorText($this->_prepare_utf8('Sie haben zu viele Optionen gewhlt'));
                break;
              }
            }
          }
        }
        elseif(!empty($tmpKonfigurationArticle)) {
          $bAllowForBasket = false;
          $oItemResponse->setIsBuyable(0);
          $oItemResponse->setError(301);
          $oItemResponse->setErrorText('item product has no options');
        }
        /* EOF Konfiguration */
        if (!empty($itemOptions) && !empty($this->oExportArticle->aSelectAttribute)) {
          foreach ($itemOptions as $checkOption)
          {
            $option_number = $checkOption->getOptionNumber();
            $value_number = $checkOption->getValueNumber();
            if (
              array_key_exists($option_number, $this->oExportArticle->aSelectAttribute) 
              && array_key_exists($value_number, $this->oExportArticle->aSelectAttribute[$option_number]['aWerte']))
            {
              $aOption = $this->oExportArticle->aSelectAttribute[$option_number]['aWerte'][$value_number];
              if ($this->oExportArticle->cStatusAktiv == 'active' || ($this->oExportArticle->cStatusAktiv == 'stock' && ($aOption['fLagerbestand'] > 0 || $this->oExportArticle->cLagerVariation == 'N'))) {
                if ($this->oExportArticle->cStatusAktiv == 'stock') {
                  if ($this->oExportArticle->cLagerVariation == 'Y' && $aOption['fLagerbestand'] <= 0) {
                    $oItemResponse->setStockQuantity(0);
                    $oItemResponse->setIsBuyable(0);
                    $oItemResponse->setError(300);
                    $oItemResponse->setErrorText('cart item out of stock');
                  }
                }
              }
              else {
                $oItemResponse->setIsBuyable(0);
                $oItemResponse->setStockQuantity(0);
                $oItemResponse->setError(300);
                $oItemResponse->setErrorText('cart item out of stock');
              }
            }
            else {
              $oItemResponse->setError(301);
              $oItemResponse->setErrorText('cart item product-option not found');
            }
          }
        }
        elseif (!empty($itemOptions) && empty($this->oExportArticle->aSelectAttribute)) {
          $oItemResponse->setError(301);
          $oItemResponse->setErrorText('cart item product has no options');
        }
      }
      else {
        $oItemResponse->setError(301);
        $oItemResponse->setErrorText('item product not found');
      }
      
      $aItemResponse[] = $oItemResponse;
    }
    return $aItemResponse;
  }

  /**
    * Updates and returns synchronization information for the favourite list of a customer.
    *
    * @see http://developer.shopgate.com/plugin_api/customers/sync_favourite_list
    *
    * @param string $customerToken
    * @param ShopgateSyncItem[] $items A list of ShopgateSyncItem objects that need to be synchronized
    *
    * @return ShopgateSyncItem[] The updated list of ShopgateSyncItem objects
    */
  public function syncFavouriteList($customerToken, $items) 
  {
   	// Database connection can be established here.
   	// Check if the user's customer token exists in the shop
   	// If user data is available, save them in, for instance, the $dbCustomerData array.
   	
   	if ($customerNotFoundByToken) {
   		throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_CUSTOMER_TOKEN_INVALID);
   	}
   	
   	// Fetch the current favourite list into $dbFavouriteList
   	// ...
   	
   	// Walk through the list passed by Shopgate and update the local list
   	foreach ($items as $item) {
   		/* @var $item ShopgateSyncItem */
   		
   		$index = array_search($item->getItemNumber(), $dbFavouriteList);
   		switch ($item->getStatus()) {
   			case 'new':
   				if ($index === false) {
   					$dbFavouriteList[] = $item->getItemNumber();
   				}
   			break;
   			case 'deleted':
   				if ($index !== false) {
   					unset($dbFavouriteList[$index]);
   				}
   			break;
   		}
   	}
   	
   	// Save the updated list to the database
   	// ...
   	
   	// Create the list to be returned to Shopgate
   	$newFavouriteList = array();
   	foreach ($dbFavouriteList as $itemNumber) {
   		$newFavouriteList[] = new ShopgateSyncItem(array('item_number' => $itemNumber));
   	}
   	
   	// Return the new favourite list
   	return $newFavouriteList;
  }
  
  /**
   * not used, but necessary
   */
  public function createMediaCsv()
  {
    
  }
  
  /**
   * remove all html-Tags from the given string
   * @param   string  $string
   * @return  string
   */
  private function _remove_html($string)
  {
    $replace_to = array('<br>','<br />','<p>','</p>','&nbsp;','</tr>','</td>','</li>'); // needles
    $replace_with = array(chr(10),chr(10),chr(10).chr(10),chr(10),' ',chr(10),chr(9),chr(10)); // replace-with
    
    $string = str_replace($replace_to, $replace_with, $string);
    $string = strip_tags($string);
    $string = Text::htmlentitydecode($string);
    $string = trim($string);
    return $string;
  }

  /**
   * encode/decode utf8
   * @param   string  $string
   * @param   bool  encode
   * @return  string
   */
  private function _prepare_utf8($string, $encode=true) {
    if ($this->UTF8_SHOP) {
      return $string;
    }
    return Text::utf8_convert_recursive($string, $encode);
  }

  private function _getKuponErrorMessage($code)
  {
    switch ($code) {
      case 1: return 'Der Kupon ist nicht aktiv (1)'; break;
      case 2: return 'Der Kupon ist abgelaufen (2)'; break;
      case 3: return 'Der Kupon ist nicht aktiv (3)'; break;
      case 4: return 'Der Mindestbestellwert fr diesen Kupon ist nicht erreicht (4)'; break;
      case 5: return 'Kupon ist nicht aktiv (5)'; break;
      case 6: return 'Kupon ist nicht mehr aktiv (6)'; break;
      case 7: return 'Artikel im Warenkorb sind nicht Kupon-fhig (7)'; break;
      case 8: return 'Artikel im Warenkorb sind nicht Kupon-fhig (8)'; break;
      case 9: return 'Kupon ist nicht aktiv (9)'; break;
      case 10: return 'Der Kupon kann nicht mit dem gewhlten Lieferland kombiniert werden. (10)'; break;
      default: return 'Der Kupon kann nicht genutzt werden';
    }
  }
  
  protected function _checkCoupon($oKupon, $aKunde, $kKundengruppe, $aWarenwert, $aPositionen, $deliveryCountry=false)
  {
    $ungueltig = 0;
    if ($oKupon->cAktiv!="Y")
      $ungueltig = 1;
    if (!empty($oKupon->dGueltigBis) && strtotime($oKupon->dGueltigBis) < time() && $oKupon->dGueltigBis != '0000-00-00 00:00:00')
      $ungueltig = 2;
    if (strtotime($oKupon->dGueltigAb)>time())
      $ungueltig = 3;
    if ($oKupon->fMindestbestellwert>$aWarenwert['netto'])
      $ungueltig = 4;
    if ($oKupon->kKundengruppe>0 && $oKupon->kKundengruppe!=$kKundengruppe)
      $ungueltig = 5;
    if ($oKupon->nVerwendungen>0 && $oKupon->nVerwendungen<=$oKupon->nVerwendungenBisher)
      $ungueltig = 6;
    if ($oKupon->cArtikel && $this->_hatKuponfaehigeArtikel($oKupon,$aPositionen))
      $ungueltig = 7;
    if ($oKupon->cKategorien && $oKupon->cKategorien!=-1 && $this->_hatKuponfaehigeKategorien($oKupon,$aPositionen))
      $ungueltig = 8;
    if (!empty($oKupon->cKunden) && $oKupon->cKunden!=-1 && $aKunde['kKunde']>0 && $oKupon->cKuponTyp!="neukundenkupon") {
      $tmp = explode(';', $oKupon->cKunden);
      if (!in_array($aKunde['kKunde'], $tmp)) {
        $ungueltig = 9;
      }
    }
    if ($oKupon->cKuponTyp === 'versandkupon' && $deliveryCountry && strpos($oKupon->cLieferlaender, $deliveryCountry) === false)
      $ungueltig = 10;
      
    if (!empty($aKunde['cMail'])) {
      //hat der kunde schon die max. Verwendungsanzahl erreicht?
      $aRes = $this->db->queryPrepared(
        "SELECT nVerwendungen FROM tkuponkunde WHERE cMail= :cMail AND kKupon= :kKupon",
        ['cMail' => $aKunde['cMail'], 'kKupon' => $oKupon->kKupon],
        ReturnType::SINGLE_OBJECT
      );
      if ($aRes) {
        if ($aRes->nVerwendungen >= $oKupon->nVerwendungenProKunde && $oKupon->nVerwendungenProKunde>0) {
          $ungueltig = 6;
        }
      }
    }
    
    return $ungueltig;
  }

  private function _hatKuponfaehigeArtikel($oKupon,$aPositionen)
  {
    if (is_array($aPositionen)) {
      $aKouponArticle = explode(';',$oKupon->cArtikel);
      foreach ($aPositionen as $Pos) {
        if (in_array($Pos['cArtNr'], $aKouponArticle)) {
          return false;
        }
      }
    }
    else {
      return false;
    }
    return true;
  }

  private function _hatKuponfaehigeKategorien($oKupon,$aPositionen)
  {
    if (is_array($aPositionen)) {
      $kats = array();
      foreach ($aPositionen as $Pos) {
        $tKats = $Pos['aKategorien'];
        if($this->_pruefeIstVariKind($Pos['kArtikel'])) {
          $kArtikel = $this->_gibkVaterArtikel($Pos['kArtikel']);
          $tKats = array();
          $Kats_arr = $this->db->queryPrepared(
            "SELECT kKategorie FROM tkategorieartikel WHERE kArtikel = :kArtikel",
            ['kArtikel' => $kArtikel],
            ReturnType::ARRAY_OF_OBJECTS
          );
          if (is_array($Kats_arr)) {
            foreach ($Kats_arr as $Kat) {
              if (!in_array($Kat->kKategorie,$tKats)) {
                $tKats[] = $Kat->kKategorie;
              }
            }
          }
        }
        if (!empty($tKats)) {
          $kats = array_merge($kats, $tKats);
        }
      }
      $kats = array_unique($kats);
      foreach ($kats as $kat) {
        if(stristr($oKupon->cKategorien, $kat . ";") !== false) {
          return false;
        }
      }
    }
    else {
      return false;
    }
    return true;
  }
  
  private function _pruefeIstVariKind($kArtikel)
  {
    if(intval($kArtikel) > 0) {
      $oArtikel = $this->db->queryPrepared(
        "SELECT kEigenschaftKombi FROM tartikel WHERE kArtikel= :kArtikel",
        ['kArtikel' => (int)$kArtikel],
        ReturnType::SINGLE_OBJECT
      );
      if(isset($oArtikel->kEigenschaftKombi) && intval($oArtikel->kEigenschaftKombi) > 0) {
        return true;
      }
    }
    return false;
  }
  
  private function _gibkVaterArtikel($kArtikel)
  {
    $kArtikel = intval($kArtikel);	
    if($kArtikel > 0) {
      $oArtikel = $this->db->queryPrepared(
        "SELECT kVaterArtikel FROM tartikel WHERE kArtikel= :kArtikel",
        ['kArtikel' => (int)$kArtikel],
        ReturnType::SINGLE_OBJECT
      );
      if (isset($oArtikel->kVaterArtikel) && intval($oArtikel->kVaterArtikel) > 0) {
        return $oArtikel->kVaterArtikel;
      }
    }
    return 0;
  }
  
  private function _gibMoeglicheVersandarten($lieferland, $plz, $versandklassen, $kKundengruppe)
  {
    $moeglicheVersandarten = array();
    $minVersand = 10000;
    
    if (empty($versandklassen)) {
      $versandklassen = $this->JTLPluginConf['nStandardVersandklasse'];
    }
    if (!$lieferland) {
      $lieferland = $this->cLieferLand;
    }
    $cISO = $lieferland;

    $cNurAbhaengigeVersandart = "N";
    if (ShippingMethod::normalerArtikelversand($lieferland) == false) {
      $cNurAbhaengigeVersandart = "Y";
    }
    
    $versandarten = $this->db->queryPrepared(
      "SELECT * FROM `tversandart` WHERE `cNurAbhaengigeVersandart`='" . $cNurAbhaengigeVersandart . "'
        AND `cLaender` LIKE \"%" . $cISO . "%\"
        AND (`cVersandklassen`='-1' OR (`cVersandklassen` LIKE \"% " . $versandklassen . " %\" OR `cVersandklassen` LIKE \"% " . $versandklassen . "\"))
        AND (`cKundengruppen`='-1' OR `cKundengruppen` LIKE '%;" . $kKundengruppe . ";%') ORDER BY `nSort`",
      [],
      ReturnType::ARRAY_OF_OBJECTS
    );
    $cnt = count($versandarten);

    for ($i=0; $i<$cnt; $i++) {
      $versandarten[$i]->Zuschlag = $plz ? $this->_gibVersandZuschlag($versandarten[$i], $cISO, $plz) : null;
      $versandarten[$i]->fEndpreis = ShippingMethod::calculateShippingFees($versandarten[$i], $cISO, null);
      $name_spr = $this->db->queryPrepared(
        "SELECT cName, cLieferdauer, cHinweistext FROM tversandartsprache WHERE kVersandart= :kVersandart AND cISOSprache= :cISOSprache",
        ['kVersandart' => $versandarten[$i]->kVersandart, 'cISOSprache' => Shop::Lang()->getIsoFromLangID((int)$this->kSprache)->cISO],
        ReturnType::SINGLE_OBJECT
      );
      $versandarten[$i]->angezeigterName = $name_spr->cName;
			$versandarten[$i]->angezeigterHinweistext = $name_spr->cHinweistext;
			$versandarten[$i]->cLieferdauer = $name_spr->cLieferdauer;
      if ($versandarten[$i]->fEndpreis == -1) {
        unset($versandarten[$i]);
        continue;
      }
      if ($versandarten[$i]->fEndpreis < $minVersand) {
        $minVersand = $versandarten[$i]->fEndpreis;
      }
    }
    //auf anzeige filtern
    foreach ($versandarten as $versandart) {
      switch ($versandart->cAnzeigen) {
        case 'guenstigste':
          if ($versandart->fEndpreis <= $minVersand) {
            array_push($moeglicheVersandarten,$versandart);
          }
          break;
        case 'immer':
            array_push($moeglicheVersandarten,$versandart);
          break;
      }
    }
    return $moeglicheVersandarten;
  }
  
  private function _gibVersandZuschlag($versandart, $cISO, $plz)
  {
    $waehrung = $this->oWaehrung;
    $versandzuschlaege = $this->db->queryPrepared(
      "SELECT * FROM tversandzuschlag WHERE kVersandart= :kVersandart AND cISO= :cISO",
      ['kVersandart' => $versandart->kVersandart, 'cISO' => $cISO],
      ReturnType::ARRAY_OF_OBJECTS
    );
    foreach ($versandzuschlaege as $versandzuschlag) {
      $plz_x = $this->db->queryPrepared(
        "SELECT * FROM tversandzuschlagplz WHERE ((cPLZAb<= :cPLZAb AND cPLZBis>= :cPLZBis) OR cPLZ= :cPLZ) AND kVersandzuschlag= :kVersandzuschlag",
        ['cPLZAb' => $plz, 'cPLZBis' => $plz, 'cPLZ' => $plz, 'kVersandzuschlag' => $versandzuschlag->kVersandzuschlag],
        ReturnType::SINGLE_OBJECT
      );
      if ($plz_x && $plz_x->kVersandzuschlagPlz>0) {
        $versandzuschlag->fZuschlag = number_format($versandzuschlag->fZuschlag*$waehrung->fFaktor, 2, $waehrung->cTrennzeichenCent, $waehrung->cTrennzeichenTausend);
        return $versandzuschlag;
      }
    }
    return null;
  }
  
  protected function _get_optionsPriceNetto($options, $tax = 19)
  {
    $sumNet = 0;
    if (!empty($options)) {
      foreach($options as $option) {
        $sumNet += round($option->getAdditionalAmountWithTax()*100/(100+$tax), 4);
      }
    }
    return $sumNet;
  }
  
  protected function _gibUID(int $length = 40) 
  {
    $uid      = '';
    $salt     = '';
    $saltBase = 'aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ0123456789';
    // Gen SALT
    for ($j = 0; $j < 30; $j++) {
      $salt .= mb_substr($saltBase, mt_rand(0, mb_strlen($saltBase) - 1), 1);
    }
    $salt = md5($salt);
    mt_srand();
    $passwordService = Shop::Container()->getPasswordService();
    $uid = $passwordService->hash(md5(M_PI . $salt . md5((string)(time() - mt_rand()))));
    return $length > 0 ? mb_substr($uid, 0, $length) : $uid;
  }
  
  private function _getKVersandartByName($cShippingName, $aWarenkorbVersandklassen, $kKundengruppe, $cLieferland = 'DE')
  {
    $kVersandart = 0;
    $aVersandartSprache = $this->db->queryPrepared(
      "SELECT * FROM tversandartsprache WHERE cName= :cName",
      ['cName' => $cShippingName],
      ReturnType::ARRAY_OF_OBJECTS
    );
    if (count($aVersandartSprache) > 1) {
      $diff_keys = false;
      $check_key = $aVersandartSprache[0]->kVersandart;
      foreach ($aVersandartSprache as $versandartSprache) {
        if ($check_key <> $versandartSprache->kVersandart) {
          $diff_keys = true;
          break;
        }
        $check_key = $versandartSprache->kVersandart;
      }
      $kVersandart = $check_key;
      if ($diff_keys) {
        sort($aWarenkorbVersandklassen);
        $cVersandklassen = implode('-', $aWarenkorbVersandklassen);
        $versandarten = $this->db->queryPrepared(
          "SELECT kVersandart FROM tversandart WHERE cNurAbhaengigeVersandart= :cNurAbhaengigeVersandart AND cLaender LIKE :cLaender 
          AND (
            cVersandklassen='-1' OR (cVersandklassen LIKE :cVersandklassen_1 OR cVersandklassen LIKE :cVersandklassen_2)
          )
          AND (
            cKundengruppen='-1' OR cKundengruppen LIKE :cKundengruppen
          )
          order by nSort",
          [
            'cNurAbhaengigeVersandart' => $this->cNurAbhaengigeVersandart, 
            'cLaender' => '%'.$cLieferland.'%', 
            'cVersandklassen_1' => '% '.$cVersandklassen.' %',
            'cVersandklassen_2' => '% '.$cVersandklassen,
            'cKundengruppen'  => '%;'.$kKundengruppe.';%'
          ],
          ReturnType::ARRAY_OF_OBJECTS
        );
        $cnt = count($versandarten);
        if ($cnt > 0) {
          $aVersand = array();
          foreach ($versandarten as $value) {
            $aVersand[] = $value->kVersandart;
          }
          foreach ($aVersandartSprache as $oVersandartSprache) {
            if (in_array($oVersandartSprache->kVersandart, $aVersand)) {
              $kVersandart = $oVersandartSprache->kVersandart;
              break;
            }
          }
        }
      }
    }
    elseif (count($aVersandartSprache) == 1) {
      $kVersandart = $aVersandartSprache[0]->kVersandart;
    }
    return $kVersandart;
  }
  
  /* BOF GPSR 2/2 */
  private function _build_gpsr_address($a_address, $group) {
    if (method_exists($this, 'agi_shopgate_connect__build_gpsr_address')) {
      return $this->agi_shopgate_connect__build_gpsr_address($a_address, $group);
    }
    $address = array();
    if (array_key_exists('name', $a_address)) {
      $address[] = $a_address['name'];
    }
    
    $s = array();
    if (array_key_exists('street', $a_address)) {
      $s[] = $a_address['street'];
    }
    if (array_key_exists('housenumber', $a_address)) {
      $s[] = $a_address['housenumber'];
    }
    if (!empty($s)) {
      $address[] = implode(' ', $s);
    }
    
    $s = array();
    if (array_key_exists('postalcode', $a_address)) {
      $s[] = $a_address['postalcode'];
    }
    if (array_key_exists('city', $a_address)) {
      $s[] = $a_address['city'];
    }
    if (!empty($s)) {
      $address[] = implode(' ', $s);
    }
    
    $s = array();
    if (array_key_exists('country', $a_address)) {
      $s[] = $a_address['country'];
    }
    if (array_key_exists('state', $a_address)) {
      $s[] = $a_address['state'];
    }
    if (!empty($s)) {
      $address[] = implode(', ', $s);
    }
    
    $s = array();
    if (array_key_exists('email', $a_address)) {
      $s[] = $a_address['email'];
    }
    if (array_key_exists('homepage', $a_address)) {
      $s[] = $a_address['homepage'];
    }
    if (!empty($s)) {
      $address[] = implode(', ', $s);
    }
    
    return implode('<br />'.chr(10), $address);
  }
  /* EOF GPSR 2/2 */
  
  /*
    @param $order: Shopgate $order-Object
    @param $return_result: array
    return array
  */
  public function agi_pluginsPaymentMethod_DEBIT($order, $return_result) {
    $paymentInfos = $order->getPaymentInfos();
    $return_result['tZahlungsinfo'] = array(
      'cBankName' => $this->_prepare_utf8($paymentInfos["bank_name"], false),
      'cIBAN'     => $this->_prepare_utf8($paymentInfos["iban"], false),
      'cBIC'      => $this->_prepare_utf8($paymentInfos["bic"], false),
      'cInhaber'  => $this->_prepare_utf8($paymentInfos["bank_account_holder"], false),
      'cAbgeholt' => 'N'
    );
    $return_result['paymentComment'] = '';
    $return_result['paymentComment'] .= 'bank_account_holder: '.$this->_prepare_utf8($paymentInfos["bank_account_holder"], false).chr(10);
    $return_result['paymentComment'] .= 'iban: '.$this->_prepare_utf8($paymentInfos["iban"], false).chr(10);
    $return_result['paymentComment'] .= 'bank_name: '.$this->_prepare_utf8($paymentInfos["bank_name"], false).chr(10);
    $return_result['paymentComment'] .= 'bic: '.$this->_prepare_utf8($paymentInfos["bic"], false).chr(10);
    
    return $return_result;
  }
  public function agi_pluginsPaymentMethod_BILLSAFE($order, $return_result) {
    // Prfe ob Plugin existiert und aktiv
    $pData = $this->db->queryPrepared(
      "SELECT * FROM tplugin WHERE cPluginID= :cPluginID",
      ['cPluginID' => 'billsafe_sus_its'],
      ReturnType::SINGLE_OBJECT
    );
    if (!empty($pData)) {
      $paymentInfos = $order->getPaymentInfos();
      $oPlugin = PluginHelper::getPluginById('billsafe_sus_its');
      if (!empty($oPlugin -> oPluginEinstellungAssoc_arr['merchantId']) && !empty($oPlugin -> oPluginEinstellungAssoc_arr['applicationSignature'])) {
        require_once ($oPlugin -> getAdminPath() . "BillsafeSDK/Sdk.php");
        require_once ($oPlugin -> getAdminPath() . "BillsafeSDK/HttpResponse.php");
        require_once ($oPlugin -> getAdminPath() . "BillsafeSDK/Exception.php");
        require_once ($oPlugin -> getAdminPath() . "BillsafeSDK/HttpClient.php");
        require_once ($oPlugin -> getAdminPath() . "BillsafeSDK/Logger.php");
        require_once ($oPlugin -> getAdminPath() . "BillsafeSDK/LoggerNull.php");
        $oPlugin -> name = 'Billsafe';
        $oPlugin -> caption = 'Billsafe';
        $oPlugin -> bs = new Billsafe_Sdk();
        $oPlugin -> bs -> setCredentials(array('merchantId' => $oPlugin -> oPluginEinstellungAssoc_arr['merchantId'], 'applicationSignature' => $oPlugin -> oPluginEinstellungAssoc_arr['applicationSignature'], 'applicationVersion' => $oPlugin -> nVersion));
        if ($oPlugin -> oPluginEinstellungAssoc_arr["sandboxmodus"] == 'False') {
          $oPlugin -> bs -> setCredentials(array('merchantLicenseLive' => $oPlugin -> oPluginEinstellungAssoc_arr["merchantLicenseLive"]));
          $oPlugin -> bs -> setMode('LIVE');
        } 
        else {
          $oPlugin -> bs -> setCredentials(array('merchantLicenseSandbox' => $oPlugin -> oPluginEinstellungAssoc_arr["merchantLicenseSandbox"]));
          $oPlugin -> bs -> setMode('SANDBOX');
        }
        $comment_params = array('transactionId' => $this->_prepare_utf8($paymentInfos['billsafe_transaction_id'], false), 'outputType' => 'STRUCTURED');
        $comment_response = null; $callMethodCount = 0; $callMethodCountMax = 3; 
        while( empty($comment_response) && $callMethodCount < $callMethodCountMax ) {
          usleep(400);
          $comment_response = $oPlugin -> bs -> callMethod('getPaymentInstruction', $comment_params);
          $callMethodCount++;
        }
        if ($comment_response -> ack == 'OK') {
          $comment_text .= 'Bitte berweisen Sie den Rechnungsbetrag von ' . number_format($comment_response -> instruction -> amount, 2) . ' ' . $comment_response -> instruction -> currencyCode . ' auf folgendes Konto:\n';
          $comment_text .= $comment_response -> instruction -> recipient . '\n';
          $comment_text .= $comment_response -> instruction -> bankName . '\n';
          //$comment_text .= 'KTN:  ' . $comment_response -> instruction -> accountNumber . '\n';
          //$comment_text .= 'BLZ:  ' . $comment_response -> instruction -> bankCode . '\n';
          $comment_text .= 'IBAN: ' . $comment_response -> instruction -> iban . '\n';
          $comment_text .= 'BIC:  ' . $comment_response -> instruction -> bic . '\n';
          $comment_text .= 'Verwendungszweck: ' . $comment_response -> instruction -> reference . '\n\n';
          $comment_text .= $comment_response -> instruction -> note . '\n';
          $comment_text .= $comment_response -> instruction -> legalNote . '\n';
          $return_result['paymentComment'] = $comment_text;
        }
      }
    }
    return $return_result;
  }
  public function agi_pluginsPaymentMethod_PAYPAL($order, $return_result) {
    $paymentInfos = $order->getPaymentInfos();
    if (strtolower($this->_prepare_utf8($paymentInfos["payment_status"], false)) == 'completed') {
      $return_result['tZahlungseingang'] = array(
        'cZahlungsanbieter' => $return_result['cZahlart'],
        'fZahlungsgebuehr'  => $this->_prepare_utf8($paymentInfos["mc_fee"], false),
        'fBetrag'           => $this->_prepare_utf8($paymentInfos["mc_gross"], false),
        'cISO'              => $this->_prepare_utf8($paymentInfos["mc_currency"], false),
        'cEmpfaenger'       => $this->_prepare_utf8($paymentInfos["receiver_email"], false),
        'cZahler'           => $this->_prepare_utf8($paymentInfos["payer_email"], false),
        'cHinweis'          => $this->_prepare_utf8($paymentInfos["transaction_id"], false),
        'cAbgeholt'         => 'N',
        'dZeit'             => $order->getPaymentTime('Y-m-d H:i:s'),
      );
    }
    else {
      unset($return_result['tZahlungseingang']);
    }
    return $return_result;
  }
  public function agi_pluginsPaymentMethod_PPAL_PLUS($order, $return_result) {
    $paymentInfos = $order->getPaymentInfos();
    if (strtolower($this->_prepare_utf8($paymentInfos["status"], false)) == 'completed') {
      //$tmp = $return_result['tZahlungseingang'];
      $return_result['tZahlungseingang'] = array(
        'cZahlungsanbieter' => $return_result['cZahlart'],
        'fZahlungsgebuehr'  => 0,//$this->_prepare_utf8($paymentInfos["mc_fee"]),
        'fBetrag'           => $this->_prepare_utf8($paymentInfos["transaction_info"][0]['amount']['total'], false),
        'cISO'              => $this->_prepare_utf8($paymentInfos["transaction_info"][0]['amount']["currency"], false),
        'cEmpfaenger'       => $this->_prepare_utf8($paymentInfos["receiver_email"], false),
        'cZahler'           => $this->_prepare_utf8($paymentInfos["payer_email"], false),
        'cHinweis'          => $this->_prepare_utf8($paymentInfos["payment_transaction_id"], false),
        'cAbgeholt'         => 'N',
        'dZeit'             => $order->getPaymentTime('Y-m-d H:i:s')
      );
    }
    else {
      unset($return_result['tZahlungseingang']);
    }
    $return_result['cSession'] = $paymentInfos['payment_id'];
    
    if (isset($paymentInfos['payment_info']['payment_instruction'])) {
      if ($paymentInfos['payment_info']['payment_instruction']['instruction_type'] == 'PAY_UPON_INVOICE') {
        $pData = $this->db->queryPrepared(
          "SELECT * FROM tplugin WHERE cPluginID= :cPluginID",
          ['cPluginID' => 'jtl_paypal'],
          ReturnType::SINGLE_OBJECT
        );
        if (!empty($pData)) {
          $oPlugin = PluginHelper::getPluginById('jtl_paypal');
          $instruction = $paymentInfos['payment_info']['payment_instruction'];
          $company = new Firma();
          
          $replacement = array(
            '%reference_number%'    => $instruction['reference_number'],
            '%bank_name%'           => $instruction['recipient_banking_instruction']['bank_name'],
            '%account_holder_name%' => $instruction['recipient_banking_instruction']['account_holder_name'],
            '%international_bank_account_number%' => $instruction['recipient_banking_instruction']['international_bank_account_number'],
            '%bank_identifier_code%'=> $instruction['recipient_banking_instruction']['bank_identifier_code'],
            '%value%'             => $instruction['amount'],
            '%payment_due_date%'  => strftime('%d.%m.%Y', strtotime($instruction['payment_due_date'])),
            '%company%'           => $company->cName
          );
        
          $pui = sprintf(
            "%s\r\n\r\n%s",
            $oPlugin->getLocalization()->getTranslation('jtl_paypal_pui'),
            $oPlugin->getLocalization()->getTranslation('jtl_paypal_pui_legal')
          );

          $return_result['cPUIZahlungsdaten'] = str_replace(
            array_keys($replacement),
            array_values($replacement),
            $pui
          );
        }
        if (!empty($return_result['tZahlungseingang'])) {
          $return_result['tZahlungseingang']['cZahlungsanbieter'] = 'Rechnung (PayPal PLUS)';
        }
      }
      $return_result['cZahlart'] = 'Rechnung (PayPal PLUS)';
      if (strtolower($this->_prepare_utf8($paymentInfos["status"], false)) == 'completed'){
        $return_result['tZahlungseingang']['cZahlungsanbieter'] = $return_result['cZahlart'];
      }
    }
    return $return_result;
  }
  public function agi_pluginsPaymentMethod_PPC_ALL($order, $return_result) {
    $paymentInfos = $order->getPaymentInfos();

    if (strtolower($this->_prepare_utf8($paymentInfos["status"], false)) == 'completed') {
      $return_result['tZahlungseingang'] = array(
        'cZahlungsanbieter' => $return_result['cZahlart'],
        'fZahlungsgebuehr'  => 0,//$this->_prepare_utf8($paymentInfos["mc_fee"]),
        'fBetrag'           => $order->getAmountComplete(),
        'cISO'              => $order->getCurrency(),
        'cHinweis'          => $this->_prepare_utf8($paymentInfos["transaction_id"], false),
        'cAbgeholt'         => 'N',
        'dZeit'             => $order->getPaymentTime('Y-m-d H:i:s'),
        
      );
    }
    else {
      unset($return_result['tZahlungseingang']);
    }
    return $return_result;
  }
  public function agi_pluginsPaymentMethod_PPC_PAYPAL($order, $return_result) {
    return $this->agi_pluginsPaymentMethod_PPC_ALL($order, $return_result);
  }
  public function agi_pluginsPaymentMethod_PPC_CC($order, $return_result) {
    return $this->agi_pluginsPaymentMethod_PPC_ALL($order, $return_result);
  }
  public function agi_pluginsPaymentMethod_PPC_INV($order, $return_result) {
    return $this->agi_pluginsPaymentMethod_PPC_ALL($order, $return_result);
  }
  public function agi_pluginsPaymentMethod_PPC_SEPA($order, $return_result) {
    return $this->agi_pluginsPaymentMethod_PPC_ALL($order, $return_result);
  }
  public function agi_pluginsPaymentMethod_PPC_EPS($order, $return_result) {
    return $this->agi_pluginsPaymentMethod_PPC_ALL($order, $return_result);
  }
  public function agi_pluginsPaymentMethod_PPC_GP($order, $return_result) {
    return $this->agi_pluginsPaymentMethod_PPC_ALL($order, $return_result);
  }
  public function agi_pluginsPaymentMethod_PPC_IDEAL($order, $return_result) {
    return $this->agi_pluginsPaymentMethod_PPC_ALL($order, $return_result);
  }
  public function agi_pluginsPaymentMethod_PPC_SOFORT($order, $return_result) {
    return $this->agi_pluginsPaymentMethod_PPC_ALL($order, $return_result);
  }
  public function agi_pluginsPaymentMethod_MWS($order, $return_result) {
    $paymentInfos = $order->getPaymentInfos();
    if ($return_result['bSetPayed']) {
      //$tmp = $return_result['tZahlungseingang'];
      $return_result['tZahlungseingang'] = array(
        'cZahlungsanbieter' => $return_result['cZahlart'],
        'fZahlungsgebuehr'  => 0,//$this->_prepare_utf8($paymentInfos["mc_fee"], false),
        'fBetrag'           => $order->getAmountComplete(),
        'cISO'              => $order->getCurrency(),
        //'cEmpfaenger'       => $this->_prepare_utf8($paymentInfos["payer_email"], false),
        //'cZahler'           => $this->_prepare_utf8($paymentInfos["receiver_email"], false),
        'cHinweis'          => $this->_prepare_utf8($paymentInfos["mws_order_id"], false),
        'cAbgeholt'         => 'N',
        'dZeit'             => date('Y-m-d H:i:s')
      );
    }
    else {
      unset($return_result['tZahlungseingang']);
    }
    // if plugin_exists 
    if ($this->aTeinstellungenFields['s360_amazonpay_shop5'] && $this->aTeinstellungenFields['s360_amazonpay_shop5_active']) {
      $return_result['kZahlart'] = 0;
      /*
      $db_to_order = $this->aTeinstellungenFields['bHasS360_amazon_lpa_shop4_db'] ? 'xplugin_s360_amazon_lpa_shop4_torder' : 'xplugin_s360_amazon_lpa_torder';
      $db_to_auth  = $this->aTeinstellungenFields['bHasS360_amazon_lpa_shop4_db'] ? 'xplugin_s360_amazon_lpa_shop4_tauthorization' : 'xplugin_s360_amazon_lpa_tauthorization';
      $return_result['insertToTables'] = array(
        $db_to_order  => array(
          'kBestellung'       => '@@ORDER_ID@@',
          'cOrderReferenceId' => $this->_prepare_utf8($paymentInfos["mws_order_id"], false),
          'cOrderStatus'      => 'Open',//???
          'fOrderAmount'      => $order->getAmountComplete(),
          'cOrderCurrencyCode'=> $order->getCurrency(),
          'bSandbox'          => 0
        ),
        $db_to_auth => array(
          'cOrderReferenceId' => $this->_prepare_utf8($paymentInfos["mws_order_id"], false),
          'cAuthorizationId'  => $this->_prepare_utf8($paymentInfos["mws_auth_id"], false),
          'cAuthorizationStatus' => '',
          'fAuthorizationAmount' => $order->getAmountComplete(),
          'cAuthorizationCurrencyCode'  => $order->getCurrency(),
          'bSandbox'                    => 0
        )
      );
      */
    }
    $return_result['cSession'] = $paymentInfos['payment_id'];
    $return_result['paymentComment'] = 'Amazon-Referenz: '.$paymentInfos["mws_order_id"];
    return $return_result;
  }
  public function agi_pluginsPaymentMethod_KLARNA_INV($order, $return_result) {
    $paymentInfos = $order->getPaymentInfos();
    if ($return_result['bSetPayed'])
    {
      //$tmp = $return_result['tZahlungseingang'];
      $return_result['tZahlungseingang'] = array(
        'cZahlungsanbieter' => $return_result['cZahlart'],
        'fZahlungsgebuehr'  => 0,//$this->_prepare_utf8($paymentInfos["mc_fee"]),
        'fBetrag'           => $order->getAmountComplete(),
        'cISO'              => $order->getCurrency(),
        //'cEmpfaenger'       => $this->_prepare_utf8($paymentInfos["payer_email"]),
        //'cZahler'           => $this->_prepare_utf8($paymentInfos["receiver_email"]),
        'cHinweis'          => $this->_prepare_utf8('Klarna-RNr.: '.$paymentInfos["activation_invoice_number"], false),
        'cAbgeholt'         => 'N',
        'dZeit'             => date('Y-m-d H:i:s')
      );
    }
    $return_result['paymentComment'] = $this->_prepare_utf8('Klarna-RNr.: '.$paymentInfos["activation_invoice_number"], false);
    return $return_result;
  }
  public function agi_pluginsPaymentMethod_KLARNA_PAY($order, $return_result) {
    $paymentInfos = $order->getPaymentInfos();
    if ($return_result['bSetPayed'])
    {
      //$tmp = $return_result['tZahlungseingang'];
      $return_result['tZahlungseingang'] = array(
        'cZahlungsanbieter' => $return_result['cZahlart'],
        'fZahlungsgebuehr'  => 0,//$this->_prepare_utf8($paymentInfos["mc_fee"]),
        'fBetrag'           => $order->getAmountComplete(),
        'cISO'              => $order->getCurrency(),
        //'cEmpfaenger'       => $this->_prepare_utf8($paymentInfos["payer_email"]),
        //'cZahler'           => $this->_prepare_utf8($paymentInfos["receiver_email"]),
        'cHinweis'          => $paymentInfos["order_id"],
        'cAbgeholt'         => 'N',
        'dZeit'             => date('Y-m-d H:i:s')
      );
    }
    return $return_result;
  }
  public function agi_pluginsPaymentMethod_BARZAHLEN($order, $return_result) {
    $paymentInfos = $order->getPaymentInfos();
    if ($return_result['bSetPayed']) {
      $tmp = $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos["transaction_id"], false);
    }
    return $return_result;
  }
  
  
  public function _plugin_handle_orders_payment($order)
  {
    $lang_default = Shop::Lang()->getIsoFromLangID((int)$this->kSprache)->cISO;
    $jtlZahlarten = array();
    $aRes = $this->db->queryPrepared(
      "SELECT z.kZahlungsart, zs.cName, zs.cGebuehrname FROM `tzahlungsart` AS z, `tzahlungsartsprache` AS zs WHERE zs.kZahlungsart=z.kZahlungsart AND zs.cISOSprache = :cISOSprache ORDER BY z.kZahlungsart",
      ['cISOSprache' => $lang_default],
      ReturnType::ARRAY_OF_ASSOC_ARRAYS
    );
    foreach ($aRes as $row) {
      $jtlZahlarten[$row['kZahlungsart']] = array('name'=>$row['cName'],'gebuehrName'=>$row['cGebuehrname']);
    }
    $jtlZahlarten[5360] = array('name'=>'Amazon Payments','gebuehrName'=>'Amazon Payments');
    
    $shopgate_payments = array();
    if (!empty($this->JTLPluginConf['cZahlungsartenZuordnen'])) {
      $shopgate_payments = $this->JTLPluginConf['cZahlungsartenZuordnen'];
    }
    
    $paymentInfos   = $order->getPaymentInfos();
    $check_payment  = $order->getPaymentMethod();
    
    if (!array_key_exists($check_payment, $shopgate_payments) && $check_payment != 'SHOPGATE') {
      $check_payment = 'OTHER'; // Fallback-Wert
    }
    if (!array_key_exists('OTHER',$shopgate_payments)) {
      $shopgate_payments['OTHER'] = array('id'=>1,'name'=>'OTHER','isPayed'=>0,'useName'=>'Shopgate Import konfigurieren');
    }
    
    // Shopgate-Zahlungsart
    if ($check_payment == 'SHOPGATE' && array_key_exists('shopgate_payment_name', $paymentInfos)) {
      if (!empty($this->JTLPluginConf['cShopgateZahlartenZuordnen'])) {
        $checkName = $this->_prepare_utf8($paymentInfos['shopgate_payment_name'], false);
        $check_payment = 'OTHER';
        $shopgate_payments['OTHER']['useName'] = $checkName;
        if (strpos($this->JTLPluginConf['cShopgateZahlartenZuordnen'],chr(10).chr(13)) !== false) {
          $aRows = explode(chr(10).chr(13), $this->JTLPluginConf['cShopgateZahlartenZuordnen']);
        }
        elseif (strpos($this->JTLPluginConf['cShopgateZahlartenZuordnen'],chr(10)) !== false) {
          $aRows = explode(chr(10), $this->JTLPluginConf['cShopgateZahlartenZuordnen']);
        }
        elseif (strpos($this->JTLPluginConf['cShopgateZahlartenZuordnen'],chr(13)) !== false) {
          $aRows = explode(chr(13), $this->JTLPluginConf['cShopgateZahlartenZuordnen']);
        }
        else {
          $aRows = array($this->JTLPluginConf['cShopgateZahlartenZuordnen']);
        }
        foreach ($aRows as $relationRow) {
          $relationString = trim($relationRow);
          if (!empty($relationRow)) {
            $aRelation = explode('::',$relationRow);
            if (count($aRelation) > 1 && !empty($aRelation[0]) && !empty($aRelation[1])) {
              if ($checkName == $aRelation[0]) {
                $shopgate_payments['OTHER']['useName'] = $aRelation[1];
                if (isset($aRelation[2]) && $aRelation[2] == 1) {
                  $shopgate_payments['OTHER']['isPayed'] = 1;
                }
                // try to find payment-method
                foreach ($jtlZahlarten as $key => $value) {
                  if (strtolower($value['name']) == $aRelation[1]) {
                    $shopgate_payments['OTHER']['id'] = $key;
                    break;
                  }
                }
              }
            }
          }
        }
      }
      else {
        $check_payment = 'OTHER';
      }
    }
    
    $kZahlart = $shopgate_payments[$check_payment]['id'];
    if (empty($kZahlart)) {
      $check_payment = 'OTHER'; // Fallback
      $kZahlart = $shopgate_payments[$check_payment]['id'];
    }
    // Zuweisen der Sprachwerte
    $cZahlart = $jtlZahlarten[$kZahlart]['name'];
    if (!empty($shopgate_payments[$check_payment]['useName'])) {
      $cZahlart = $shopgate_payments[$check_payment]['useName'];
    }
    $cGebuehr = $jtlZahlarten[$kZahlart]['gebuehrName'];
    $bSetPayed = $shopgate_payments[$check_payment]['isPayed'] ? 1 : $order->getIsPaid();
    
    $return_result = array(
      'kZahlart'  => $kZahlart,
      'cZahlart'  => $cZahlart,
      'cGebuehr'  => $cGebuehr,
      'bSetPayed' => $bSetPayed,
      'paymentComment'=>'',
    );
    
    if ($bSetPayed) {
      $return_result['tZahlungseingang'] = array(
        'cZahlungsanbieter' => $cZahlart,
        'fZahlungsgebuehr'  => 0,
        'fBetrag'           => $order->getAmountComplete(),
        'cISO'              => $order->getCurrency(),
        'cHinweis'          => '',
        'cAbgeholt'         => 'N',
        'dZeit'             => $order->getIsPaid() ? $order->getPaymentTime('Y-m-d H:i:s') : date('Y-m-d H:i:s'),
      );
      if (array_key_exists('transaction_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['transaction_id'], false);
      elseif (array_key_exists('mws_order_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['mws_order_id'], false);
      elseif (array_key_exists('chase_tx_ref_num', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['chase_tx_ref_num'], false);
      elseif (array_key_exists('payment_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['payment_id'], false);
      elseif (array_key_exists('upp_transaction_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['upp_transaction_id'], false);
      elseif (array_key_exists('merchantreference', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['merchantreference'], false);
      elseif (array_key_exists('transaction_tag', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['transaction_tag'], false);
      elseif (array_key_exists('ident_uid', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['ident_uid'], false);
      elseif (array_key_exists('pay_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['pay_id'], false);
      elseif (array_key_exists('paypal_transaction_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['paypal_transaction_id'], false);
      elseif (array_key_exists('orderid', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['orderid'], false);
      elseif (array_key_exists('VPSTxId', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['VPSTxId'], false);
      elseif (array_key_exists('approval_code', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['approval_code'], false);
      elseif (array_key_exists('response_switch_key', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['response_switch_key'], false);
      elseif (array_key_exists('reference_number', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['reference_number'], false);
      elseif (array_key_exists('transaction_id_six', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['transaction_id_six'], false);
      elseif (array_key_exists('mb_transaction_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['mb_transaction_id'], false);
      elseif (array_key_exists('pm_order_transaction_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['pm_order_transaction_id'], false);
      elseif (array_key_exists('billsafe_transaction_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['billsafe_transaction_id'], false);
      elseif (array_key_exists('invoice_number', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['invoice_number'], false);
      elseif (array_key_exists('upaid_transaction_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['upaid_transaction_id'], false);
      elseif (array_key_exists('response_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['response_id'], false);
      elseif (array_key_exists('purpose', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['purpose'], false);
      elseif (array_key_exists('id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['id'], false);
      elseif (array_key_exists('order_id', $paymentInfos))
        $return_result['tZahlungseingang']['cHinweis'] = $this->_prepare_utf8($paymentInfos['order_id'], false);
    }
    
    if (method_exists($this, 'agi_ownPaymentMethod_'.$check_payment)) {
      $merge_result = call_user_func(array($this,'agi_ownPaymentMethod_'.$check_payment), $order, $return_result);
      if (!empty($merge_result)) {
        $return_result = $merge_result;
      }
    }
    elseif (method_exists($this, 'agi_pluginsPaymentMethod_'.$check_payment)) {
      $merge_result = call_user_func(array($this,'agi_pluginsPaymentMethod_'.$check_payment), $order, $return_result);
      if (!empty($merge_result)) {
        $return_result = $merge_result;
      }
    }
    return $return_result;
  }
}
